unit UFlexCelImport;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, UExcelAdapter,
  {$IFDEF ConditionalExpressions}{$if CompilerVersion >= 14} variants,{$IFEND}{$ENDIF} //Delphi 6 or above
  UFlxMessages;

type
  TFlexCelImport = class(TComponent)
  private
    Workbook: TExcelFile;
    FAdapter: TExcelAdapter;
    procedure SetAdapter(const Value: TExcelAdapter);
    function GetCellValue(Row, Col: integer): variant;
    procedure SetCellValue(Row, Col: integer; const Value: variant);
    function GetActiveSheet: byte;
    procedure SetActiveSheet(const Value: byte);
    function GetSheetCount: byte;
    function GetMaxRow: integer;
    function GetComments(Row, Col: integer): variant;
    procedure SetComments(Row, Col: integer; const Value: variant);
    function GetCommentsCount(Row: integer): integer;
    function GetActiveSheetName: widestring;
    procedure SetActiveSheetName(const Value: widestring);
    { Private declarations }
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    { Protected declarations }
  public
    { Public declarations }
    destructor Destroy;override;

    procedure OpenFile(const FileName: TFileName);
    procedure Save(const FileName: TFileName);
    procedure CloseFile;

    property SheetCount: byte read GetSheetCount;
    property ActiveSheet:byte read GetActiveSheet write SetActiveSheet ;
    property ActiveSheetName: widestring read GetActiveSheetName write SetActiveSheetName;

    property MaxRow: integer read GetMaxRow;
    function IsEmptyRow(const aRow: integer): boolean;
    property CellValue[Row, Col: integer]: variant read GetCellValue write SetCellValue;

    procedure InsertAndCopyRows(const FirstRow, LastRow, DestRow, aCount: integer);
    procedure DeleteRows(const aRow, aCount: word);

    property CommentsCount[Row: integer]: integer read GetCommentsCount;
    property Comments[Row, Col: integer]: variant read GetComments write SetComments;

    function IsLoaded: boolean; 
  published
    property Adapter: TExcelAdapter read FAdapter write SetAdapter;
    { Published declarations }
  end;

procedure Register;

implementation
{$R IFlexCelImport.res}
procedure Register;
begin
  RegisterComponents('FlexCel', [TFlexCelImport]);
end;

{ TFlexCelImport }

procedure TFlexCelImport.CloseFile;
begin
  if Workbook=nil then exit;
  try
    Workbook.CloseFile;
  except
    //nothing;
  end; //Except
  try
    Workbook.Disconnect;
  except
    //nothing;
  end; //Except
  FreeAndNil(Workbook);
end;

procedure TFlexCelImport.DeleteRows(const aRow, aCount: word);
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Workbook.DeleteRows(aRow, aCount);
end;

destructor TFlexCelImport.Destroy;
begin
  CloseFile;
  inherited;
end;

function TFlexCelImport.GetActiveSheet: byte;
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Result:=Workbook.ActiveSheet;
end;

function TFlexCelImport.GetCellValue(Row, Col: integer): variant;
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Result:= Workbook.CellValue[Row, Col];
end;

function TFlexCelImport.GetComments(Row, Col: integer): variant;
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Result:= Workbook.CommentText[Row, Col];
end;

function TFlexCelImport.GetCommentsCount(Row: integer): integer;
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Result:=Workbook.CommentsCount[Row];
end;

function TFlexCelImport.GetMaxRow: integer;
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Result:=Workbook.MaxRow;
end;

function TFlexCelImport.GetSheetCount: byte;
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Result:=Workbook.SheetCount;
end;

function TFlexCelImport.GetActiveSheetName: widestring;
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Result:= Workbook.ActiveSheetName;
end;

procedure TFlexCelImport.InsertAndCopyRows(const FirstRow, LastRow,
  DestRow, aCount: integer);
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Workbook.InsertAndCopyRows(FirstRow, LastRow, DestRow, aCount, False);
end;

function TFlexCelImport.IsLoaded: boolean;
begin
  Result:= Workbook<>nil;
end;

procedure TFlexCelImport.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FAdapter then
        FAdapter:= nil;
  end;
end;

procedure TFlexCelImport.OpenFile(const FileName: TFileName);
begin
  if FAdapter=nil then raise Exception.Create(ErrNoAdapter);
  CloseFile;
  Workbook:=FAdapter.GetWorkbook;
  Workbook.Connect;
  Workbook.OpenFile(FileName);
  Workbook.SelectSheet(Workbook.ActiveSheet);
end;

procedure TFlexCelImport.Save(const FileName: TFileName);
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Workbook.Save(true, FileName, nil);
end;

procedure TFlexCelImport.SetActiveSheet(const Value: byte);
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Workbook.ActiveSheet:=Value;
  Workbook.SelectSheet(Value);
end;

procedure TFlexCelImport.SetAdapter(const Value: TExcelAdapter);
begin
  FAdapter := Value;
end;

procedure TFlexCelImport.SetCellValue(Row, Col: integer;
  const Value: variant);
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Workbook.CellValue[Row, Col]:= Value;
end;

procedure TFlexCelImport.SetComments(Row, Col: integer;
  const Value: variant);
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Workbook.AssignComment(Row, Col, Value);
end;

procedure TFlexCelImport.SetActiveSheetName(const Value: widestring);
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Workbook.ActiveSheetName:=Value;

end;

function TFlexCelImport.IsEmptyRow(const aRow: integer): boolean;
begin
  if Workbook=nil then raise Exception.Create(ErrNoOpenFile);
  Result:=Workbook.IsEmptyRow(aRow);
end;

end.
