// HREFParser Component  1.0                                                  //
////////////////////////////////////////////////////////////////////////////////
// Part of Internet EDiting component                                         //
// AUTHOR       : Sergey Kanenko                                              //
// WEBSITE      : http://iedcomp.nm.ru                                        //
////////////////////////////////////////////////////////////////////////////////
//   This code may be used and modified by anyone so long as  this header and //
// copyright  information remains intact.                                     //
//   The code is provided "as-is" and without warranty of any kind,           //
// expressed, implied or otherwise, including and without limitation, any     //
// warranty of merchantability or fitness for a  particular purpose.         //
//   In no event shall the author be liable for any special, incidental,      //
// indirect or consequential damages whatsoever (including, without           //
// limitation, damages for loss of profits, business interruption, loss       //
// of information, or any other loss), whether or not advised of the          //
// possibility of damage, and on any theory of liability, arising out of      //
// or in connection with the use or inability to use this software.         //
////////////////////////////////////////////////////////////////////////////////
unit HREFParser;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

var
  ExtStrHTML    : string = '.HTM.HTML.HTA.SHTML.DHTML.PHTML.';
  ExtStrStyle   : string = '.CSS.';
  ExtStrScript  : string = '.JS.VBS.';
  ExtStrImage   : string = '.JPG.JPEG.GIF.TIF.BMP.';
  ExtStrMedia   : string = '.AVI.MP3.MPEG.WAV.RAM.WMV.';
  ExtStrJava    : string = '.JAVA.CLASS.JSP.';
  ExtStrActiveX : string = '.CAB.DLL.OCX.ZIP.';
  ExtStrASP     : string = '.ASP.ASPX.';
  ExtStrPHP     : string = '.PHP.PHP3.PHP4.INC.';
  ExtStrPERL    : string = '.PL.CGI.';
               //           1    6     12   17   22  26     33     40     47  51   56
  ExtStrProto   : string = ':HTTP:HTTPS:NEWS:NNTP:FTP:GOPHER:TELNET:MAILTO:RES:FILE:JAVASCRIPT:';
  KeywordsPHP   : array[0..2] of string = ('include','require_once','require');
  KeywordsCSS   : array[0..1] of string = (' url',':url');

type
  TOnFoundHREF   = procedure(Sender: TObject; Hyperlink:string) of object;
  TOnFoundScript = procedure(Sender: TObject; TagName, Tag:string) of object;
  TTagType       = (ttSingle,ttJScript,ttPScript,ttAScript,ttClosed, ttOthers);
  TExtentions    = (feUnknown,feRoot,feHTML,feIMG,feMEDIA,feJava,feAX,feCSS,feScript,feASP,fePHP,fePERL);
  TExtentionSet  = set of TExtentions;
  TLinkState     = (lsUnchecked,lsBroken,lsGood);
  TCharSet       = set of Char;
  TParseProtocol = (kpUnused, kpHTTP, kpFTP, kpLocal, kpParent);

  THREF  = class
  private
    FUserName,
    FUserDir,
    FUserPwd,
    FDocument,
    FExt,
    FUExt,
    FArguments,
    FProtocol,
    FURL,
    FPort,
    FDefaultPort,
    FPath,
    FHost,
    FBookmark : string;
    FState: TLinkState;
    FData: Pointer;
    FXLevel: integer;
    FUseBookmark : boolean;
    function  CheckExt(Const ExtList:string):boolean;
    function  GetHREF: string;
    procedure SetHREF(const Value: string);
    function  GetDocumentURL: string;
    function  GetUniqueDocURL: string;
    function  GetProtocolType: TParseProtocol;
    procedure SetPath(const Value: string);
  protected
  public
    constructor Create(const A: string = ''); virtual;
    procedure   SetBaseURL(AProt, AHost, AUserDir, APath: string);
    procedure   AssignBaseURL(AHREF: THREF);
    procedure   Assign(AHREF: THREF);
    function    GetFileType: TExtentions;

    property HREF: string read GetHREF write SetHREF;
    property DocumentURL: string read GetDocumentURL;
    property UniqueDocURL: string read GetUniqueDocURL;

    property Protocol: string read FProtocol write FProtocol;
    property ProtocolType: TParseProtocol read GetProtocolType;
    property UserName: string read FUserName write FUserName;
    property UserPwd: string read FUserPwd write FUserPwd;
    property UserDir: string read FUserDir write FUserDir;
    property Host: string read FHost write FHost;
    property Path: string read FPath write SetPath;
    property Document: string read FDocument write FDocument;
    property Ext: string read FExt;
    property Arguments: string read FArguments write FArguments ;
    property Port: string read FPort write FPort;
    property Bookmark: string read FBookmark write FBookmark;

    property State : TLinkState read FState write FState default lsUnchecked;
    property Level : integer read FXLevel write FXLevel;
    property UseBookmark: boolean read FUseBookmark write FUseBookmark;
    property Data : Pointer read FData write FData;
  end;

  THREFList  = Class(TStringList)
  private
    function  GetHREFs(Index: Integer): THREF;
    function  GetURLs(Index: Integer): string;
    procedure SetHREFs(Index: Integer; const Value: THREF);
    procedure SetURLs(Index: Integer; const Value: string);
  protected
  public
   constructor Create;
   destructor Destroy; override;
   function   AddHREF(S: string): Integer;
   property   HREFs[Index: Integer]: THREF read GetHREFs write SetHREFs; default;
   property   URLs[Index: Integer]: string read GetURLs write SetURLs;
  end;

  THREFDocument = class(TPersistent)
  private
    FHREFList   : THREFList;
    FHREF       : THREF;
    FRaw        : TStrings;
    FTitle      : string;
    FKeyword    : string;
    FDescription: string;
    ClosedTag   : string;
    FOnFoundHREF: TOnFoundHREF;
    FOnFoundScript: TOnFoundScript;
  protected
    procedure AddTag(TagName,Tag:string);
    function  CheckTag(TagName: string): TTagType;
    procedure SetRaw(const Value: tstrings);
    procedure DoAddHyperlink(Hyperlink: string);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure Parse;
    procedure GetUniqueDocs(var UniqueDocs: TStringList; PossibleExt: TExtentionSet);
    procedure ParseAdditionalLinks(const Code: string; Keywords: array of string);

    property Title        : string Read FTitle Write FTitle;
    property Keyword      : string Read FKeyWord Write FKeyword;
    property Description  : string Read FDescription Write FDescription;
    property HREFSelf     : THREF  Read FHREF Write FHREF;
    property HREFList     : THREFList Read FHREFList Write FHREFList;
    property HTML         : TStrings  Read FRaw Write SetRaw;

    property OnFoundHyperlink : TOnFoundHREF Read FOnFoundHREF Write FOnFoundHREF;
    property OnFoundScript    : TOnFoundScript Read FOnFoundScript Write FOnFoundScript;
  end;

  THREFParser = class(TComponent)
  private
    FDoc     : THREFDocument;

    procedure SetRaw(Value: Tstrings);
    function  GetRaw: tstrings;
    function  GetOnFoundHREF: TOnFoundHREF;
    procedure SetOnFoundHREF(const Value: TOnFoundHREF);
    function  GetOnFoundScript: TOnFoundScript;
    procedure SetOnFoundScript(const Value: TOnFoundScript);
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   GetHREFStrings( AHREFList: TStrings );
    property    Document : THREFDocument read FDoc;
  published
    property  HTML : TStrings Read GetRaw Write SetRaw;
    property  OnFoundHyperlink  : TOnFoundHREF Read GetOnFoundHREF Write SetOnFoundHREF;
    property  OnFoundScript : TOnFoundScript Read GetOnFoundScript Write SetOnFoundScript;
  end;


function  NormalizePath(APath: string): string;
function  URLDecode(Src: string): string;
function  URLEncode(Src: string): string;
procedure ParseURL(URL: string;
                   var Protocol,
                       Host,
                       Document,
                       Extention,
                       Argument,
                       Path,
                       UserName,
                       UserPwd,
                       UserDir,
                       Port,
                       DefaultPort,
                       Bookmark: string);
procedure Register;

implementation

function RPos(Substr: char; S: string): Integer;
begin
 Result:=Length(S);
 while (Result>0)and(S[Result]<>Substr) do dec(Result);
end;

function LCut(var S: string; P : integer; Const Delta: integer=1): string;
begin
  result := '';
  if P<>0 then
   begin
    result := Copy(S, 1, P - 1);
    S:= Copy(S, P+Delta, Length(S));
   end;
end;

function RCut(var S: string; P : integer; Const Delta: integer=1): string;
begin
  result := '';
  if P<>0 then
   begin
    result := Copy(S, P+1,Length(S));
    S:= Copy(S, 1, P-Delta);
   end;
end;

function LCutToken(var S: string; Delim: char; var P : integer;
                   const CaseSensitive: Boolean = false) : string;
begin
  if CaseSensitive then
    P := Pos(Delim, S)
  else
    P := Pos(Delim, UpperCase(S));
  Result := LCut(S,P);
end;

function RCutToken(var S: string; Delim: char; var P : integer;
                   const CaseSensitive: Boolean = false) : string;
begin
  if CaseSensitive then
   P:=RPos(Delim,S)
  else
   P:=RPos(Delim,UpperCase(S));
  Result := RCut(S,P);
end;


function UnQuotedPos(Str: string; Start: integer; Delim: TCharSet): integer;
var Quote1,Quote2: boolean;
    i : integer;
begin
Quote1:=false; Quote2:=false; Result:=Length(Str);
 For i:=Start to Length(Str) do
   Case Str[i] of
    '''' : if not Quote2 then Quote1:=not Quote1;
    '"'  : if not Quote1 then Quote2:=not Quote2;
    else
     if (not Quote1)and(not Quote2)and(Str[i] in Delim) then
      begin
       Result:=i; exit;
      end
  end;
end;

function NormalizePath(APath: string): string;
var
  i: Integer;
begin
  i:= 1; result:='';  if APath='' then exit;
  APath:= trim(APath);
  while i <= Length(APath) do
  begin
    if APath[i] in LeadBytes then
     begin
      result:= result + copy(APath,i,2);
      inc(i,2);
     end
    else
    if APath[i] = '\' then
     begin
      result:= result + '/';
      inc(i, 1);
     end
    else
    if APath[i] = '''' then
     begin
      result:= result + '"';
      inc(i, 1);
     end
    else
    if Copy(APath,i,3)='/./' then
     begin
      result:= result + '/';
      inc(i, 3);
     end
    else
     begin
      result:= result + APath[i];
      inc(i, 1);
     end;
  end;
end;

function URLDecode(Src: string): string;
var
  i: integer;
  ESC: string[2];
  CharCode: integer;
begin
  Result := '';
  Src := StringReplace(Src, '+', ' ', [rfReplaceAll]);
  i := 1;
  while i <= Length(Src) do
   begin
    if Src[i] <> '%' then
      Result := Result + Src[i]
    else
     begin
      Inc(i);
      ESC := Copy(Src, i, 2);
      Inc(i, 1);
      try
        CharCode := StrToInt('$' + ESC);
        if (CharCode > 0) and (CharCode < 256) then
         Result := Result + Char(CharCode);
      except end;
     end;
    Inc(i);
   end;
end;

function URLEncode(Src: string): string;
const
  BadChar = ['*', '#', '%', '<', '>', '+'];
var
  i: Integer;
begin
  Result := '';
  for i := 1 to Length(Src) do
    if Src[i] = ' ' then
      Result := Result + '+'
    else
     begin
      if (Src[i] in BadChar) or (Src[i] >= #$80) then
        Result := Result + '%' + IntToHex(Ord(Src[i]), 2)
      else
        Result := Result + Src[i];
     end;
end;

Function ExtractQuotedStr(Str:string):string;
Var Index:integer;
    Started : boolean;
    Quote: char;
Begin
 Result:='';
 Quote:=' ';
 Started:=false;
 For Index:=1 to Length(Str) do
  If Str[Index] in ['"',''''] then
   begin
    if not Started then
     begin
      Started := true;
      Quote:= Str[Index];
     end
    else
     begin
      if Quote=Str[Index] then  break;
     end;
   end
  else
   begin
    if Started then Result:=Result+Str[Index];
   end;
end;

Function ExtractParametr(Str:string):string;
Var i,s,f:integer;
Begin
 result:='';
 s:= Pos('(', Str); if s=0 then s:=1;
 f:= Pos(')', Str); if f=0 then f:=Length(Str);
 str:=trim(copy(Str,s+1,f-s-1));
 For i:=1 to Length(Str) do
  If not (Str[i] in ['"','''']) then
   Result:=Result+Str[i];
end;


Function ExtractValue( Attr:string ): string;
Begin
 Result:=ExtractQuotedStr(Copy(Attr,Pos('=',Attr)+1,Length(Attr)));
End;

Function GetTagAttr( Tag, Attribute:string ):string;
Var AttrPos, SpacePos, Count : Integer;
     Quoted : Boolean;
begin
 SpacePos:=0;
 Quoted:=False;
 AttrPos:=Pos(Attribute,UpperCase(Tag));
 If AttrPos<>0 then
  Begin
   For Count:=AttrPos to Length(Tag) do
   Begin
    If Tag[Count] in ['"','''']  then
     Quoted:= not Quoted;
    If ((Tag[Count]=' ') and Not Quoted) then
     Begin
      SpacePos:=Count;
      Break;
     End
    else
    If (Tag[Count]='>') or (Count=Length(tag)) then
     Begin
      SpacePos:=Count;
      Break;
     End;
    End;
   Result:=Copy(Tag, AttrPos, SpacePos-AttrPos+1);
  End;
End;

procedure ParseURL(URL: string; var Protocol, Host, Document, Extention,
                                    Argument, Path, UserName, UserPwd, UserDir,
                                    Port, DefaultPort, Bookmark: string);
var
   TP,PP: Integer;
begin
  Host := '';
  Protocol := '';
  Path := '';
  Document := '';
  URL:=NormalizePath(URL);
  if URL='' then exit;

  TP := Pos(':', URL);
  if TP<>0 then               //Protocol may be present
   for PP:=1 to TP do
    if URL[PP] in ['/','.','#'] then //Protocol not present
     begin
      TP:=0;
      break;
     end;

   if TP<>0 then
    begin // Have protocol - absolute path
     if TP>2 then
      begin
       Protocol:= LCut(URL,TP)+':';
       case Pos(':'+UpperCase(Protocol),ExtStrProto) of
         1    : DefaultPort:= '80';
         6    : DefaultPort:= '443';
        12,17 : DefaultPort:= '119';
        22    : DefaultPort:= '21';
        26    : DefaultPort:= '70';
        33    : DefaultPort:= '23';
       end;
      end;

    if copy(URL,1,2)='//' then
     begin
      Protocol:=Protocol +'//';
      URL:= copy(URL,3,Length(URL));
      TP := Pos('/', URL);

      if TP<>0 then
       begin
        if TP>3 then
          Host:= LCut(URL,TP,0);
       end
      else
       begin
        Host:= URL;
        URL := '';
       end;
      UserPwd := LCutToken(Host,'@',PP);
      UserName := LCutToken(UserPwd, ':', PP);
      if UserName='' then UserPwd:='';
      Port:= RCutToken(Host,':',PP);
     end;
    end
   else
    begin
     //Havn't protocol and host here
    end;

    Bookmark := RCutToken(URL,'#',PP);
    Argument := RCutToken(URL,'?',PP);
  
     TP := RPos('/', URL);
     if TP<>0 then
      begin
       Document:= RCut(URL,TP,0);
       Path:=URL;
      end
     else
       Document:=URL;

    Document:=URLDecode(Document);
    Path:=URLDecode(Path);

    if copy(Path,1,2)='/~' then
     begin
       Path:= copy(Path,3,Length(Path));
       PP:= Pos('/',Path);
       UserDir := '/~'+LCut(Path,PP,0);
     end;

   Extention := ExtractFileExt(Document);
end;

//==============================================================================
constructor THREF.Create(const A: string = '');
begin
  inherited Create;
  FBookmark:='';
  FXLevel:=0;
  FUseBookmark:=false;
  FState := lsUnchecked;
  if length(A) > 0 then HREF := A;
end;

function THREF.GetHREF: string;
begin
  Result := GetDocumentURL;
  if Length(FArguments) > 0 then
   Result:= Result +'?'+FArguments;
  if (FUseBookmark)and(Length(FBookmark)>0) then
   Result := Result + '#' + FBookmark;
end;

function THREF.GetDocumentURL: string;
begin
  Result:= FProtocol;
  if Length(FUserName) > 0 then
   begin
    Result := Result + FUserName;
    if Length(FUserPwd) > 0 then Result := Result + ':' + FUserPwd;
    Result := Result + '@';
   end;
  Result := Result + FHost;
  if Length(FPort) > 0 then Result:= Result + ':' + FPort;
  Result := Result + {URLEncode(}FUserDir + FPath + FDocument{)};
end;

function THREF.GetUniqueDocURL: string;
begin
 if (ProtocolType=kpLocal) then
  result:=GetDocumentURL
 else
  result:=GetHREF;
end;

procedure THREF.SetHREF(const Value: string);
begin
  FURL := NormalizePath(Trim(Value));
  ParseURL(FURL, FProtocol, FHost, FDocument, FExt, FArguments,
           FPath,FUserName,FUserPwd,FUserDir,
           FPort,FDefaultPort, FBookmark);
  FUExt:=UpperCase(FExt)+'.';
end;

function THREF.CheckExt(const ExtList: string): boolean;
begin
 result:= (FUExt<>'.')and(Pos(FUExt,ExtList)<>0);
end;

function  THREF.GetFileType: TExtentions;
begin
 if (FHost<>'')and(FDocument='') then result:=feRoot
 else if CheckExt(ExtStrHTML) then result:=feHTML
 else if CheckExt(ExtStrScript) then result:=feScript
 else if CheckExt(ExtStrStyle) then result:=feCSS
 else if CheckExt(ExtStrImage) then result:=feIMG
 else if CheckExt(ExtStrPHP) then result:=fePHP
 else if CheckExt(ExtStrASP) then result:=feASP
 else if CheckExt(ExtStrPERL) then result:=fePERL
 else if CheckExt(ExtStrJava) then result:=feJava
 else if CheckExt(ExtStrActiveX) then result:=feAX
 else result:=feUnknown;
end;

procedure THREF.SetBaseURL(AProt, AHost, AUserDir, APath: string);
var pp : TParseProtocol;
begin
 pp := GetProtocolType;
 if (pp<>kpUnused)and(FHost='') then
  begin
   if (FPath='') then
    Path:= APath
   else if (not(FPath[1] in ['/','\'])) then
    Path:= APath+FPath;
   Host:= AHost;
   UserDir:= AUserDir;
   Protocol:= AProt;
   pp:= GetProtocolType;
   if (pp in [kpHTTP,kpFTP])and((FPath='')or(FPath[1]<>'/')) then
     FPath:='/'+FPath;
  end;
end;

procedure THREF.AssignBaseURL(AHREF: THREF);
begin
 SetBaseURL( AHREF.Protocol, AHREF.Host, AHREF.UserDir, AHREF.Path );
end;

procedure THREF.Assign(AHREF: THREF);
begin
 HREF := AHREF.HREF;
 FState:= AHREF.State;
 FData:= AHREF.Data;
 FXLevel:= AHREF.Level;
end;

function THREF.GetProtocolType: TParseProtocol;
var uProtocol : string;
begin
 uProtocol := UpperCase(Protocol);
 if (Pos('FILE',uProtocol)<>0)or((Length(uProtocol)>1)and(uProtocol[2]=':')) then result:= kpLocal
 else
 if uProtocol='' then result:= kpParent
 else
 if (Pos('HTTP',uProtocol)<>0)or(Pos('HTTPS',uProtocol)<>0) then result:= kpHTTP
 else
 if (Pos('FTP',uProtocol)<>0) then result:= kpFTP
 else
 result:= kpUnused;
end;

{ THREFList }
constructor THREFList.Create;
begin
 inherited;
 Sorted:=true;
 Duplicates:= dupIgnore;
end;

destructor THREFList.Destroy;
begin
 inherited;
end;

function THREFList.AddHREF(S: string): Integer;
begin
 result:=-1; s:= trim(s); if s='' then exit;
 result := AddObject(s,THREF.Create(s));
end;

procedure THREF.SetPath(const Value: string);
begin
 FPath:= NormalizePath(Value);
end;

function THREFList.GetHREFs(Index: Integer): THREF;
begin
 result:= Objects[Index] as THREF;
end;

function THREFList.GetURLs(Index: Integer): string;
begin
 result:= strings[Index];
end;

procedure THREFList.SetHREFs(Index: Integer; const Value: THREF);
var
  TempObject: THREF;
begin
  TempObject := GetHREFs(Index);
  if Assigned(TempObject) then TempObject.Free;
  Objects[Index]:= Value;
end;

procedure THREFList.SetURLs(Index: Integer; const Value: string);
begin
 try
  strings[Index]:= Value;
 except
  exit;
 end;
  HREFS[Index].HREF:= Value;
end;

{ THREFDocument }
constructor THREFDocument.Create;
begin
 inherited;
 FHREFList := THREFList.Create;
 FRaw      := TStringList.Create;
 FHREF     := THREF.Create;
end;

destructor THREFDocument.Destroy;
begin
  if Assigned(FHREFList) then FHREFList.Free;
  if Assigned(FRaw) then FRaw.Free;
  if Assigned(FHREF) then FHREF.Free;
  inherited;
end;

procedure THREFDocument.SetRaw(const Value: tstrings);
begin
  FRaw.Assign(Value);
end;

function THREFDocument.CheckTag(TagName: string):TTagType;
begin
 result:= ttOthers;
 If TagName='A' then
  result:=ttSingle
 else
 If TagName='IMG' then
  result:=ttSingle
 else
 If TagName='FRAME' then
  result:=ttSingle
 else
 If TagName='LINK' then
  result:=ttSingle
 else
 If TagName='IFRAME' then
  result:=ttSingle
 else
 If TagName='BGSOUND' then
  result:=ttSingle
 else
 If TagName='BODY' then
  result:=ttSingle
 else
 If TagName='STYLE' then
  Begin
   result:=ttClosed;
   ClosedTag := '</STYLE>';
  End
 else
 If TagName='META' then
  result:=ttSingle
 else
 If TagName='TITLE' then
  Begin
   result:=ttClosed;
   ClosedTag := '</TITLE>';
  End
 else
 If TagName='SCRIPT' then
  Begin
   result:=ttJScript;
   ClosedTag := '</SCRIPT>';
  End
 else
 If (TagName='?')or(TagName='?PHP') then
  Begin
   result:=ttPScript;
   ClosedTag := '?>';
  End
 else
 If pos('%',TagName)<>0 then
  Begin
   result:=ttAScript;
   ClosedTag := '%>';
  End
end;

procedure THREFDocument.AddTag(TagName,Tag: string);
var
 Hyperlink,
 MetaName : string;
Begin
 Hyperlink:='';
 //Others
 If TagName='A' then
   Hyperlink:=ExtractValue(GetTagAttr(Tag,'HREF'))
 else
 If TagName='LINK' then
   Hyperlink:=ExtractValue(GetTagAttr(Tag,'HREF'))
 else
 If TagName='IMG' then
   Hyperlink:=ExtractValue(GetTagAttr(Tag,'SRC'))
 else
 If TagName='FRAME' then
   Hyperlink:=ExtractValue(GetTagAttr(Tag,'SRC'))
 else
 If TagName='IFRAME' then
   Hyperlink:=ExtractValue(GetTagAttr(Tag,'SRC'))
 else
 If TagName='BGSOUND' then
   Hyperlink:=ExtractValue(GetTagAttr(Tag,'SRC'))
 else
 If TagName='BODY' then
   Hyperlink:=ExtractValue(GetTagAttr(Tag,'BACKGROUND'))
 else
 If TagName='STYLE' then
   ParseAdditionalLinks(Tag,KeywordsCSS)
 else
 If TagName='META' then
  Begin
   MetaName:=UpperCase(ExtractValue(GetTagAttr(Tag,'NAME')));
   if MetaName='KEYWORD' then
    FKeyword:=ExtractValue(GetTagAttr(Tag,'CONTENT'))
   else
   if MetaName='DESCRIPTION' then
    FDescription:=ExtractValue(GetTagAttr(Tag,'CONTENT'));
  End
 else
 If TagName='TITLE' then
   FTitle:=Tag
 else
 If TagName='SCRIPT' then
  Begin
   Hyperlink:=ExtractValue(GetTagAttr(Tag,'SRC'));
   If (Hyperlink='') and Assigned(FOnFoundScript) then
     FOnFoundScript(Self,'SCRIPT',Tag);
  End
 else
 If (TagName='?')or(TagName='?PHP') then
  Begin
    ParseAdditionalLinks(Tag,KeywordsPHP);
    If Assigned(FOnFoundScript) then
       FOnFoundScript(Self,TagName,Tag);
  End
 else
 If pos('%',TagName)<>0 then
  Begin
     If Assigned(FOnFoundScript) then
       FOnFoundScript(Self,TagName,Tag);
  End;

 DoAddHyperlink(Hyperlink);
End;

procedure THREFDocument.DoAddHyperlink(Hyperlink: string);
begin
 if ((Hyperlink<>'')and(Hyperlink<>'#')) then
  begin
   FHREFList.AddHREF(Hyperlink);
   If Assigned(FOnFoundHREF) then
    FOnFoundHREF(Self,Hyperlink);
  end;
end;

procedure THREFDocument.Parse;
var TagIdx,EndTagIdx,i : integer;
    TagName,Str,UStr : string;
    StartP,EndP : PChar;
    FT : TExtentions;
Begin
  FT := FHREF.GetFileType;
  if FT in [feIMG,feMEDIA,feJava,feAX] then exit;  //doesn't parsed
  Str := FRaw.Text;
  if FT = feCSS then
   begin
    ParseAdditionalLinks(Str,KeywordsCSS);
    exit;
   end;
  UStr:=UpperCase(Str);
  i:=1;
  while i<=Length(Str) do
   begin
    if (Str[i]='<') then
     begin
      TagIdx:=UnQuotedPos(Str,i+1,[' ','<','>',#13]);
      TagName:=UpperCase(Trim(Copy(Str,i+1,TagIdx-i-1)));
      case CheckTag(TagName) of
       ttSingle:
                 begin
                  EndTagIdx:=UnQuotedPos(Str,i+1,['>']);
                  AddTag(TagName,Copy(Str,TagIdx+1,EndTagIdx-TagIdx-2));
                  i:= EndTagIdx;
                 end;
       ttClosed,ttJScript:
                 begin
                  StartP:=PChar(@UStr[TagIdx+1]);
                  EndP:=StrPos(StartP,PChar(ClosedTag));
                  AddTag(TagName,Copy(Str,TagIdx+1,EndP-StartP));
                  i:= i+EndP-StartP+Length(ClosedTag);
                 end;
       ttPScript,ttAScript:
                 begin
                  StartP:=PChar(@UStr[i+1]);
                  EndP:=StrPos(StartP,PChar(ClosedTag));
                  AddTag(TagName,Copy(Str,i+1,EndP-StartP+Length(ClosedTag)));
                  i:= i+EndP-StartP+Length(ClosedTag);
                 end;
      end;
     end;
    inc(i);
   end;
End;

procedure THREFDocument.GetUniqueDocs;
var i  : integer;
begin
  if not Assigned(UniqueDocs) then exit;
  UniqueDocs.Clear;
  if not UniqueDocs.Sorted then
   begin
    UniqueDocs.Sorted:= true;
    UniqueDocs.Duplicates:= dupIgnore;
   end;
  for i:=0 to FHREFList.Count-1 do
   if FHREFList[i].GetFileType in PossibleExt then
     With FHREFList[i] do
      UniqueDocs.AddObject(FHREFList[i].UniqueDocURL, FHREFList[i]);
end;

procedure THREFDocument.ParseAdditionalLinks(Const Code: string; Keywords : array of string );
var i : integer;
    str : PChar;
begin
  for i:=Low(Keywords) to High(Keywords) do
   begin
    str:=PChar(Code);
    While str<>nil do
     begin
      str:=StrPos(str,PChar(Keywords[i]));
      if str<>nil then
       begin
        inc(Str,Length(Keywords[i]));
        if (Str[0]='(')or(Str[1]='(') then
         begin
          DoAddHyperlink(ExtractParametr(Str));
          str:=StrPos(str,')');
         end;
       end;
     end;
   end;
end;

procedure THREFDocument.Assign(Source: TPersistent);
begin
  if Source is THREFDocument then
   begin
    FHREFList.Assign(THREFDocument(Source).FHREFList);
    FHREF.Assign(THREFDocument(Source).FHREF);
    FRaw.Assign(THREFDocument(Source).FRaw);
    FTitle      := THREFDocument(Source).FTitle;
    FKeyword    := THREFDocument(Source).FKeyword;
    FDescription:= THREFDocument(Source).FDescription;
   end; 
 end;

{THREFParser}
constructor THREFParser.Create(AOwner: TComponent);
Begin
 inherited;
 FDoc := THREFDocument.Create;
End;

destructor THREFParser.Destroy;
Begin
 If Assigned(FDoc) then FDoc.Free;
 inherited Destroy;
End;

function THREFParser.GetOnFoundHREF: TOnFoundHREF;
begin
 result:= FDoc.OnFoundHyperlink;
end;

procedure THREFParser.SetOnFoundHREF(const Value: TOnFoundHREF);
begin
 FDoc.OnFoundHyperlink:=Value;
end;

function THREFParser.GetOnFoundScript: TOnFoundScript;
begin
 result:= FDoc.OnFoundScript;
end;

procedure THREFParser.SetOnFoundScript(const Value: TOnFoundScript);
begin
 FDoc.OnFoundScript:=Value;
end;

procedure THREFParser.SetRaw(Value: tstrings);
begin
 FDoc.HTML := Value;
 FDoc.Parse;
end;

procedure THREFParser.GetHREFStrings( AHREFList: TStrings);
var i : integer;
begin
 AHREFList.Clear;
 for i:=0 to FDoc.HREFList.Count-1 do
   AHREFList.Add(FDoc.HREFList[i].HREF)
end;

function THREFParser.GetRaw: TStrings;
begin
 result:= FDoc.HTML;
end;

procedure Register;
begin
 RegisterComponents('IED', [THREFParser]);
end;

end.
