////////////////////////////////////////////////////////////////////////////////
// Part of Internet EDiting component                                         //
// AUTHOR       : Sergey Kanenko                                              //
// WEBSITE      : http://iedcomp.nm.ru                                        //
////////////////////////////////////////////////////////////////////////////////
//   This code may be used and modified by anyone so long as  this header and //
// copyright  information remains intact.                                     //
//   The code is provided "as-is" and without warranty of any kind,           //
// expressed, implied or otherwise, including and without limitation, any     //
// warranty of merchantability or fitness for a  particular purpose.         //
//   In no event shall the author be liable for any special, incidental,      //
// indirect or consequential damages whatsoever (including, without           //
// limitation, damages for loss of profits, business interruption, loss       //
// of information, or any other loss), whether or not advised of the          //
// possibility of damage, and on any theory of liability, arising out of      //
// or in connection with the use or inability to use this software.         //
////////////////////////////////////////////////////////////////////////////////
unit UnMainDiagram;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Buttons, ComCtrls, ToolWin, ImgList,
  IdFTP, IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP,
  IEShapes, HREFParser, HREFLinker, UnParserShapes;
type
  TfrmMain = class(TForm)
    ControlBar1: TControlBar;
    ToolBar1: TToolBar;
    Diagram: TScrollBox;
    TBImages: TImageList;
    ToolButton4: TToolButton;
    OpenDialog1: TOpenDialog;
    ToolButton1: TToolButton;
    DocImages: TImageList;
    LNK: THREFLinker;
    HTTPgate: TIdHTTP;
    FTPgate: TIdFTP;
    ToolButton5: TToolButton;
    StatusBar1: TStatusBar;
    edURL: TEdit;
    ToolButton6: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton7: TToolButton;
    procedure ToolButton1Click(Sender: TObject);
    procedure DiagramMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ToolButton4Click(Sender: TObject);
    procedure LNKDocumentParsed(Sender: TObject; Doc: THREFDocument;  LinksList: TStrings);
    procedure LNKNewDocument(Sender: TObject; Link: THREF);
    procedure ToolButton6Click(Sender: TObject);
    procedure ToolButton2Click(Sender: TObject);
    procedure ToolButton7Click(Sender: TObject);
  private
  public
    procedure ShapeDblClick(Sender: TObject);
    procedure GO(URL: string);
  end;

var
  frmMain: TfrmMain;
  CashList : TStringList;

implementation

uses UnData, UnConfig;

{$R *.DFM}

procedure TfrmMain.ToolButton1Click(Sender: TObject);
begin
 ClearParent(Diagram);
end;

procedure TfrmMain.DiagramMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 TIECustomShape.UnselectAllShapes(Diagram);
end;

procedure TfrmMain.ToolButton4Click(Sender: TObject);
begin
if OpenDialog1.Execute then
 begin
  edURL.Text := OpenDialog1.FileName;
  GO(edURL.Text);
 end;
end;

procedure TfrmMain.ShapeDblClick(Sender: TObject);
begin
 With TIEParserShape(Sender) do
  begin
   if HREFdoc.HTML.Count=0 then
    begin
     ShowMessage('Source code hasn''t been loaded');
     frmData.Close;
     exit;
    end;
   frmData.MemoData.Lines.Text:=HREFdoc.HTML.Text;
   frmData.Caption := URL+' ['+HREFdoc.Title+']';
   frmData.Show;
  end;
end;

procedure TfrmMain.LNKDocumentParsed(Sender: TObject; Doc: THREFDocument;  LinksList: TStrings);
var i : integer;
    SrcShape : TIEParserShape;
begin
 SrcShape:=ShapeOf(Diagram,Doc.HREFself.UniqueDocURL);
 SrcShape.HREFdoc.Assign(Doc);  //Store document sources
 for i:=0 to LinksList.Count-1 do
   AddLink(Diagram,SrcShape,ShapeOf(Diagram, THREF(LinksList.Objects[i]).UniqueDocURL));
end;


procedure TfrmMain.LNKNewDocument(Sender: TObject; Link: THREF);
begin
 AddShape(Diagram,DocImages,Link).OnDblClick:=ShapeDblClick;
end;

procedure TfrmMain.ToolButton6Click(Sender: TObject);
begin
 GO(edURL.Text);
end;

procedure TfrmMain.GO(URL:string);
begin
  ClearParent(Diagram);
  Diagram.Cursor := crHourGlass;
 try
  LNK.ParseFileByURL(URL);
 finally
  Diagram.Cursor := crDefault;
  Diagram.Invalidate;
 end;
end;

procedure TfrmMain.ToolButton2Click(Sender: TObject);
begin
 frmConfig.ShowModal;
end;

procedure TfrmMain.ToolButton7Click(Sender: TObject);
begin
 LNK.DoTerminate := true;
 HTTPgate.Disconnect;
 FTPgate.Disconnect;
end;

end.

