{------------------------------------------------------------------------------
  TSrvCtrl v2.1
 ------------------------------------------------------------------------------
  v1.0:
  Copyright 2002 on TMEDIA, Dettelbachstr. 2,
  D-78315 Radolfzell, info@tmedia.de

  v2.x, based on the original component:
  Copyright 2002 by GSC
  info@gsc.hu

  FREE for non commercial use
 ------------------------------------------------------------------------------

  With this Component you can handle NT/2000-Services.

  Please send me feedback or buginfo to info@tmedia.de or to info@gsc.hu
 ------------------------------------------------------------------------------
  Date last modified:  Jun 22, 2002
 ------------------------------------------------------------------------------
  History: (* : change/bugfix; + : addition; - : removed)
    v2.1:
    + Delete method to mark a service for deletion
    + Access property to allow you setup witch access right do you want to get
      while connecting to service manager
    * Renamed local variable KeyName in DisplayNametoKeyName to prevent conflict
      with the property called KeyName
    + Implemented Lock/Unlock methods
    + VerifyAction property to prevent exception when service control is not done
    * SCHandle was not reset correctly on disconnect
    * Machine property returned empty string when setting 'localhost'
    + New string constants: SLocalSystem, SGroupID
    * Retriving/changing service description on Windows 2000
    * Some minor bugfixes
    v2.0:
    + Component completely rewritten by GSC.
      You may use it from now on just like TRegistry.
 ------------------------------------------------------------------------------
  Future plans:
    * Changing service dependencies
    * Retrive lock status information
    * Integrating new, Win2000 specific functions:
       * Retriving/changing failure actions
       * Retriving processid and service flags
 ------------------------------------------------------------------------------}
unit SrvCtrl;

interface

uses
  Windows, Messages, SysUtils, Classes, Dialogs, Forms, WinSvc, WinSvcEx;

const
  stKernel       = SERVICE_KERNEL_DRIVER;
  stFileSys      = SERVICE_FILE_SYSTEM_DRIVER;
  stAdapter      = SERVICE_ADAPTER;
  stReconDriver  = SERVICE_RECOGNIZER_DRIVER;
  stDriver       = SERVICE_DRIVER;
  stOwnProcess   = SERVICE_WIN32_OWN_PROCESS;
  stShareProcess = SERVICE_WIN32_SHARE_PROCESS;
  stWin32        = SERVICE_WIN32;
  stInteractive  = SERVICE_INTERACTIVE_PROCESS;
  stAllType      = SERVICE_TYPE_ALL;

  caStop         = SERVICE_ACCEPT_STOP;
  caPauseCont    = SERVICE_ACCEPT_PAUSE_CONTINUE;
  caShutdown     = SERVICE_ACCEPT_SHUTDOWN;

  SGroupID       = SC_GROUP_IDENTIFIER;
  SLocalSystem   = 'LocalSystem';
  
type
  TListServiceType  = (lstAll, lstWin32, lstDriver);
  TListServiceState = (lssAll, lssActive, lssInactive);
  TReturnFormat     = (rfKeyNames, rfDisplayNames, rfAll);
  TSrvRunState      = (srtStopped, srtStartPending, srtStopPending, srtRunning,
                       srtContinuePending, srtPausePending, srtPaused);
  TSrvStartType     = (sstBoot, sstSystem, sstAuto, sstDemand, sstDisabled);
  TSrvErrorCtrl     = (secIgnore, secNormal, secSevere, secCritical);
  TSrvAccessType    = (satGetConfig, satChangeConfig, satGetStatus, satEnumDependents, satStart,
                       satStop, satPauseContinue, satInterrogate, satUserDefCtrl);
  TSrvAccessTypes   = set of TSrvAccessType;

  ESrvCtrlException = class(Exception)
  private
    FErrorCode : Cardinal;
  public
    property ErrorCode : Cardinal read FErrorCode;
    constructor Create(AErrorCode : Cardinal); overload;
    constructor Create(const Msg : string); overload;
  end;

  TOnProgressEvent = procedure(Sender : TObject; Progress : Cardinal; var WaitFor : boolean) of object;
  
  TSrvCtrl = class(TComponent)
  private
    { Private declarations }
    FAccess : TSrvAccessTypes; 
    FDependencies : TStrings;
    FKeyName : string;
    FLockHandle : SC_LOCK;
    FMachine : string;
    FSCHandle : SC_HANDLE;
    FService : SC_HANDLE;
    FAfterClose : TNotifyEvent;
    FAfterConnect : TNotifyEvent;
    FAfterDisconnect : TNotifyEvent;
    FAfterOpen : TNotifyEvent;
    FAfterStateChange : TNotifyEvent;
    FBeforeClose : TNotifyEvent;
    FBeforeConnect : TNotifyEvent;
    FBeforeDisconnect : TNotifyEvent;
    FBeforeOpen : TNotifyEvent;
    FBeforeStateChange : TNotifyEvent;
    FOnProgress : TOnProgressEvent;
    FVerifyControl: boolean;
    function GetActive : boolean;
    procedure SetActive(Value : boolean);
    function GetMachine : string;
    procedure SetMachine(const Value : string);
    procedure SetKeyName(const Value : string);
    function GetServiceType : Cardinal;
    procedure SetServiceType(Value : Cardinal);
    function GetStartType : TSrvStartType;
    procedure SetStartType(Value : TSrvStartType);
    function GetErrorControl : TSrvErrorCtrl;
    procedure SetErrorControl(Value : TSrvErrorCtrl);
    function GetTagId : Cardinal;
    procedure SetTagId(Value : Cardinal);
    function GetBinaryPathName : string;
    procedure SetBinaryPathName(const Value : string);
    function GetLoadOrderGroup : string;
    procedure SetLoadOrderGroup(Value : string);
    function GetStartName : string;
    function GetDisplayName : string;
    procedure SetDisplayName(const Value : string);
    function GetControlsAccepted : Cardinal;
    function GetDependencies : TStrings;
    function CheckOpen(Service : boolean; RaiseException : boolean = true) : boolean;
    procedure DoControlService(CtrlCode : Cardinal);
    procedure DoProgress(ExpectedState : Cardinal);
    procedure SetAccess(const Value: TSrvAccessTypes);
    function GetDescription: string;
    procedure SetDescription(const Value: string);
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure OpenServiceByDisp(const ADisplayName : string);
    property BinaryPathName : string read GetBinaryPathName write SetBinaryPathName;
    property ControlsAccepted : Cardinal read GetControlsAccepted;
    property Dependencies: TStrings read GetDependencies;
    property Description : string read GetDescription write SetDescription;
    property DisplayName : string read GetDisplayName write SetDisplayName;
    property ErrorControl: TSrvErrorCtrl read GetErrorControl write SetErrorControl;
    property LoadOrderGroup: string read GetLoadOrderGroup write SetLoadOrderGroup;
    property LockHandle : SC_LOCK read FLockHandle;
    property SCHandle : SC_HANDLE read FSCHandle;
    property Service : SC_HANDLE read FService;
    property ServiceStartName : string read GetStartName;
    property ServiceType : Cardinal read GetServiceType write SetServiceType;
    property StartType : TSrvStartType read GetStartType write SetStartType;
    property TagId : Cardinal read GetTagId write SetTagId;
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Connect;
    procedure Disconnect;
    procedure ListServices(Strings : TStrings; ServiceType : TListServiceType; ServiceState : TListServiceState; WithInterActive : boolean; ReturnFormat : TReturnFormat);
    procedure GetConfig(var Config : TQueryServiceConfig);
    procedure GetStatus(var Status : TServiceStatus);
    function KeyNameToDisplayName(const AKeyName : string) : string;
    function DisplayNameToKeyName(const ADisplayName : string) : string;
    function GetRunState : TSrvRunState;
    function InRunState(RunState : TSrvRunState) : boolean;
    procedure SetupStartAccount(const AccountName, Password : string);
    procedure CloseService;
    procedure Start;overload;
    procedure Start(Args : array of PChar);overload;
    procedure Stop;
    procedure Pause;
    procedure Continue;
    procedure Interrogate;
    procedure Delete(Close : boolean = true);
    procedure Lock;
    procedure Unlock;
    function AcceptControl(Code : Cardinal) : boolean;
    function IsGroupName(const AName : string) : boolean;
  published
    { Published declarations }
    property Access : TSrvAccessTypes read FAccess write SetAccess;
    property Active : boolean read GetActive write SetActive;
    property Machine : string read GetMachine write SetMachine;
    property KeyName : string read FKeyName write SetKeyName;
    property VerifyAction : boolean read FVerifyControl write FVerifyControl;
    property AfterClose : TNotifyEvent read FAfterClose write FAfterClose;
    property AfterConnect : TNotifyEvent read FAfterConnect write FAfterConnect;
    property AfterDisconnect : TNotifyEvent read FAfterDisconnect write FAfterDisconnect;
    property AfterOpen : TNotifyEvent read FAfterOpen write FAfterOpen;
    property AfterStateChange : TNotifyEvent read FAfterStateChange write FAfterStateChange;
    property BeforeClose : TNotifyEvent read FBeforeClose write FBeforeClose;
    property BeforeConnect : TNotifyEvent read FBeforeConnect write FBeforeConnect;
    property BeforeDisconnect : TNotifyEvent read FBeforeDisconnect write FBeforeDisconnect;
    property BeforeOpen : TNotifyEvent read FBeforeOpen write FBeforeOpen;
    property BeforeStateChange : TNotifyEvent read FBeforeStateChange write FBeforeStateChange;
    property OnProgress : TOnProgressEvent read FOnProgress write FOnProgress;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('System', [TSrvCtrl]);
end;

resourcestring

{$DEFINE ENGLISH}

{$IFDEF ENGLISH}
  SNoServiceOpen = 'No service opened!';
  SNotCennected  = 'Not connected to Service Manager!';
  SErrorStateChg = 'Error while changing service state!';
  SAlreadyLocked = 'The database is already locked!';
  SNotLocked     = 'The database is not locked!';
  SNotSupported  = 'That function is not supported on this operating system!';
{$ENDIF}

{$IFDEF HUNGARIAN}
  SNoServiceOpen = 'Nem nyitott meg szolgltatst!';
  SNotCennected  = 'Nincs kapcsolat a Service Manager-hez!';
  SErrorStateChg = 'Hiba a szolgltats llapotnak vltsakor!';
  SAlreadyLocked = 'Az adatbzis mr zrolva van!';
  SNotLocked     = 'Az adatbzis nincs zrolva!';
  SNotSupported  = 'Ez a funkci nem tmogatott ezen az opercis rendszeren!';
{$ENDIF}

{$IFDEF DEUTCH}
  SNoServiceOpen = 'Kein Service geffnet!!';
  SNotCennected  = 'Keine Verbindung zum Service Manager!';
  SErrorStateChg = 'Fehler beim ndern des Service-Status!';
  SAlreadyLocked = 'Die Datenbank ist schon gesichert!';
  SNotLocked     = 'Die Datenbank ist noch nicht gesichert!';
  SNotSupported  = 'Diese Funktion ist nicht verfgbar auf diesem OS!';
{$ENDIF}

{ ESrvCtrlException }

constructor ESrvCtrlException.Create(AErrorCode: Cardinal);
begin
  FErrorCode := AErrorCode;
  inherited Create(SysErrorMessage(AErrorCode));
end;

constructor ESrvCtrlException.Create(const Msg : string);
begin
  FErrorCode := 0;
  inherited Create(Msg);
end;

{ TSrvCtrl }

constructor TSrvCtrl.Create;
begin
  inherited Create(AOwner);
  FSCHandle := 0;
  FService := 0;
  FMachine := '';
  FKeyName := '';
  FDependencies := TStringList.Create;
  FAccess := [satChangeConfig, satEnumDependents, satInterrogate,
              satPauseContinue, satGetConfig, satGetStatus, satStart,
              satStop, satUserDefCtrl];
  FLockHandle := nil;
end;

destructor TSrvCtrl.Destroy;
begin
  if Assigned(FLockHandle) then
    Unlock;

  Disconnect;

  FDependencies.Free;

  inherited Destroy;
end;

function TSrvCtrl.GetActive: boolean;
begin
  Result := FSCHandle <> 0;
end;

procedure TSrvCtrl.SetActive(Value: boolean);
begin
  Disconnect;

  if Value then
    Connect;
end;

procedure TSrvCtrl.Connect;
begin
  if Active then
    Disconnect;

  if Assigned(FBeforeConnect) then
    FBeforeConnect(Self);

  FSCHandle := OpenSCManager(PChar(FMachine),nil,SC_MANAGER_ALL_ACCESS);
  if FSCHandle = 0 then
    raise ESrvCtrlException.Create(GetLastError);

  if Assigned(FAfterConnect) then
    FAfterConnect(Self);

  if FKeyName <> '' then
    SetKeyName(FKeyName);
end;

procedure TSrvCtrl.Disconnect;
begin
  if CheckOpen(true,false) then
    CloseService;

  if CheckOpen(false,false) then
    begin
      if Assigned(FBeforeDisconnect) then
        FBeforeDisconnect(Self);

      CloseServiceHandle(FSCHandle);
      FSCHandle := 0;

      if Assigned(FAfterDisconnect) then
        FAfterDisconnect(Self);
    end;
end;

procedure TSrvCtrl.SetKeyName(const Value : string);
const
  AccessValues : array[satGetConfig..satUserDefCtrl] of DWORD =
    (SERVICE_QUERY_CONFIG, SERVICE_CHANGE_CONFIG, SERVICE_QUERY_STATUS,
     SERVICE_ENUMERATE_DEPENDENTS, SERVICE_START, SERVICE_STOP,
     SERVICE_PAUSE_CONTINUE, SERVICE_INTERROGATE, SERVICE_USER_DEFINED_CONTROL);
var
  Temp : string;
  DesiredAccess : DWORD;
  i : TSrvAccessType;
begin
  if FService <> 0 then
    CloseService;

  Temp := FKeyName;
  FKeyName := Value;
  if FSCHandle <> 0 then
    begin
      if Assigned(FBeforeOpen) then
        FBeforeOpen(Self);

      DesiredAccess := 0;
      for i := satGetConfig to satUserDefCtrl do
        if i in FAccess then
          DesiredAccess := DesiredAccess or AccessValues[i];

      FService := OpenService(FSCHandle,PChar(Value),DesiredAccess);
      if FService = 0 then
        begin
          FKeyName := Temp;
          raise ESrvCtrlException.Create(GetLastError);
        end;

      if Assigned(FAfterOpen) then
        FAfterOpen(Self);
    end;
end;

procedure TSrvCtrl.CloseService;
begin
  CheckOpen(true);

  if Assigned(FBeforeClose) then
    FBeforeClose(Self);

  if not CloseServiceHandle(FService) then
    raise ESrvCtrlException.Create(GetLastError);

  FService := 0;

  if Assigned(FAfterClose) then
    FAfterClose(Self);
end;

function TSrvCtrl.DisplayNameToKeyName(const ADisplayName: string): string;
var
  ServiceName : PChar;
  NeedLength : Cardinal;
  ErrorCode : Cardinal;
begin
  Result := '';

  CheckOpen(false);

  NeedLength := 2;
  ServiceName := StrAlloc(NeedLength);

  if not GetServiceKeyName(FSCHandle, PChar(ADisplayName), ServiceName, NeedLength) then
    begin
      ErrorCode := GetLastError;
      if ErrorCode = ERROR_INSUFFICIENT_BUFFER then
        begin
          StrDispose(ServiceName);
          ServiceName := StrAlloc(NeedLength + 1);
          if not GetServiceKeyName(FSCHandle, PChar(ADisplayName), ServiceName, NeedLength) then
            raise ESrvCtrlException.Create(GetLastError)
        end
        else
          raise ESrvCtrlException.Create(ErrorCode)
    end;

  Result := StrPas(ServiceName);
end;

procedure TSrvCtrl.ListServices(Strings : TStrings; ServiceType : TListServiceType; ServiceState : TListServiceState; WithInterActive : boolean; ReturnFormat : TReturnFormat);
var
  st : TEnumServiceStatus;
  Buffer : array of TEnumServiceStatus;
  NumNeed, NeedBytes, Ret, Resume, ErrorCode : Cardinal;
  i : integer;
  SrvState,
  SrvType : DWord;
begin
  CheckOpen(false);

  case ServiceState of
    lssAll: SrvState := SERVICE_ACTIVE or SERVICE_INACTIVE;
    lssActive: SrvState := SERVICE_ACTIVE;
    lssInactive: SrvState := SERVICE_INACTIVE;
    else
      exit;
  end;

  case ServiceType of
    lstAll : SrvType := SERVICE_WIN32 or SERVICE_DRIVER;
    lstWin32 : SrvType := SERVICE_WIN32;
    lstDriver : SrvType := SERVICE_DRIVER;
    else
      exit;
  end;

  if WithInterActive then
    SrvType := SrvType or SERVICE_INTERACTIVE_PROCESS;

  Strings.Clear;
  Ret := 0;
  NeedBytes := 0;
  Resume := 0;
  EnumServicesStatus(FSCHandle,SrvType,SrvState,st,0,NeedBytes,Ret,Resume);
  ErrorCode := GetLastError;
  if ErrorCode = ERROR_MORE_DATA then
    begin
      NumNeed := NeedBytes div SizeOf(TEnumServiceStatus) + 1;
      SetLength(Buffer,NumNeed);
      if EnumServicesStatus(FSCHandle,SrvType,SrvState,Buffer[0],SizeOf(TEnumServiceStatus) * numneed,NeedBytes,Ret,Resume) then
        begin
          for i := low(buffer) to ret - 1 do
            if Assigned(buffer[i].lpServiceName) then
              case ReturnFormat of
                rfKeyNames     : Strings.Add(buffer[i].lpServiceName);
                rfDisplayNames : Strings.Add(buffer[i].lpDisplayName);
                rfAll          : Strings.Add(buffer[i].lpServiceName + '=' + buffer[i].lpDisplayName);
              end;
        end;
    end
    else
      raise ESrvCtrlException.Create(ErrorCode);
end;

function TSrvCtrl.KeyNameToDisplayName(const AKeyName: string): string;
var
  ResultName : PChar;
  NeedLength : Cardinal;
  ErrorCode : Cardinal;
begin
  Result := '';

  CheckOpen(false);

  NeedLength := 2;
  ResultName := StrAlloc(NeedLength);

  if not GetServiceDisplayName(FSCHandle, PChar(AKeyName), ResultName, NeedLength) then
    begin
      ErrorCode := GetLastError;
      if ErrorCode = ERROR_INSUFFICIENT_BUFFER then
        begin
          StrDispose(ResultName);
          ResultName := StrAlloc(NeedLength + 1);
          if not GetServiceDisplayName(FSCHandle, PChar(AKeyName), ResultName, NeedLength) then
            raise ESrvCtrlException.Create(GetLastError)
        end
        else
          raise ESrvCtrlException.Create(ErrorCode)
    end;

  Result := StrPas(ResultName);
end;

function TSrvCtrl.GetMachine : string;
begin
  if FMachine = '' then Result := 'localhost'
                   else Result := FMachine;
end;

procedure TSrvCtrl.SetMachine(const Value: string);
begin
  if Value <> FMachine then
    begin
      SetActive(false);
      if Value = 'localhost' then FMachine := ''
                             else FMachine := Value;
    end;
end;

procedure TSrvCtrl.OpenServiceByDisp(const ADisplayName: string);
begin
  SetKeyName(DisplayNameToKeyName(ADisplayName));
end;

procedure TSrvCtrl.GetConfig(var Config: TQueryServiceConfig);
var
  NeedBytes, ErrorCode : Cardinal;
  Buffer : PQueryServiceConfig;
begin
  CheckOpen(true);

  Buffer := nil;
  if not QueryServiceConfig(FService,nil,0,NeedBytes) then
    begin
      ErrorCode := GetLastError;
      if ErrorCode = ERROR_INSUFFICIENT_BUFFER then
        begin
          GetMem(Buffer,NeedBytes + 1);

          try
            if not QueryServiceConfig(FService,Buffer,NeedBytes,NeedBytes) then
              raise ESrvCtrlException.Create(GetLastError)
          finally
//            FreeMem(Buffer);
          end;
        end
        else
          raise ESrvCtrlException.Create(ErrorCode)
    end;

  Config := Buffer^;
end;

procedure TSrvCtrl.GetStatus(var Status: TServiceStatus);
begin
  CheckOpen(true);

  if not QueryServiceStatus(FService,Status) then
    raise ESrvCtrlException.Create(GetLastError);
end;

function TSrvCtrl.CheckOpen(Service : boolean; RaiseException : boolean = true) : boolean;
begin
  if Service then
    begin
      Result := FService <> 0;
      if not Result and RaiseException then
        raise ESrvCtrlException.Create(SNoServiceOpen);
    end
    else
    begin
      Result := FSCHandle <> 0;
      if not Result and RaiseException then
        raise ESrvCtrlException.Create(SNotCennected);
    end;
end;

function TSrvCtrl.GetServiceType: Cardinal;
var
  SrvConfig : TQueryServiceConfig;
begin
  CheckOpen(true);

  GetConfig(SrvConfig);
  Result := SrvConfig.dwServiceType;
end;

function TSrvCtrl.GetBinaryPathName: string;
var
  SrvConfig : TQueryServiceConfig;
begin
  CheckOpen(true);

  GetConfig(SrvConfig);
  Result := SrvConfig.lpBinaryPathName;
end;

function TSrvCtrl.GetDisplayName: string;
var
  SrvConfig : TQueryServiceConfig;
begin
  CheckOpen(true);

  GetConfig(SrvConfig);
  Result := SrvConfig.lpDisplayName;
end;

function TSrvCtrl.GetErrorControl: TSrvErrorCtrl;
var
  SrvConfig : TQueryServiceConfig;
begin
  CheckOpen(true);

  GetConfig(SrvConfig);
  Result := TSrvErrorCtrl(SrvConfig.dwErrorControl);
end;

function TSrvCtrl.GetLoadOrderGroup: string;
var
  SrvConfig : TQueryServiceConfig;
begin
  CheckOpen(true);

  GetConfig(SrvConfig);
  Result := SrvConfig.lpLoadOrderGroup;
end;

function TSrvCtrl.GetStartName: string;
var
  SrvConfig : TQueryServiceConfig;
begin
  CheckOpen(true);

  GetConfig(SrvConfig);
  Result := SrvConfig.lpServiceStartName;
end;

function TSrvCtrl.GetStartType: TSrvStartType;
var
  SrvConfig : TQueryServiceConfig;
begin
  CheckOpen(true);

  GetConfig(SrvConfig);
  Result := TSrvStartType(SrvConfig.dwStartType);
end;

function TSrvCtrl.GetTagId: Cardinal;
var
  SrvConfig : TQueryServiceConfig;
begin
  CheckOpen(true);

  GetConfig(SrvConfig);
  Result := SrvConfig.dwTagId;
end;

function TSrvCtrl.GetDependencies: TStrings;
var
  SrvConfig : TQueryServiceConfig;
  TempDepend : PChar;
begin
  CheckOpen(true);

  GetConfig(SrvConfig);
  FDependencies.Clear;
  if SrvConfig.lpDependencies <> nil then
    begin
      if StrPas(SrvConfig.lpDependencies) <> '' then
        begin
          TempDepend := SrvConfig.lpDependencies;
          while (TempDepend[1] <> #0) do
            begin
              if (TempDepend[0] = #0) and (TempDepend[1] <> #0) then
                TempDepend := TempDepend + 1;
              FDependencies.Add(StrPas(TempDepend));
              TempDepend := TempDepend + StrLen(TempDepend);
            end;
        end;
    end;

  Result := FDependencies;
end;

procedure TSrvCtrl.Continue;
begin
  DoControlService(SERVICE_CONTROL_CONTINUE);
end;

procedure TSrvCtrl.Pause;
begin
  DoControlService(SERVICE_CONTROL_PAUSE);
end;

procedure TSrvCtrl.Start(Args: array of PChar);
var
  TempArgs : PChar;
  NumArgs : Cardinal;
begin
  if High(Args) = -1 then
    begin
      TempArgs := nil;
      NumArgs := 0;
    end
    else
    begin
      TempArgs := Args[0];
      NumArgs := High(Args) + 1;
    end;

  if Assigned(FBeforeStateChange) then
    FBeforeStateChange(Self);

  if not StartService(FService,NumArgs,TempArgs) then
    raise ESrvCtrlException.Create(GetLastError);

  DoProgress(SERVICE_RUNNING);
  
  if Assigned(FAfterStateChange) then
    FAfterStateChange(Self);
end;

procedure TSrvCtrl.Start;
begin
  Start([]);
end;

procedure TSrvCtrl.Stop;
begin
  DoControlService(SERVICE_CONTROL_STOP);
end;

procedure TSrvCtrl.Interrogate;
begin
  DoControlService(SERVICE_CONTROL_INTERROGATE);
end;

function TSrvCtrl.GetRunState : TSrvRunState;
var
  SrvStatus : TServiceStatus;
begin
  GetStatus(SrvStatus);
  Result := TSrvRunState(SrvStatus.dwCurrentState - 1);
end;

function TSrvCtrl.InRunState(RunState : TSrvRunState) : boolean;
begin
  Result := GetRunState = RunState;
end;

function TSrvCtrl.GetControlsAccepted: Cardinal;
var
  SrvStatus : TServiceStatus;
begin
  GetStatus(SrvStatus);
  Result := SrvStatus.dwControlsAccepted;
end;

procedure TSrvCtrl.DoProgress(ExpectedState : Cardinal);
var
  State : TServiceStatus;
  CheckPoint,
  StartTime : Cardinal;
  WaitFor : boolean;
begin
  if not FVerifyControl then
    exit;
    
  GetStatus(State);

  while (State.dwCurrentState <> ExpectedState) do
    begin
      //dwCheckPoint contains a value that the service increments periodically
      // to report its progress during a lengthy operation.
      CheckPoint := State.dwCheckPoint;

      if Assigned(FOnProgress) then
        begin
          WaitFor := true;
          FOnProgress(Self,CheckPoint,WaitFor);
          if not WaitFor then
            exit;
        end;

      Application.ProcessMessages;

      // wait a bit before checking status again - dwWaitHint is the estimated amount of time
      // the calling program should wait before calling QueryServiceStatus() again
      StartTime := GetTickCount;
      while ((GetTickCount - StartTime) <= State.dwWaitHint){ and (State.dwCurrentState <> ExpectedState) }do
        begin
          Application.ProcessMessages;

          GetStatus(State);

          if Assigned(FOnProgress) then
            begin
              WaitFor := true;
              FOnProgress(Self,CheckPoint,WaitFor);
              if not WaitFor then
                exit;
            end;
        end;

      GetStatus(State);
      if (State.dwCheckPoint = CheckPoint) and (State.dwCurrentState <> ExpectedState) then
        begin
          // QueryServiceStatus didn't increment dwCheckPoint as it should have.
          // avoid an infinite loop by breaking
          raise ESrvCtrlException.Create(SErrorStateChg);
          break;
        end;
    end;
end;

procedure TSrvCtrl.DoControlService(CtrlCode: Cardinal);
var
  State : TServiceStatus;
  ExpectState : Cardinal;
begin
  case CtrlCode of
    SERVICE_CONTROL_STOP : ExpectState := SERVICE_STOPPED;
    SERVICE_CONTROL_PAUSE : ExpectState := SERVICE_PAUSED;
    SERVICE_CONTROL_CONTINUE : ExpectState := SERVICE_RUNNING;
    SERVICE_CONTROL_INTERROGATE : ExpectState := $FFFFFFFF;
    else
      exit;
  end;

  if Assigned(FBeforeStateChange) then
    FBeforeStateChange(Self);

  if not ControlService(FService,CtrlCode,State) then
    raise ESrvCtrlException.Create(GetLastError);

  if ExpectState < $FFFFFFFF then
    DoProgress(ExpectState);
    
  if Assigned(FAfterStateChange) then
    FAfterStateChange(Self);
end;

procedure TSrvCtrl.SetStartType(Value: TSrvStartType);
begin
  CheckOpen(true);

  if not ChangeServiceConfig(FService,SERVICE_NO_CHANGE,Cardinal(Value),SERVICE_NO_CHANGE,nil,nil,nil,nil,nil,nil,nil) then
    raise ESrvCtrlException.Create(GetLastError);
end;

procedure TSrvCtrl.SetErrorControl(Value: TSrvErrorCtrl);
begin
  CheckOpen(true);

  if not ChangeServiceConfig(FService,SERVICE_NO_CHANGE,SERVICE_NO_CHANGE,Cardinal(Value),nil,nil,nil,nil,nil,nil,nil) then
    raise ESrvCtrlException.Create(GetLastError);
end;

procedure TSrvCtrl.SetBinaryPathName(const Value: string);
begin
  CheckOpen(true);

  if not ChangeServiceConfig(FService,SERVICE_NO_CHANGE,SERVICE_NO_CHANGE,SERVICE_NO_CHANGE,PChar(Value),nil,nil,nil,nil,nil,nil) then
    raise ESrvCtrlException.Create(GetLastError);
end;

procedure TSrvCtrl.SetLoadOrderGroup(Value: string);
begin
  CheckOpen(true);

  if not ChangeServiceConfig(FService,SERVICE_NO_CHANGE,SERVICE_NO_CHANGE,SERVICE_NO_CHANGE,nil,PChar(Value),nil,nil,nil,nil,nil) then
    raise ESrvCtrlException.Create(GetLastError);
end;

procedure TSrvCtrl.SetTagId(Value: Cardinal);
begin
  CheckOpen(true);

  if not ChangeServiceConfig(FService,SERVICE_NO_CHANGE,SERVICE_NO_CHANGE,SERVICE_NO_CHANGE,nil,nil,@Value,nil,nil,nil,nil) then
    raise ESrvCtrlException.Create(GetLastError);
end;

procedure TSrvCtrl.SetDisplayName(const Value: string);
var
  Temp : string;
begin
  CheckOpen(true);

  Temp := Copy(Value,1,256);
  if not ChangeServiceConfig(FService,SERVICE_NO_CHANGE,SERVICE_NO_CHANGE,SERVICE_NO_CHANGE,nil,nil,nil,nil,nil,nil,PChar(Temp)) then
    raise ESrvCtrlException.Create(GetLastError);
end;

procedure TSrvCtrl.SetupStartAccount(const AccountName,
  Password: string);
begin
  CheckOpen(true);

  if not ChangeServiceConfig(FService,SERVICE_NO_CHANGE,SERVICE_NO_CHANGE,SERVICE_NO_CHANGE,nil,nil,nil,nil,PChar(AccountName),PChar(Password),nil) then
    raise ESrvCtrlException.Create(GetLastError);
end;

procedure TSrvCtrl.SetServiceType(Value: Cardinal);
begin
  CheckOpen(true);

  if not ChangeServiceConfig(FService,Value,SERVICE_NO_CHANGE,SERVICE_NO_CHANGE,nil,nil,nil,nil,nil,nil,nil) then
    raise ESrvCtrlException.Create(GetLastError);
end;

function TSrvCtrl.AcceptControl(Code: Cardinal): boolean;
begin
  Result := ControlsAccepted and Code = Code;
end;

procedure TSrvCtrl.Delete(Close: boolean);
begin
  CheckOpen(true);
  if not DeleteService(FService) then
    raise ESrvCtrlException.Create(GetLastError);

  if Close then
    CloseService;
end;

procedure TSrvCtrl.SetAccess(const Value: TSrvAccessTypes);
begin
  FAccess := Value;

  if CheckOpen(false,false) then
    begin
      Disconnect;
      Connect;
    end;
end;

procedure TSrvCtrl.Lock;
begin
  CheckOpen(false);

  if Assigned(FLockHandle) then
    raise ESrvCtrlException.Create(SAlreadyLocked);
    
  FLockHandle := LockServiceDatabase(FSCHandle);
  if not Assigned(FLockHandle) then
    raise ESrvCtrlException.Create(GetLastError);
end;

procedure TSrvCtrl.Unlock;
begin
  CheckOpen(false);

  if not Assigned(FLockHandle) then
    raise ESrvCtrlException.Create(SNotLocked);

  if not UnlockServiceDatabase(FLockHandle) then
    raise ESrvCtrlException.Create(GetLastError);

  FLockHandle := nil;
end;

function TSrvCtrl.GetDescription: string;
var
  Buffer : PServiceDescription;
  NeededBytes, LastError : Cardinal;
begin
  CheckOpen(true);
  
  Result := SNotSupported;
  if Assigned(QueryServiceConfig2) then
    begin
      if not QueryServiceConfig2(FService,SERVICE_CONFIG_DESCRIPTION,nil,0,NeededBytes) then
        begin
          LastError := GetLastError;
          if LastError = ERROR_INSUFFICIENT_BUFFER then
            begin
              GetMem(Buffer,NeededBytes + 1);
              try
                if not QueryServiceConfig2(FService,SERVICE_CONFIG_DESCRIPTION,Buffer,NeededBytes + 1,NeededBytes) then
                  raise ESrvCtrlException.Create(GetLastError);

                Result := StrPas(Buffer^.lpDescription);
              finally
                FreeMem(Buffer);
              end;
            end
            else
              raise ESrvCtrlException.Create(LastError);
        end;
    end;
end;

procedure TSrvCtrl.SetDescription(const Value: string);
var
  srvdesc : PServiceDescription;
  desc : string;
begin
  CheckOpen(true);

  if Assigned(ChangeServiceConfig2) then
    begin
      // Service descriptions can't be longer than 1024!!!
      desc := Copy(Value,1,1024);
      GetMem(srvdesc,SizeOf(TServiceDescription));
      GetMem(srvdesc^.lpDescription,Length(desc) + 1);
      try
        StrPCopy(srvdesc^.lpDescription, desc);

        if not ChangeServiceConfig2(FService,SERVICE_CONFIG_DESCRIPTION,srvdesc) then
          raise ESrvCtrlException.Create(GetLastError);
      finally
        FreeMem(srvdesc^.lpDescription);
        FreeMem(srvdesc);
      end;
    end;
end;

function TSrvCtrl.IsGroupName(const AName: string): boolean;
begin
  Result := Copy(AName,1,Length(SC_GROUP_IDENTIFIER)) = SC_GROUP_IDENTIFIER;
end;

end.
