unit OpEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, Options, DsgnIntf, DsgnWnds;

type
  TfrmOpEditor = class(TDesignWindow)
    pnlButtons: TPanel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    btnNew: TSpeedButton;
    btnDelete: TSpeedButton;
    Bevel3: TBevel;
    btnUp: TSpeedButton;
    btnDown: TSpeedButton;
    pnlData: TPanel;
    pnlOps: TPanel;
    pnlOpsLabel: TPanel;
    Label2: TLabel;
    lbxOptions: TListBox;
    pnlCat: TPanel;
    pnlCatLabel: TPanel;
    Label1: TLabel;
    lbxCategories: TListBox;
    splCatOps: TSplitter;
    procedure lbxCategoriesClick(Sender: TObject);
    procedure lbxOptionsClick(Sender: TObject);
    procedure btnNewClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    procedure ListCategories;
    procedure ListOptions;
    procedure UpdateData;
  protected
    Options: TOptions;
    function UniqueName(Component: TComponent): string; override;
    procedure Activated; override;
  public
    procedure FormClosed(Form: TCustomForm); override;
    procedure ComponentDeleted(Component: IPersistent); override;
    procedure FormModified; override;
  end;

  TOptionsEditor = class(TComponentEditor)
  public
    function GetVerbCount: integer; override;
    function GetVerb(index: integer): string; override;
    procedure ExecuteVerb(index: integer); override;
    procedure Edit; override;
  end;

  THKEYProperty = class(TPropertyEditor)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
    function GetValue: string; override;
    function AllEqual: Boolean; override;
  end;

  TCatProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
  end;

  TAssocProperty = class(TComponentProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

var
  frmOpEditor: TfrmOpEditor;

procedure Register;

implementation

uses
  Registry;

{$R *.DFM}

procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(TCategory), TOption, 'Category', TCatProperty);
  RegisterPropertyEditor(TypeInfo(TComponent), TOption, 'Associate', TAssocProperty);
  RegisterPropertyEditor(TypeInfo(HKEY), TOptions, 'RootKey', THKEYProperty);
  RegisterComponentEditor(TOptions, TOptionsEditor);
end;

type
  THKEYRec = record
    KeyName: string;
    Code: HKEY;
  end;

const
  MaxKeyNum = 7;

  RegKeys: array[1..MaxKeyNum] of THKEYRec = (
    (KeyName: 'HKEY_CLASSES_ROOT'; Code: HKEY_CLASSES_ROOT),
    (KeyName: 'HKEY_CURRENT_USER'; Code: HKEY_CURRENT_USER),
    (KeyName: 'HKEY_LOCAL_MACHINE'; Code: HKEY_LOCAL_MACHINE),
    (KeyName: 'HKEY_USERS'; Code: HKEY_USERS),
    (KeyName: 'HKEY_PERFORMANCE_DATA'; Code: HKEY_PERFORMANCE_DATA),
    (KeyName: 'HKEY_CURRENT_CONFIG'; Code: HKEY_CURRENT_CONFIG),
    (KeyName: 'HKEY_DYN_DATA'; Code: HKEY_DYN_DATA));

    strUnknown = '(Unknown)';

{$IFDEF VER120}
  RegEditor = 'Software\Borland\Delphi\4.0\Property Editors\OptionList Editor';
{$ENDIF}

{$IFDEF VER130}
  RegEditor = 'Software\Borland\Delphi\5.0\Property Editors\OptionList Editor';
{$ENDIF}

  rkTop = 'Top';
  rkLeft = 'Left';
  rkHeight = 'Height';
  rkWidth = 'Width';
  rkSplitter = 'Splitter';

function FindEditor(AOptions: TOptions): TfrmOpEditor;
var
  i: integer;
begin
  result := nil;
  for i := 0 to Screen.FormCount - 1 do begin
    if Screen.Forms[i] is TfrmOpEditor then begin
      if TfrmOpEditor(Screen.Forms[i]).Options = AOptions then begin
        result := TfrmOpEditor(Screen.Forms[i]);
        Break;
      end;
    end;
  end;
end;

procedure ShowItemsEditor(Designer: IDesigner; AOptions: TOptions);
var
  Editor: TfrmOpEditor;
begin
  if AOptions = nil then
    Exit;
  Editor := FindEditor(AOptions);
  if Editor = nil then begin
    Editor := TfrmOpEditor.Create(Application);
    try
      Editor.Caption := 'Editing ' + AOptions.Name + '.OptionList';
      Editor.Designer := IFormDesigner(Designer);
      Editor.Options := AOptions;
      Editor.Show;
    except
      Editor.Free;
      raise
    end;
  end else begin
    Editor.Show;
    if Editor.WindowState = wsMinimized then
      Editor.WindowState := wsNormal;
  end;
end;

{ THKEYProperty }

function THKEYProperty.GetAttributes: TPropertyAttributes;
begin
  result := [paValueList, paMultiSelect, paReadOnly];
end;

procedure THKEYProperty.GetValues(Proc: TGetStrProc);
var
  i: integer;
begin
  for i := 1 to MaxKeyNum do
    Proc(RegKeys[i].KeyName);
end;

procedure THKEYProperty.SetValue(const Value: string);
var
  i: integer;
begin
  i := 1;
  while (i <= MaxKeyNum) and (RegKeys[i].KeyName <> Value) do
    Inc(i);
  if i <= MaxKeyNum then
    SetOrdValue(RegKeys[i].Code)
  else
    SetOrdValue(LongInt(HKEY_CURRENT_USER));
end;

function THKEYProperty.GetValue: string;
var
  i: integer;
  h: HKEY;
begin
  h := GetOrdValue;
  i := 1;
  while (i <= MaxKeyNum) and (h <> RegKeys[i].Code) do
    Inc(i);
  if i <= MaxKeyNum then
    result := RegKeys[i].KeyName
  else
    result := strUnknown;
end;

function THKEYProperty.AllEqual: Boolean;
var
  I: Integer;
  V: LongInt;
begin
  Result := False;
  if PropCount > 1 then
  begin
    V := GetOrdValue;
    for I := 1 to PropCount - 1 do
      if GetOrdValueAt(I) <> V then Exit;
  end;
  Result := True;
end;

{ TOptionsEditor }

function TOptionsEditor.GetVerbCount: integer;
begin
  result := 1;
end;

function TOptionsEditor.GetVerb(index: integer): string;
begin
  if index = 0 then
    result := '&Options editor...'
end;

procedure TOptionsEditor.ExecuteVerb(index: integer);
begin
  if index = 0 then
    ShowItemsEditor(Designer, Component as TOptions);
end;

procedure TOptionsEditor.Edit;
begin
  if GetVerbCount > 0 then
    ExecuteVerb(0);
end;

{ TCatProperty }

function TCatProperty.GetAttributes: TPropertyAttributes;
begin
  result := [paMultiSelect, paValueList, paSortList, paRevertable];
end;

procedure TCatProperty.GetValues(Proc: TGetStrProc);
var
  O: TOptions;
  i: integer;
  S: TStringList;
begin
  S := TStringList.Create;
  try
    S.Sorted := true;
    S.Duplicates := dupIgnore;
    S.Add('(None)');
    O := (GetComponent(0) as TOption).Options;
    if O = nil then
      Exit;
    for i := 0 to O.Count - 1 do
      if (O.Items[i] as TOption).Category <> '' then
        S.Add(TOption(O.Items[i]).Category);
    for i := 0 to S.Count - 1 do
      Proc(S[i]);
  finally
    S.Free;
  end;
end;

function TCatProperty.GetValue: string;
begin
  result := GetStrValue;
  if result = '' then
    result := '(None)';
end;

procedure TCatProperty.SetValue(const Value: string);
begin
  if UpperCase(Value) = '(NONE)' then
    SetStrValue('')
  else
    SetStrValue(Value);
end;

{ TAssocProperty }

function TAssocProperty.GetAttributes: TPropertyAttributes;
begin
  result := [paValueList, paSortList, paRevertable];
end;

procedure TAssocProperty.GetValues(Proc: TGetStrProc);
var
  Root: TComponent;
  i: integer;
begin
  Root := Designer.GetRoot;
  for i := 0 to Root.ComponentCount - 1 do
    if (GetComponent(0) as TOption).AcceptComponent(Root.Components[i]) then
      Proc(Root.Components[i].Name);
end;

{ TfrmOpEditor }

procedure TfrmOpEditor.ListCategories;
var
  S: TStringList;
  i: integer;
begin
  S := TStringList.Create;
  try
    S.Sorted := true;
    S.Duplicates := dupIgnore;
    for i := 0 to Options.Count - 1 do
      S.Add((Options.Items[i] as TOption).Category);
    with lbxCategories do begin
      Clear;
      Items.Add('(None)');
      for i := 0 to S.Count - 1 do
        if S[i] <> '' then
          Items.Add(S[i]);
      if S.Count > 1 then
        Items.Add('(All options)');
    end;
  finally
    S.Free;
  end;
end;

procedure TfrmOpEditor.ListOptions;
var
  S: TStringList;
  i: integer;
  c: string;
  e: boolean;
begin
  S := TStringList.Create;
  try
    S.Sorted := false;
    if lbxCategories.ItemIndex <> -1 then begin
      c := UpperCase(lbxCategories.Items[lbxCategories.ItemIndex]);
      e := c = '(ALL OPTIONS)';
      if c = '(NONE)' then
        c := '';
      for i := 0 to Options.Count - 1 do
        if e or (UpperCase((Options.Items[i] as TOption).Category) = c) then
          S.AddObject((Options.Items[i] as TOption).Name,
              Options.Items[i] as TOption);
    end;
    lbxOptions.Clear;
    for i := 0 to S.Count - 1 do
      lbxOptions.Items.AddObject(S[i], S.Objects[i]);
  finally
    S.Free;
  end;
end;

procedure TfrmOpEditor.lbxCategoriesClick(Sender: TObject);
begin
  ListOptions;
  lbxOptionsClick(Sender);
end;

procedure TfrmOpEditor.lbxOptionsClick(Sender: TObject);
begin
  btnDelete.Enabled := lbxOptions.ItemIndex >= 0;
  btnUp.Enabled := lbxOptions.ItemIndex > 0;
  btnDown.Enabled := (lbxOptions.ItemIndex >= 0) and (lbxOptions.ItemIndex <
      lbxOptions.Items.Count - 1);
  if Active and (lbxOptions.ItemIndex >= 0) then
    Designer.SelectComponent(lbxOptions.Items.Objects[lbxOptions.ItemIndex] as TOption);
end;

procedure TfrmOpEditor.btnNewClick(Sender: TObject);
var
  i: integer;
  O: TOption;
begin
  i := lbxCategories.ItemIndex;
  O := TOption.Create(Options.Owner);
  O.Options := Options;
  O.Name := UniqueName(O);
  if (lbxCategories.ItemIndex <= 0) or (lbxCategories.ItemIndex = lbxCategories.Items.Count - 1) then
    O.Category := ''
  else
    O.Category := lbxCategories.Items[lbxCategories.ItemIndex];
  ListCategories;
  lbxCategories.ItemIndex := i;
  ListOptions;
  lbxOptions.ItemIndex := lbxOptions.Items.Count - 1;
  lbxOptionsClick(Sender);
end;

function TfrmOpEditor.UniqueName(Component: TComponent): string;
var
  s: string;
begin
  result := '';
  if Component <> nil then
    s := Component.ClassName
  else
    s := TOption.ClassName;
  if (Length(s) > 1) and (UpCase(s[1]) = 'T') then
    Delete(s, 1, 1);
  s := Options.Name + s;
  result := Designer.UniqueName(s);
end;

procedure TfrmOpEditor.btnDeleteClick(Sender: TObject);
var
  O: TOption; 
begin
  if lbxOptions.ItemIndex >= 0 then begin
    O := lbxOptions.Items.Objects[lbxOptions.ItemIndex] as TOption;
    O.Options := nil;
    O.Free;
    ListCategories;
    lbxCategories.ItemIndex := 0;
    ListOptions;
    if lbxOptions.Items.Count > 0 then
      lbxOptions.ItemIndex := 0
    else
      lbxOptions.ItemIndex := -1;
    lbxOptionsClick(Self);
  end;
end;

procedure TfrmOpEditor.FormClosed(Form: TCustomForm);
begin
  if Form = Designer.Form then
    Free;
end;

procedure TfrmOpEditor.Activated;
begin
  UpdateData;
end;

procedure TfrmOpEditor.UpdateData;
var
  TempC, TempO: string;
begin
  if lbxCategories.ItemIndex >= 0 then
    TempC := lbxCategories.Items[lbxCategories.ItemIndex]
  else
    TempC := '';
  if lbxOptions.ItemIndex >= 0 then
    TempO := lbxOptions.Items[lbxOptions.ItemIndex]
  else
    TempO := '';
  ListCategories;
  lbxCategories.ItemIndex := lbxCategories.Items.IndexOf(TempC);
  if lbxCategories.ItemIndex < 0 then
    lbxCategories.ItemIndex := 0;
  lbxCategoriesClick(Self);
  lbxOptions.ItemIndex := lbxOptions.Items.IndexOf(TempO);
  lbxOptionsClick(Self);
end;

procedure TfrmOpEditor.FormModified;
begin
  UpdateData;
end;

procedure TfrmOpEditor.ComponentDeleted(Component: IPersistent);
begin
  if ExtractPersistent(Component) = Options then begin
    Options := nil;
    Close;
  end;
end;

procedure TfrmOpEditor.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  with TRegistry.Create do
    try
      OpenKey(RegEditor, true);
      WriteInteger(rkTop, Top);
      WriteInteger(rkLeft, Left);
      WriteInteger(rkHeight, Height);
      WriteInteger(rkWidth, Width);
      WriteInteger(rkSplitter, splCatOps.Left);
    finally
      Free;
    end;
  Action := caFree;
end;

procedure TfrmOpEditor.btnUpClick(Sender: TObject);
var
  Oa, Op: TOption;
begin
  Oa := lbxOptions.Items.Objects[lbxOptions.ItemIndex] as TOption;
  Op := lbxOptions.Items.Objects[lbxOptions.ItemIndex - 1] as TOption;
  Oa.Index := Op.Index;
  UpdateData;
  Designer.Modified;
end;

procedure TfrmOpEditor.btnDownClick(Sender: TObject);
var
  Oa, Op: TOption;
begin
  Oa := lbxOptions.Items.Objects[lbxOptions.ItemIndex] as TOption;
  Op := lbxOptions.Items.Objects[lbxOptions.ItemIndex + 1] as TOption;
  Oa.Index := Op.Index;
  UpdateData;
  Designer.Modified;
end;

procedure TfrmOpEditor.FormCreate(Sender: TObject);
begin
  with TRegistry.Create do
    try
      OpenKey(RegEditor, true);
      if ValueExists(rkTop) then
        Top := ReadInteger(rkTop);
      if ValueExists(rkLeft) then
        Left := ReadInteger(rkLeft);
      if ValueExists(rkHeight) then
        Height := ReadInteger(rkHeight);
      if ValueExists(rkWidth) then
        Width := ReadInteger(rkWidth);
      if ValueExists(rkSplitter) then
        splCatOps.Left := ReadInteger(rkSplitter);
    finally
      Free;
    end;
end;

procedure TfrmOpEditor.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Shift = [] then begin
    if Key = VK_PRIOR then begin
      btnUpClick(Self);
      Key := 0;
    end else if Key = VK_NEXT then begin
      btnDownClick(Self);
      Key := 0;
    end else if Key = VK_INSERT then begin
      btnNewClick(Self);
      Key := 0;
    end else if Key = VK_DELETE then begin
      btnDeleteClick(Self);
      Key := 0;
    end;
  end;
end;

end.

