unit Options;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Registry, IniFiles;

type
  TBoolStr = (bsTrue, bsTrueUpper, bsTrueLower, bsOn, bsOnUpper, bsOnLower, bsNum);

  TTarget = (stReg, stIni, stMem);

  TDirLocation =
    (dirAppData, dirApplication, dirDevice, dirCommonAltStartUp, dirCommonDesktop,
    dirCommonFavorites, dirCommonFiles, dirCommonPrograms, dirCommonStartMenu,
    dirCommonStartUp, dirConfig, dirControls, dirCookies, dirDesktop, dirDesktopDir,
    dirFavorites, dirFonts, dirHistory, dirInternetCache, dirMedia, dirMyDocuments,
    dirNetHood, dirNone, dirOtherDevice, dirPrintHood, dirProgramFiles, dirPrograms,
    dirRecent, dirSendTo, dirStartMenu, dirStartUp, dirSystem, dirTemp, dirTemplates,
    dirWallpaper, dirWindows);

  TOptionType =
    (otString, otInteger, otFloat, otBoolean, otColor, otDate, otTime,
        otDateTime, otFont, otFlags, otStrArray, otIntArray, otFloatArray);

  TStoreMode = (stmString, stmBinary, stmStrArray, stmBinArray);

  TCategory = string;
  TDefValue = string;
  TOptions = class;

  TOption = class(TComponent)
  private
    FArraySeparator: string;
    FAssociate: TComponent;
    FBoolStr: TBoolStr;
    FCategory: TCategory;
    FDefValue: TDefValue;
    FFloatMask: string;
    FNameSeparator: string;
    FOptions: TOptions;
    FOptionType: TOptionType;
    FParentArraySeparator: boolean;
    FParentBoolStr: boolean;
    FParentFloatMask: boolean;
    FParentNameSeparator: boolean;
    FParentStoreMode: boolean;
    FStoreMode: TStoreMode;
    FValue: string;

    FOnUpdated: TNotifyEvent;

    function GetAsBoolean: boolean;
    function GetAsColor: TColor;
    function GetAsDate: TDateTime;
    function GetAsDateTime: TDateTime;
    function GetAsFlags(index: integer): boolean;
    function GetAsFloat: double;
    function GetAsFloatArray(index: integer): double;
    function GetAsFont: TFont;
    function GetAsIntArray(index: integer): integer;
    function GetAsInteger: integer;
    function GetAsStrArray(index: integer): string;
    function GetAsString: string;
    function GetAsTime: TDateTime;
    procedure GetBooleanType;
    procedure GetColorType;
    function GetCount: integer;
    procedure GetDateTimeType;
    procedure GetDateType;
    function GetDefAsBoolean: boolean;
    function GetDefAsColor: TColor;
    function GetDefAsDate: TDateTime;
    function GetDefAsDateTime: TDateTime;
    function GetDefAsFlags(index: integer): boolean;
    function GetDefAsFloat: double;
    function GetDefAsFloatArray(index: integer): double;
    function GetDefAsFont: TFont;
    function GetDefAsIntArray(index: integer): integer;
    function GetDefAsInteger: integer;
    function GetDefAsStrArray(index: integer): string;
    function GetDefAsString: string;
    function GetDefAsTime: TDateTime;
    function GetDefCount: integer;
    procedure GetFlagsType;
    procedure GetFloatArrayType;
    procedure GetFloatType;
    procedure GetFontType;
    function GetIndex: Integer;
    procedure GetIntArrayType;
    procedure GetIntegerType;
    procedure GetStrArrayType;
    procedure GetStringType;
    procedure GetTimeType;
    procedure SetArraySeparator(const AValue: string);
    procedure SetAsBoolean(ABoolean: boolean);
    procedure SetAsColor(AColor: TColor);
    procedure SetAsDate(ADate: TDateTime);
    procedure SetAsDateTime(ADateTime: TDateTime);
    procedure SetAsFlags(index: integer; ABoolean: boolean);
    procedure SetAsFloat(AFloat: double);
    procedure SetAsFloatArray(index: integer; AFloat: double);
    procedure SetAsFont(AFont: TFont);
    procedure SetAsIntArray(index: integer; AInteger: integer);
    procedure SetAsInteger(AInteger: integer);
    procedure SetAssociate(const Value: TComponent);
    procedure SetAsStrArray(index: integer; AString: string);
    procedure SetAsString(const AString: string);
    procedure SetAsTime(ATime: TDateTime);
    procedure SetBooleanType(Def: boolean);
    procedure SetBoolStr(const AValue: TBoolStr);
    procedure SetColorType(Def: boolean);
    procedure SetDateTimeType(Def: boolean);
    procedure SetDateType(Def: boolean);
    procedure SetDefAsBoolean(ABoolean: boolean);
    procedure SetDefAsColor(AColor: TColor);
    procedure SetDefAsDate(ADate: TDateTime);
    procedure SetDefAsDateTime(ADateTime: TDateTime);
    procedure SetDefAsFlags(index: integer; ABoolean: boolean);
    procedure SetDefAsFloat(AFloat: double);
    procedure SetDefAsFloatArray(index: integer; AFloat: double);
    procedure SetDefAsFont(AFont: TFont);
    procedure SetDefAsIntArray(index: integer; AInteger: integer);
    procedure SetDefAsInteger(AInteger: integer);
    procedure SetDefAsStrArray(index: integer; AString: string);
    procedure SetDefAsString(const AString: string);
    procedure SetDefAsTime(ATime: TDateTime);
    procedure SetDefValue(const Value: TDefValue);
    procedure SetFlagsType(Def: boolean);
    procedure SetFloatMask(const AValue: string);
    procedure SetFloatArrayType(Def: boolean);
    procedure SetFloatType(Def: boolean);
    procedure SetFontType(Def: boolean);
    procedure SetIndex(AValue: Integer);
    procedure SetIntArrayType(Def: boolean);
    procedure SetIntegerType(Def: boolean);
    procedure SetNameSeparator(const AValue: string);
    procedure SetOptions(const AValue: TOptions);
    procedure SetOptionType(T: TOptionType);
    procedure SetParentArraySeparator(const AValue: boolean);
    procedure SetParentBoolStr(const AValue: boolean);
    procedure SetParentFloatMask(const AValue: boolean);
    procedure SetParentNameSeparator(const AValue: boolean);
    procedure SetParentStoreMode(b: boolean);
    procedure SetStoreMode(S: TStoreMode);
    procedure SetStrArrayType(Def: boolean);
    procedure SetStringType(Def: boolean);
    procedure SetTimeType(Def: boolean);
    procedure SetValue(const AValue: string);
  protected
    procedure Load(ARoot: HKEY; AKey: string; Target: TTarget; MemIni: TStrings);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Save(ARoot: HKEY; AKey: string; Target: TTarget; MemIni: TStrings);
    procedure SetDefault;
    procedure SetParentComponent(AParent: TComponent); override;
    procedure Update;

    property Value: string read FValue write SetValue;
  public
    function AcceptComponent(Component: TComponent): boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GetData;
    function GetParentComponent: TComponent; override;
    function HasParent: boolean; override;
    procedure SetData;

    property AsBoolean: boolean read GetAsBoolean write SetAsBoolean;
    property AsColor: TColor read GetAsColor write SetAsColor;
    property AsDate: TDateTime read GetAsDate write SetAsDate;
    property AsDateTime: TDateTime read GetAsDateTime write SetAsDateTime;
    property AsFlags[index: integer]: boolean read GetAsFlags write SetAsFlags;
    property AsFloat: double read GetAsFloat write SetAsFloat;
    property AsFloatArray[index: integer]: double read GetAsFloatArray write SetAsFloatArray;
    property AsFont: TFont read GetAsFont write SetAsFont;
    property AsIntArray[index: integer]: integer read GetAsIntArray write SetAsIntArray;
    property AsInteger: integer read GetAsInteger write SetAsInteger;
    property AsStrArray[index: integer]: string read GetAsStrArray write SetAsStrArray;
    property AsString: string read GetAsString write SetAsString;
    property AsTime: TDateTime read GetAsTime write SetAsTime;
    property Count: integer read GetCount;
    property DefAsBoolean: boolean read GetDefAsBoolean write SetDefAsBoolean;
    property DefAsColor: TColor read GetDefAsColor write SetDefAsColor;
    property DefAsDate: TDateTime read GetDefAsDate write SetDefAsDate;
    property DefAsDateTime: TDateTime read GetDefAsDateTime write SetDefAsDateTime;
    property DefAsFlags[index: integer]: boolean read GetDefAsFlags write SetDefAsFlags;
    property DefAsFloat: double read GetDefAsFloat write SetDefAsFloat;
    property DefAsFloatArray[index: integer]: double read GetDefAsFloatArray write SetDefAsFloatArray;
    property DefAsFont: TFont read GetDefAsFont write SetDefAsFont;
    property DefAsIntArray[index: integer]: integer read GetDefAsIntArray write SetDefAsIntArray;
    property DefAsInteger: integer read GetDefAsInteger write SetDefAsInteger;
    property DefAsStrArray[index: integer]: string read GetDefAsStrArray write SetDefAsStrArray;
    property DefAsString: string read GetDefAsString write SetDefAsString;
    property DefAsTime: TDateTime read GetDefAsTime write SetDefAsTime;
    property DefCount: integer read GetDefCount;
    property Index: Integer read GetIndex write SetIndex stored false;
    property Options: TOptions read FOptions write SetOptions stored false;
  published
    property Associate: TComponent read FAssociate write SetAssociate;
    property ArraySeparator: string read FArraySeparator write SetArraySeparator;
    property BoolStr: TBoolStr read FBoolStr write SetBoolStr default bsTrue;
    property Category: TCategory read FCategory write FCategory;
    property DefValue: TDefValue read FDefValue write SetDefValue;
    property FloatMask: string read FFloatMask write SetFloatMask;
    property NameSeparator: string read FNameSeparator write SetNameSeparator;
    property OptionType: TOptionType read FOptionType write SetOptionType default otString;
    property ParentArraySeparator: boolean read FParentArraySeparator write SetParentArraySeparator;
    property ParentBoolStr: boolean read FParentBoolStr write SetParentBoolStr default true;
    property ParentFloatMask: boolean read FParentFloatMask write SetParentFloatMask default true;
    property ParentNameSeparator: boolean read FParentNameSeparator write SetParentNameSeparator default true;
    property ParentStoreMode: boolean read FParentStoreMode write SetParentStoreMode default true;
    property StoreMode: TStoreMode read FStoreMode write SetStoreMode default stmStrArray;

    property OnUpdated: TNotifyEvent read FOnUpdated write FOnUpdated;
  end;

  TBeforeStoreEvent = procedure (Sender: TObject; Item: TOption; var Accept: boolean) of object;

  TOptionEvent = procedure(Sender: TObject; Item: TOption) of object;

  TOptions = class(TComponent)
  private
    FArraySeparator: string;
    FAutoLoad: boolean;
    FBoolStr: TBoolStr;
    FFloatMask: string;
    FIniFileDir: string;
    FIniFileLoc: TDirLocation;
    FIniFileName: string;
    FMemIni: TStrings;
    FNameSeparator: string;
    FOptionList: TList;
    FRegKey: string;
    FRootKey: HKEY;
    FSaveOnFree: boolean;
    FStoreMode: TStoreMode;
    FTarget: TTarget;

    FAfterLoad: TNotifyEvent;
    FAfterSave: TNotifyEvent;
    FBeforeLoad: TNotifyEvent;
    FBeforeSave: TNotifyEvent;
    FOnLoad: TBeforeStoreEvent;
    FOnSave: TBeforeStoreEvent;
    FOnLoaded: TOptionEvent;
    FOnSaved: TOptionEvent;
    FSaveOnUpdate: boolean;
    FOnUpdated: TOptionEvent;

    function GetCount: integer;
    function GetFileName: string;
    function GetItems(index: integer): TOption;
    function GetOtherDir(L: TDirLocation): string;
    function GetSpecialDir(I: integer): string;
    procedure SetArraySeparator(const AValue: string);
    procedure SetBoolStr(const AValue: TBoolStr);
    procedure SetFloatMask(const AValue: string);
    procedure SetNameSeparator(const AValue: string);
    procedure SetIniFileDir(S: string);
    procedure SetIniFileLoc(L: TDirLocation);
    procedure SetStoreMode(S: TStoreMode);
  protected
    procedure AddOption(AOption: TOption);
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure RemoveOption(AOption: TOption);
    procedure SetChildOrder(Component: TComponent; Order: Integer); override;
  public
    function Add: TOption;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GetData;
    function IndexOf(AOption: TOption): integer;
    procedure Load(index: integer);
    procedure LoadAll;
    procedure LoadCat(Category: string);
    function OptByName(const AName: string): TOption;
    procedure Save(index: integer);
    procedure SaveAll;
    procedure SaveCat(Category: string);
    procedure SetData;
    procedure SetDefault(index: integer);
    procedure SetDefaults;
    procedure SetDefaultsCat(Category: string);

    property Count: integer read GetCount;
    property Items[index: integer]: TOption read GetItems; default;
    property MemIni: TStrings read FMemIni write FMemIni;
  published
    property ArraySeparator: string read FArraySeparator write SetArraySeparator;
    property AutoLoad: boolean read FAutoLoad write FAutoLoad default true;
    property BoolStr: TBoolStr read FBoolStr write SetBoolStr;
    property FloatMask: string read FFloatMask write SetFloatMask;
    property IniFileDir: string read FIniFileDir write SetIniFileDir;
    property IniFileLoc: TDirLocation read FIniFileLoc write SetIniFileLoc default dirNone;
    property IniFileName: string read FIniFileName write FIniFileName;
    property NameSeparator: string read FNameSeparator write SetNameSeparator;
    property RegKey: string read FRegKey write FRegKey;
    property RootKey: HKEY read FRootKey write FRootKey default HKEY_CURRENT_USER;
    property SaveOnFree: boolean read FSaveOnFree write FSaveOnFree default true;
    property SaveOnUpdate: boolean read FSaveOnUpdate write FSaveOnUpdate default false;
    property StoreMode: TStoreMode read FStoreMode write SetStoreMode default stmStrArray;
    property Target: TTarget read FTarget write FTarget default stReg;

    property AfterLoad: TNotifyEvent read FAfterLoad write FAfterLoad;
    property AfterSave: TNotifyEvent read FAfterSave write FAfterSave;
    property BeforeLoad: TNotifyEvent read FBeforeLoad write FBeforeLoad;
    property BeforeSave: TNotifyEvent read FBeforeSave write FBeforeSave;
    property OnLoad: TBeforeStoreEvent read FOnLoad write FOnLoad;
    property OnLoaded: TOptionEvent read FOnLoaded write FOnLoaded;
    property OnSave: TBeforeStoreEvent read FOnSave write FOnSave;
    property OnSaved: TOptionEvent read FOnSaved write FOnSaved;
    property OnUpdated: TOptionEvent read FOnUpdated write FOnUpdated;
  end;

  TMemSaver = class(TObject)
  private
    FLines: TStrings;
    function AddSection(const Section: string): integer;
    function FindIdent(const Ident: string; SectionStart: integer): integer;
    function FindSection(const Section: string): integer;
    function FindSectionEnd(const Section: string): integer;
  public
    constructor Create(ALines: TStrings; ABoolStr: TBoolStr; const AFloatMask: string);
    function ReadBool(const Section, Ident: string; Default: boolean): boolean;
    function ReadFloat(const Section, Ident: string; Default: double): double;
    function ReadInteger(const Section, Ident: string; Default: integer): integer;
    procedure ReadSection(const Section: string; Strings: TStrings);
    function ReadString(const Section, Ident, Default: string): string;
    procedure WriteBool(const Section, Ident: string; Value: boolean);
    procedure WriteFloat(const Section, Ident: string; Value: double);
    procedure WriteInteger(const Section, Ident: string; Value: integer);
    procedure WriteString(const Section, Ident, Value: string);
  end;

  TOpSaver = class(TObject)
  private
    FIni: TIniFile;
    FReg: TRegistry;
    FMem: TMemSaver;
    Option: TOption;

    procedure LIBi;
    procedure LIBA;
    procedure LISt;
    procedure LISA;

    procedure SIBi;
    procedure SIBA;
    procedure SISt;
    procedure SISA;

    procedure LRBi;
    procedure LRBA;
    procedure LRSt;
    procedure LRSA;

    procedure SRBi;
    procedure SRBA;
    procedure SRSt;
    procedure SRSA;

    procedure LTBi;
    procedure LTBA;
    procedure LTSt;
    procedure LTSA;

    procedure STBi;
    procedure STBA;
    procedure STSt;
    procedure STSA;
    
    procedure LIBiSt;
    procedure LIBiIn;
    procedure LIBiFs;
    procedure LIBiBo;
    procedure LIBiCo;
    procedure LIBiDa;
    procedure LIBiTi;
    procedure LIBiDT;
    procedure LIBiFo;
    procedure LIBiFl;
    procedure LIBiSA;
    procedure LIBiIA;
    procedure LIBiFA;

    procedure LIBASt;
    procedure LIBAIn;
    procedure LIBAFs;
    procedure LIBABo;
    procedure LIBACo;
    procedure LIBADa;
    procedure LIBATi;
    procedure LIBADT;
    procedure LIBAFo;
    procedure LIBAFl;
    procedure LIBASA;
    procedure LIBAIA;
    procedure LIBAFA;

    procedure LIStSt;
    procedure LIStIn;
    procedure LIStFs;
    procedure LIStBo;
    procedure LIStCo;
    procedure LIStDa;
    procedure LIStTi;
    procedure LIStDT;
    procedure LIStFo;
    procedure LIStFl;
    procedure LIStSA;
    procedure LIStIA;
    procedure LIStFA;

    procedure LISASt;
    procedure LISAIn;
    procedure LISAFs;
    procedure LISABo;
    procedure LISACo;
    procedure LISADa;
    procedure LISATi;
    procedure LISADT;
    procedure LISAFo;
    procedure LISAFl;
    procedure LISASA;
    procedure LISAIA;
    procedure LISAFA;

    procedure LRBiSt;
    procedure LRBiIn;
    procedure LRBiFs;
    procedure LRBiBo;
    procedure LRBiCo;
    procedure LRBiDa;
    procedure LRBiTi;
    procedure LRBiDT;
    procedure LRBiFo;
    procedure LRBiFl;
    procedure LRBiSA;
    procedure LRBiIA;
    procedure LRBiFA;

    procedure LRBASt;
    procedure LRBAIn;
    procedure LRBAFs;
    procedure LRBABo;
    procedure LRBACo;
    procedure LRBADa;
    procedure LRBATi;
    procedure LRBADT;
    procedure LRBAFo;
    procedure LRBAFl;
    procedure LRBASA;
    procedure LRBAIA;
    procedure LRBAFA;

    procedure LRStSt;
    procedure LRStIn;
    procedure LRStFs;
    procedure LRStBo;
    procedure LRStCo;
    procedure LRStDa;
    procedure LRStTi;
    procedure LRStDT;
    procedure LRStFo;
    procedure LRStFl;
    procedure LRStSA;
    procedure LRStIA;
    procedure LRStFA;

    procedure LRSASt;
    procedure LRSAIn;
    procedure LRSAFs;
    procedure LRSABo;
    procedure LRSACo;
    procedure LRSADa;
    procedure LRSATi;
    procedure LRSADT;
    procedure LRSAFo;
    procedure LRSAFl;
    procedure LRSASA;
    procedure LRSAIA;
    procedure LRSAFA;

    procedure LTBiSt;
    procedure LTBiIn;
    procedure LTBiFs;
    procedure LTBiBo;
    procedure LTBiCo;
    procedure LTBiDa;
    procedure LTBiTi;
    procedure LTBiDT;
    procedure LTBiFo;
    procedure LTBiFl;
    procedure LTBiSA;
    procedure LTBiIA;
    procedure LTBiFA;

    procedure LTBASt;
    procedure LTBAIn;
    procedure LTBAFs;
    procedure LTBABo;
    procedure LTBACo;
    procedure LTBADa;
    procedure LTBATi;
    procedure LTBADT;
    procedure LTBAFo;
    procedure LTBAFl;
    procedure LTBASA;
    procedure LTBAIA;
    procedure LTBAFA;

    procedure LTStSt;
    procedure LTStIn;
    procedure LTStFs;
    procedure LTStBo;
    procedure LTStCo;
    procedure LTStDa;
    procedure LTStTi;
    procedure LTStDT;
    procedure LTStFo;
    procedure LTStFl;
    procedure LTStSA;
    procedure LTStIA;
    procedure LTStFA;

    procedure LTSASt;
    procedure LTSAIn;
    procedure LTSAFs;
    procedure LTSABo;
    procedure LTSACo;
    procedure LTSADa;
    procedure LTSATi;
    procedure LTSADT;
    procedure LTSAFo;
    procedure LTSAFl;
    procedure LTSASA;
    procedure LTSAIA;
    procedure LTSAFA;

    procedure SIBiSt;
    procedure SIBiIn;
    procedure SIBiFs;
    procedure SIBiBo;
    procedure SIBiCo;
    procedure SIBiDa;
    procedure SIBiTi;
    procedure SIBiDT;
    procedure SIBiFo;
    procedure SIBiFl;
    procedure SIBiSA;
    procedure SIBiIA;
    procedure SIBiFA;

    procedure SIBASt;
    procedure SIBAIn;
    procedure SIBAFs;
    procedure SIBABo;
    procedure SIBACo;
    procedure SIBADa;
    procedure SIBATi;
    procedure SIBADT;
    procedure SIBAFo;
    procedure SIBAFl;
    procedure SIBASA;
    procedure SIBAIA;
    procedure SIBAFA;

    procedure SIStSt;
    procedure SIStIn;
    procedure SIStFs;
    procedure SIStBo;
    procedure SIStCo;
    procedure SIStDa;
    procedure SIStTi;
    procedure SIStDT;
    procedure SIStFo;
    procedure SIStFl;
    procedure SIStSA;
    procedure SIStIA;
    procedure SIStFA;

    procedure SISASt;
    procedure SISAIn;
    procedure SISAFs;
    procedure SISABo;
    procedure SISACo;
    procedure SISADa;
    procedure SISATi;
    procedure SISADT;
    procedure SISAFo;
    procedure SISAFl;
    procedure SISASA;
    procedure SISAIA;
    procedure SISAFA;

    procedure SRBiSt;
    procedure SRBiIn;
    procedure SRBiFs;
    procedure SRBiBo;
    procedure SRBiCo;
    procedure SRBiDa;
    procedure SRBiTi;
    procedure SRBiDT;
    procedure SRBiFo;
    procedure SRBiFl;
    procedure SRBiSA;
    procedure SRBiIA;
    procedure SRBiFA;

    procedure SRBASt;
    procedure SRBAIn;
    procedure SRBAFs;
    procedure SRBABo;
    procedure SRBACo;
    procedure SRBADa;
    procedure SRBATi;
    procedure SRBADT;
    procedure SRBAFo;
    procedure SRBAFl;
    procedure SRBASA;
    procedure SRBAIA;
    procedure SRBAFA;

    procedure SRStSt;
    procedure SRStIn;
    procedure SRStFs;
    procedure SRStBo;
    procedure SRStCo;
    procedure SRStDa;
    procedure SRStTi;
    procedure SRStDT;
    procedure SRStFo;
    procedure SRStFl;
    procedure SRStSA;
    procedure SRStIA;
    procedure SRStFA;

    procedure SRSASt;
    procedure SRSAIn;
    procedure SRSAFs;
    procedure SRSABo;
    procedure SRSACo;
    procedure SRSADa;
    procedure SRSATi;
    procedure SRSADT;
    procedure SRSAFo;
    procedure SRSAFl;
    procedure SRSASA;
    procedure SRSAIA;
    procedure SRSAFA;

    procedure STBiSt;
    procedure STBiIn;
    procedure STBiFs;
    procedure STBiBo;
    procedure STBiCo;
    procedure STBiDa;
    procedure STBiTi;
    procedure STBiDT;
    procedure STBiFo;
    procedure STBiFl;
    procedure STBiSA;
    procedure STBiIA;
    procedure STBiFA;

    procedure STBASt;
    procedure STBAIn;
    procedure STBAFs;
    procedure STBABo;
    procedure STBACo;
    procedure STBADa;
    procedure STBATi;
    procedure STBADT;
    procedure STBAFo;
    procedure STBAFl;
    procedure STBASA;
    procedure STBAIA;
    procedure STBAFA;

    procedure STStSt;
    procedure STStIn;
    procedure STStFs;
    procedure STStBo;
    procedure STStCo;
    procedure STStDa;
    procedure STStTi;
    procedure STStDT;
    procedure STStFo;
    procedure STStFl;
    procedure STStSA;
    procedure STStIA;
    procedure STStFA;

    procedure STSASt;
    procedure STSAIn;
    procedure STSAFs;
    procedure STSABo;
    procedure STSACo;
    procedure STSADa;
    procedure STSATi;
    procedure STSADT;
    procedure STSAFo;
    procedure STSAFl;
    procedure STSASA;
    procedure STSAIA;
    procedure STSAFA;

    function CheckFloat(const s: string): boolean;
    function CheckInt(const s: string): boolean;
    function IName(i: integer): string;
    function NameI(const s: string): integer;
    function ReadFlags(const s: string): string;
    procedure ReadFont(const s: string; F: TFont);
    function SName(const s: string): string;
    function WriteFont(F: TFont; SM: boolean; const Sep: string): string;
  protected
    procedure LoadIni(const AIniName: string);
    procedure LoadMem(MemIni: TStrings);
    procedure LoadReg(ARoot: HKEY; const ARegKey: string);
    procedure SaveIni(const AIniName: string);
    procedure SaveMem(MemIni: TStrings);
    procedure SaveReg(ARoot: HKEY; const ARegKey: string);
  public
    constructor Create(AOption: TOption); reintroduce;
  end;

  EOptError = class(Exception)
  private
    FErrorCode: integer;
  public
    constructor Create(Code: integer);

    property ErrorCode: integer read FErrorCode;
  end;

const
  oerUnknownError                 = 1;
  oerInvalidName                  = 2;
  oerNameAlreadyExists            = 3;
  oerNotInteger                   = 4;
  oerNotColor                     = 5;
  oerNotFloat                     = 6;
  oerNotDateTime                  = 7;
  oerNotFont                      = 8;
  oerIndexOutOfBounds             = 9;
  oerRegLoad                      = 10;

  procedure Register;

implementation

uses
  ShlObj, StdCtrls, ComCtrls, ExtCtrls, Spin, Mask, Buttons, CheckLst, Menus;

{$R *.res}

procedure Register;
begin
  RegisterClasses([TOptions, TOption]);
  RegisterNoIcon([TOption]);
  RegisterComponents('Options', [TOptions]);
end;

const
  strUnknownError                 = 'Unknown error (code: %d)';
  strInvalidName                  = 'Invalid option name';
  strNameAlreadyExists            = 'Option already exists with that name';
  strNotInteger                   = 'Option value is not a valid integer';
  strNotColor                     = 'Option value is not a valid color';
  strNotFloat                     = 'Option value is not a valid float';
  strNotDateTime                  = 'Option value is not a valid date and/or time';
  strNotFont                      = 'Option value is not a valid font';
  strIndexOutOfBounds             = 'Array index is out of bounds';
  strRegLoad                      = 'Error loading option value from registry';

  strDefRegKey                    = '\software\Company\Project';
  strOption                       = 'Option';
  strDefMask                      = '0.##';
  strFalse                        = 'False';
  strFalseUpper                   = 'FALSE';
  strFalseLower                   = 'false';
  strTrue                         = 'True';
  strTrueUpper                    = 'TRUE';
  strTrueLower                    = 'true';
  strOn                           = 'On';
  strOnUpper                      = 'ON';
  strOnLower                      = 'on';
  strOff                          = 'Off';
  strOffUpper                     = 'OFF';
  strOffLower                     = 'off';
  strDefFont                      = '1'#$0D#$0A'-2147483640'#$0D#$0A'MS Sans Serif'#$0D#$0A'0'#$0D#$0A'8'#$0D#$0A'0000';
  strDelim                        = ', ';
  strBold                         = 'BOLD';
  strItalic                       = 'ITALIC';
  strUnderline                    = 'UNDERLINE';
  strStrikeOut                    = 'STRIKEOUT';
  strCharset                      = 'CHARSET';
  strColor                        = 'COLOR';
  strName                         = 'NAME';
  strPitch                        = 'PITCH';
  strSize                         = 'SIZE';
  strStyle                        = 'STYLE';
  strDefault                      = 'DEFAULT';
  strFixed                        = 'FIXED';
  strVariable                     = 'VARIABLE';
  strCRLF                         = #13#10;

{ Common procedures and functions }

function BoolTxt(const BoolStr: TBoolStr; const ABool: boolean): string;
const
  BStr: array[TBoolStr, boolean] of string =
    ((strFalse, strTrue),
    (strFalseUpper, strTrueUpper),
    (strFalseLower, strTrueLower),
    (strOff, strOn),
    (strOffUpper, strOnUpper),
    (strOffLower, strOnLower),
    ('0', '1'));
begin
  result := BStr[BoolStr, ABool];
end;

function GCount(s: string): integer;
var
  L: TStringList;
begin
  L := TStringList.Create;
  try
    L.BeginUpdate;
    L.Text := s;
    L.EndUpdate;
    result := L.Count;
  finally
    L.Free;
  end;
end;

function GetBoolean(const s: string): boolean;
begin
  result := (s <> '') and ((s[1] = '1') or (AnsiUpperCase(Copy(s, 1, 2)) = strOnUpper) or
      (AnsiUpperCase(Copy(s, 1, 4)) = strTrueUpper));
end;

function GetColor(const s: string): TColor;
var
  ok: boolean;
begin
  ok := true;
  result := clBlack;
  try
    if s <> '' then
      result := StringToColor(s)
  except
    ok := false;
  end;
  if not ok then
  try
    result := StrToInt(s);
  except
    raise EOptError.Create(oerNotColor);
  end;
end;

function GetDate(const s: string): TDateTime;
begin
  try
    if s = '' then
      result := 0
    else
      result := StrToDate(s);
  except
    raise EOptError.Create(oerNotDateTime);
  end;
end;

function GetDateTime(const s: string): TDateTime;
begin
  try
    if s = '' then
      result := 0
    else
      result := StrToDateTime(s);
  except
    raise EOptError.Create(oerNotDateTime);
  end;
end;

function GetFlags(const s: string; index: integer): boolean;
begin
  if (index < 0) or (index >= Length(s)) then
    raise EOptError.Create(oerIndexOutOfBounds);
  result := s[index + 1] = '1';
end;

function GetFloat(const s: string): double;
begin
  try
    if s = '' then
      result := 0
    else
      result := StrToFloat(s);
  except
    raise EOptError.Create(oerNotFloat);
  end;
end;

function GetFloatArray(const s: string; index: integer): double;
var
  SL: TStringList;
  t: string;
begin
  if index < 0 then
    raise EOptError.Create(oerIndexOutOfBounds);
  t := '';
  SL := TStringList.Create;
  try
    try
      SL.BeginUpdate;
      SL.Text := s;
      SL.EndUpdate;
      t := SL[index];
    finally
      SL.Free;
    end;
  except
    raise EOptError(oerIndexOutOfBounds);
  end;
  try
    if t = '' then
      result := 0
    else
      result := StrToFloat(t);
  except
    raise EOptError(oerNotFloat);
  end;
end;

function GetFont(s: string): TFont;
var
  SL: TStringList;
begin
  result := TFont.Create;
  SL := TStringList.Create;
  try
    try
      SL.BeginUpdate;
      SL.Text := s;
      SL.EndUpdate;
      result.Charset := StrToInt(SL[0]);
      result.Color := StrToInt(SL[1]);
      result.Name := SL[2];
      result.Pitch := TFontPitch(StrToInt(SL[3]));
      result.Size := StrToInt(SL[4]);
      result.Style := [];
      if SL[5][1] = '1' then
        result.Style := result.Style + [fsBold];
      if SL[5][2] = '1' then
        result.Style := result.Style + [fsItalic];
      if SL[5][3] = '1' then
        result.Style := result.Style + [fsUnderline];
      if SL[5][4] = '1' then
        result.Style := result.Style + [fsStrikeOut];
    except
      raise EOptError.Create(oerNotFont);
    end;
  finally
    SL.Free;
  end;
end;

function GetFontStyle(st: TFontStyles): string;
begin
  result := '0000';
  if fsBold in st then
    result[1] := '1';
  if fsItalic in st then
    result[2] := '1';
  if fsUnderline in st then
    result[3] := '1';
  if fsStrikeOut in st then
    result[4] := '1';
end;

function GetIntArray(const s: string; index: integer): integer;
var
  SL: TStringList;
  t: string;
begin
  if index < 0 then
    raise EOptError.Create(oerIndexOutOfBounds);
  t := '';
  SL := TStringList.Create;
  try
    try
      SL.BeginUpdate;
      SL.Text := s;
      SL.EndUpdate;
      t := SL[index];
    finally
      SL.Free;
    end;
  except
    raise EOptError(oerIndexOutOfBounds);
  end;
  try
    if t = '' then
      result := 0
    else
      result := StrToInt(t);
  except
    raise EOptError(oerNotInteger);
  end;
end;

function GetInteger(const s: string): integer;
begin
  try
    if s = '' then
      result := 0
    else
      result := StrToInt(s);
  except
    raise EOptError.Create(oerNotInteger);
  end;
end;

function GetStrArray(const s: string; index: integer): string;
var
  SL: TStringList;
begin
  if index < 0 then
    raise EOptError.Create(oerIndexOutOfBounds);
  SL := TStringList.Create;
  try
    try
      SL.BeginUpdate;
      SL.Text := s;
      SL.EndUpdate;
      result := SL[index];
    finally
      SL.Free;
    end;
  except
    raise EOptError.Create(oerIndexOutOfBounds);
  end;
end;

function GetTime(const s: string): TDateTime;
begin
  try
    if s = '' then
      result := 0
    else
      result := StrToTime(s);
  except
    raise EOptError.Create(oerNotDateTime);
  end;
end;

function Integer2Style(s: integer): TFontStyles;
begin
  result := [];
  if s and 1 <> 0 then
    result := result + [fsBold];
  if s and 2 <> 0 then
    result := result + [fsItalic];
  if s and 4 <> 0 then
    result := result + [fsUnderline];
  if s and 8 <> 0 then
    result := result + [fsStrikeOut];
end;

function Pitch2String(P: TFontPitch): string;
begin
  case P of
    fpFixed: result := strFixed;
    fpVariable: result := strVariable;
    else result := strDefault;
  end;
end;

function SetFlags(const s: string; index: integer; b: boolean): string;
begin
  if index < 0 then
    raise EOptError.Create(oerIndexOutOfBounds);
  result := s;
  if index >= Length(result) then
    result := result + StringOfChar('0', index - Length(result) + 1);
  if b then
    result[index + 1] := '1'
  else
    result[index + 1] := '0';
end;

function SetFloatArray(var s: string; index: integer; const m: string; f: double): string;
var
  SL: TStringList;
begin
  if index < 0 then
    raise EOptError.Create(oerIndexOutOfBounds);
  SL := TStringList.Create;
  try
    SL.BeginUpdate;
    SL.Text := s;
    SL.EndUpdate;
    while SL.Count <= index do
      SL.Add('0');
    SL.BeginUpdate;
    SL[index] := FormatFloat(m, f);
    SL.EndUpdate;
    result := SL.Text;
  finally
    SL.Free;
  end;
end;

function SetFont(f: TFont): string;
var
  SL: TStringList;
begin
  SL := TStringList.Create;
  try
    SL.Add(IntToStr(f.Charset));
    SL.Add(IntToStr(f.Color));
    SL.Add(f.Name);
    SL.Add(IntToStr(Ord(f.Pitch)));
    SL.Add(IntToStr(f.Size));
    SL.Add(GetFontStyle(f.Style));
    result := SL.Text;
  finally
    SL.Free;
  end;
end;

function SetIntArray(var s: string; index: integer; i: integer): string;
var
  SL: TStringList;
begin
  if index < 0 then
    raise EOptError.Create(oerIndexOutOfBounds);
  SL := TStringList.Create;
  try
    SL.BeginUpdate;
    SL.Text := s;
    SL.EndUpdate;
    while SL.Count <= index do
      SL.Add('0');
    SL.BeginUpdate;
    SL[index] := IntToStr(i);
    SL.EndUpdate;
    result := SL.Text;
  finally
    SL.Free;
  end;
end;

function SetStrArray(const s: string; index: integer; str: string): string;
var
  SL: TStringList;
begin
  if index < 0 then
    raise EOptError.Create(oerIndexOutOfBounds);
  SL := TStringList.Create;
  try
    SL.BeginUpdate;
    SL.Text := s;
    SL.EndUpdate;
    while SL.Count <= index do
      SL.Add('');
    SL.BeginUpdate;
    SL[index] := str;
    SL.EndUpdate;
    result := SL.Text;
  finally
    SL.Free;
  end;
end;

function String2Color(const s: string): TColor;
var
  ok: boolean;
  c: TColor;
begin
  c := clBlack;
  ok := true;
  try
    c := StrToInt(s);
  except
    ok := false;
  end;
  if ok then
    result := c
  else
    result := StringToColor(s);
end;

function String2Pitch(const s: string): TFontPitch;
var
  Temp: string;
begin
  Temp := AnsiUpperCase(s);
  if Temp = strDefault then
    result := fpDefault
  else if Temp = strFixed then
    result := fpFixed
  else if Temp = strVariable then
    result := fpVariable
  else
    result := TFontPitch(StrToInt(s));
end;

function String2Style(const s: string): TFontStyles;
var
  Temp: string;
begin
  result := [];
  Temp := AnsiUpperCase(s);
  if (Temp <> '') and (Temp[1] = '[') and (Temp[Length(Temp)] = ']') then begin
    if Pos(strBold, Temp) <> 0 then
      result := result + [fsBold];
    if Pos(strItalic, Temp) <> 0 then
      result := result + [fsItalic];
    if Pos(strUnderline, Temp) <> 0 then
      result := result + [fsUnderline];
    if Pos(strStrikeOut, Temp) <> 0 then
      result := result + [fsStrikeOut];
  end else if Length(Temp) = 4 then begin
    if Temp[1] = '1' then
      result := result + [fsBold];
    if Temp[2] = '1' then
      result := result + [fsItalic];
    if Temp[3] = '1' then
      result := result + [fsUnderline];
    if Temp[4] = '1' then
      result := result + [fsStrikeOut];
  end else
    raise EOptError.Create(oerNotFont);
end;

function Style2Integer(S: TFontStyles): integer;
begin
  result := 0;
  if fsBold in S then
    Inc(result);
  if fsItalic in S then
    Inc(result, 2);
  if fsUnderline in S then
    Inc(result, 4);
  if fsStrikeOut in S then
    Inc(result, 8);
end;

function Style2String(S: TFontStyles; SM: boolean): string;
begin
  if SM then begin
    result := '[';
    if fsBold in S then
      result := result + strBold + strDelim;
    if fsItalic in S then
      result := result + strItalic + strDelim;
    if fsUnderline in S then
      result := result + strUnderline + strDelim;
    if fsStrikeOut in S then
      result := result + strStrikeOut;
    if (Length(result) >= 3) and (Copy(result, Length(result) - 1, 2) = strDelim) then
      Delete(result, Length(result) - 1, 2);
    result := result + ']';
  end else begin
    result := StringOfChar('0', 4);
    if fsBold in S then
      result[1] := '1';
    if fsItalic in S then
      result[2] := '1';
    if fsUnderline in S then
      result[3] := '1';
    if fsStrikeOut in S then
      result[4] := '1';
  end;
end;

{ TOptions }

function TOptions.GetCount: integer;
begin
  result := FOptionList.Count;
end;

function TOptions.GetFileName: string;
begin
  case Target of
    stIni: begin
      if IniFileDir[Length(IniFileDir)] <> '\' then
        result := '\'
      else
        result := '';
      result := IniFileDir + result + IniFileName;
    end;
    stReg: result := RegKey;
    else
      result := '';
  end;
end;

function TOptions.GetItems(index: integer): TOption;
begin
  result := TOption(FOptionList.Items[index]);
end;

function TOptions.GetOtherDir(L: TDirLocation): string;
var
  OS :TOSVersionInfo;
  OK: boolean;
  Path: PChar;
const
  rkOSInfo95 = 'SOFTWARE\Microsoft\Windows\CurrentVersion';
  rkOSInfoNT = 'SOFTWARE\Microsoft\Windows NT\CurrentVersion';
  rvVersionName95 = 'Version';
  rvVersionNameNT = 'CurrentType';
begin
  ZeroMemory(@OS,SizeOf(OS));
  OS.dwOSVersionInfoSize:=SizeOf(OS);
  GetVersionEx(OS);
  with TRegistry.Create do begin
    RootKey := HKEY_LOCAL_MACHINE;
    if OS.dwPlatformId = VER_PLATFORM_WIN32_NT then
      OK:=OpenKey(rkOSInfoNT,False)
    else
      OK:=OpenKey(rkOSInfo95,False);
    if OK then begin
      Path := StrAlloc(MAX_PATH);
      case L of
        dirCommonFiles: result := ReadString('CommonFilesDir');
        dirProgramFiles: result := ReadString('ProgramFilesDir');
        dirDevice: result := ReadString('DevicePath');
        dirOtherDevice: result := ReadString('OtherDevicePath');
        dirMedia: result := ReadString('MediaPath');
        dirConfig: result := ReadString('ConfigPath');
        dirWallpaper: result := ReadString('WallpaperDir');
        dirWindows: begin
          GetWindowsDirectory(Path, MAX_PATH);
          result := StrPas(Path);
        end;
        dirSystem: begin
          GetSystemDirectory(Path, MAX_PATH);
          result := StrPas(Path);
        end;
        dirTemp: begin
          GetTempPath(MAX_PATH, Path);
          result := StrPas(Path);
        end;
      else
        result := '';
      end;
      StrDispose(Path);
    end;
    Free;
  end;
end;

function TOptions.GetSpecialDir(I: integer): string;
var
  PIDL: PItemIDList;
  Path: LPSTR;
begin
  Result:='';
  Path:=StrAlloc(MAX_PATH);
  SHGetSpecialFolderLocation(GetDesktopWindow, I, PIDL);
  if SHGetPathFromIDList(PIDL, Path) then
    Result:=StrPas(Path);
  StrDispose(Path);
end;

procedure TOptions.SetArraySeparator(const AValue: string);
var
  i: integer;
begin
  if AValue <> FArraySeparator then begin
    FArraySeparator := AValue;
    for i := 0 to Count - 1 do
      if Items[i].ParentArraySeparator then
        Items[i].ArraySeparator := AValue;
  end;
end;

procedure TOptions.SetBoolStr(const AValue: TBoolStr);
var
  i: integer;
begin
  if FBoolStr <> AValue then begin
    FBoolStr := AValue;
    for i := 0 to Count - 1 do
      if Items[i].ParentBoolStr then
        Items[i].BoolStr := AValue;
  end;
end;

procedure TOptions.SetFloatMask(const AValue: string);
var
  i: integer;
begin
  if FFloatMask <> AValue then begin
    FFloatMask := AValue;
    for i := 0 to Count - 1 do
      if Items[i].ParentFloatMask then
        Items[i].FloatMask := AValue;
  end;
end;

procedure TOptions.SetNameSeparator(const AValue: string);
var
  i: integer;
begin
  if FNameSeparator <> AValue then begin
    FNameSeparator := AValue;
    for i := 0 to Count - 1 do
      if Items[i].ParentNameSeparator then
        Items[i].NameSeparator := AValue;
  end;
end;

procedure TOptions.SetIniFileDir(S: string);
begin
  FIniFileDir := S;
  FIniFileLoc := dirNone;
end;

procedure TOptions.SetIniFileLoc(L: TDirLocation);
begin
  FIniFileLoc := L;
  if L <> dirNone then
    case L of
      dirAppData: FIniFileDir := GetSpecialDir(CSIDL_APPDATA);
      dirApplication: FIniFileDir := ExtractFilePath(Application.ExeName);
      dirCommonAltStartUp: FIniFileDir := GetSpecialDir(CSIDL_COMMON_ALTSTARTUP);
      dirCommonDesktop: FIniFileDir := GetSpecialDir(CSIDL_COMMON_DESKTOPDIRECTORY);
      dirCommonFavorites: FIniFileDir := GetSpecialDir(CSIDL_COMMON_FAVORITES);
      dirCommonPrograms: FIniFileDir := GetSpecialDir(CSIDL_COMMON_PROGRAMS);
      dirCommonStartMenu: FIniFileDir := GetSpecialDir(CSIDL_COMMON_STARTMENU);
      dirCommonStartUp: FIniFileDir := GetSpecialDir(CSIDL_COMMON_STARTUP);
      dirControls: FIniFileDir := GetSpecialDir(CSIDL_CONTROLS);
      dirCookies: FIniFileDir := GetSpecialDir(CSIDL_COOKIES);
      dirDesktop: FIniFileDir := GetSpecialDir(CSIDL_DESKTOP);
      dirDesktopDir: FIniFileDir := GetSpecialDir(CSIDL_DESKTOPDIRECTORY);
      dirFavorites: FIniFileDir := GetSpecialDir(CSIDL_FAVORITES);
      dirFonts: FIniFileDir := GetSpecialDir(CSIDL_FONTS);
      dirHistory: FIniFileDir := GetSpecialDir(CSIDL_HISTORY);
      dirInternetCache: FIniFileDir := GetSpecialDir(CSIDL_INTERNET_CACHE);
      dirMyDocuments: FIniFileDir := GetSpecialDir(CSIDL_PERSONAL);
      dirNetHood: FIniFileDir := GetSpecialDir(CSIDL_NETHOOD);
      dirPrintHood: FIniFileDir := GetSpecialDir(CSIDL_PRINTHOOD);
      dirPrograms: FIniFileDir := GetSpecialDir(CSIDL_PROGRAMS);
      dirRecent: FIniFileDir := GetSpecialDir(CSIDL_RECENT);
      dirSendTo: FIniFileDir := GetSpecialDir(CSIDL_SENDTO);
      dirStartMenu: FIniFileDir := GetSpecialDir(CSIDL_STARTMENU);
      dirStartUp: FIniFileDir := GetSpecialDir(CSIDL_STARTUP);
      dirTemplates: FIniFileDir := GetSpecialDir(CSIDL_TEMPLATES);
      else
        FIniFileDir := GetOtherDir(L);
  end else
    FIniFileDir := '';
end;

procedure TOptions.SetStoreMode(S: TStoreMode);
var
  i: integer;
begin
  FStoreMode := S;
  for i := 0 to Count - 1 do
    with Items[i] do
      if ParentStoreMode then
        StoreMode := S;
end;

procedure TOptions.AddOption(AOption: TOption);
begin
  FOptionList.Add(AOption);
  AOption.FOptions := Self;
  if AOption.ParentArraySeparator then
    AOption.ArraySeparator := Self.ArraySeparator;
  if AOption.ParentStoreMode then
    AOption.StoreMode := Self.StoreMode;
end;

procedure TOptions.GetChildren(Proc: TGetChildProc; Root: TComponent);
var
  i: Integer;
  Option: TOption;
begin
  for i := 0 to Count - 1 do begin
    Option := Items[i] as TOption;
    if Option.Owner = Root then
      Proc(Option);
  end;
end;

procedure TOptions.Loaded;
begin
  inherited Loaded;
  if Owner <> nil then
    Owner.FreeNotification(Self);
  SetDefaults;
  if AutoLoad then
    LoadAll;
end;

procedure TOptions.Notification(AComponent: TComponent;
  Operation: TOperation);
var
  i: integer;
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = Owner) and (Operation = opRemove) then begin
    if SaveOnFree then
      SaveAll;
    for i := 0 to Count - 1 do
      (Items[i] as TOption).Associate := nil;
  end;
end;

procedure TOptions.RemoveOption(AOption: TOption);
begin
  FOptionList.Remove(AOption);
  AOption.FOptions := nil;
end;

procedure TOptions.SetChildOrder(Component: TComponent; Order: Integer);
begin
  if FOptionList.IndexOf(Component) >= 0 then
    (Component as TOption).Index := Order;
end;

function TOptions.Add: TOption;
begin
  result := TOption.Create(Owner);
  result.Options := Self;
  FOptionList.Add(result);
end;

constructor TOptions.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOptionList := TList.Create;
  FMemIni := TStringList.Create;
  FArraySeparator := '||';
  FAutoLoad := true;
  FBoolStr := bsTrue;
  FFloatMask := '0.##';
  FIniFileLoc := dirNone;
  FIniFileDir := '';
  FNameSeparator := '_';
  FRegKey := strDefRegKey;
  FRootKey := HKEY_CURRENT_USER;
  FSaveOnFree := true;
  FSaveOnUpdate := false;
  FStoreMode := stmStrArray;
  FTarget := stReg;
end;

destructor TOptions.Destroy;
begin
  while Count > 0 do
    Items[0].Free;
  FOptionList.Free;
  FMemIni.Free;
  inherited Destroy;
end;

procedure TOptions.GetData;
var
  i: integer;
begin
  for i := 0 to Count - 1 do
    if Items[i].Associate <> nil then
      Items[i].GetData;
end;

function TOptions.IndexOf(AOption: TOption): integer;
begin
  result := FOptionList.IndexOf(AOption);
end;

procedure TOptions.Load(index: integer);
var
  Accept: boolean;
begin
  if (index < 0) or (index >= Count) then
    raise EOptError.Create(oerIndexOutOfBounds);
  Accept := true;
  if Assigned(FOnLoad) then
    FOnLoad(Self, Items[index], Accept);
  if Accept then begin
    Items[index].Load(RootKey, GetFileName, Target, MemIni);
    if Assigned(FOnLoaded) then
      FOnLoaded(Self, Items[index]);
  end;
end;

procedure TOptions.LoadAll;
var
  i: integer;
  Accept: boolean;
begin
  if Assigned(FBeforeLoad) then
    FBeforeLoad(Self);
  for i := 0 to Count - 1 do begin
    Accept := true;
    if Assigned(FOnLoad) then
      FOnLoad(Self, Items[i], Accept);
    if Accept then begin
      Items[i].Load(RootKey, GetFileName, Target, MemIni);
      if Assigned(FOnLoaded) then
        FOnLoaded(Self, Items[i]);
    end;
  end;
  if Assigned(FAfterLoad) then
    FAfterLoad(Self);
end;

procedure TOptions.LoadCat(Category: string);
var
  i: integer;
  Accept: boolean;
begin
  if Assigned(FBeforeLoad) then
    FBeforeLoad(Self);
  Category := AnsiUpperCase(Category);
  for i := 0 to Count - 1 do
    if AnsiUpperCase(Items[i].Category) = Category then begin
      Accept := true;
      if Assigned(FOnLoad) then
        FOnLoad(Self, Items[i], Accept);
      if Accept then begin
        Items[i].Load(RootKey, GetFileName, Target, MemIni);
        if Assigned(FOnLoaded) then
          FOnLoaded(Self, Items[i]);
      end;
    end;
  if Assigned(FAfterLoad) then
    FAfterLoad(Self);
end;

function TOptions.OptByName(const AName: string): TOption;
var
  i: integer;
begin
  result := nil;
  i := 0;
  while (i < Count) and (TOption(Items[i]).Name <> AName) do
    Inc(i);
  if i < Count then
    result := TOption(Items[i]);
end;

procedure TOptions.Save(index: integer);
var
  Accept: boolean;
begin
  if (index < 0) or (index >= Count) then
    raise EOptError.Create(oerIndexOutOfBounds);
  Accept := true;
  if Assigned(FOnSave) then
    FOnSave(Self, Items[index], Accept);
  if Accept then begin
    Items[index].Save(RootKey, GetFileName, Target, MemIni);
    if Assigned(FOnSaved) then
      FOnSaved(Self, Items[index]);
  end;
end;

procedure TOptions.SaveAll;
var
  i: integer;
  Accept: boolean;
begin
  if Assigned(FBeforeSave) then
    FBeforeSave(Self);
  for i := 0 to Count - 1 do begin
    Accept := true;
    if Assigned(FOnSave) then
      FOnSave(Self, Items[i], Accept);
    if Accept then begin
      Items[i].Save(RootKey, GetFileName, Target, MemIni);
      if Assigned(FOnSaved) then
        FOnSaved(Self, Items[i]);
    end;
  end;
  if Assigned(FAfterSave) then
    FAfterSave(Self);
end;

procedure TOptions.SaveCat(Category: string);
var
  i: integer;
  Accept: boolean;
begin
  if Assigned(FBeforeSave) then
    FBeforeSave(Self);
  Category := AnsiUpperCase(Category);
  for i := 0 to Count - 1 do
    if AnsiUpperCase(Items[i].Category) = Category then begin
      Accept := true;
      if Assigned(FOnSave) then
        FOnSave(Self, Items[i], Accept);
      if Accept then begin
        Items[i].Save(RootKey, GetFileName, Target, MemIni);
        if Assigned(FOnSaved) then
          FOnSaved(Self, Items[i]);
      end;
    end;
  if Assigned(FAfterSave) then
    FAfterSave(Self);
end;

procedure TOptions.SetData;
var
  i: integer;
begin
  for i := 0 to Count - 1 do
    if Items[i].Associate <> nil then
      Items[i].SetData;
end;

procedure TOptions.SetDefault(index: integer);
begin
  if (index < 0) or (index >= Count) then
    raise EOptError.Create(oerIndexOutOfBounds);
  Items[index].SetDefault;
end;

procedure TOptions.SetDefaults;
var
  i: integer;
begin
  for i := 0 to Count - 1 do
    Items[i].SetDefault;
end;

procedure TOptions.SetDefaultsCat(Category: string);
var
  i: integer;
begin
  Category := AnsiUpperCase(Category);
  for i := 0 to Count - 1 do
    if AnsiUpperCase(Items[i].Category) = Category then
      Items[i].SetDefault;
end;

{ TOption }

function TOption.GetAsBoolean: boolean;
begin
  result := GetBoolean(FValue);
end;

function TOption.GetAsColor: TColor;
begin
  result := GetColor(FValue);
end;

function TOption.GetAsDate: TDateTime;
begin
  result := GetDate(FValue);
end;

function TOption.GetAsDateTime: TDateTime;
begin
  result := GetDateTime(FValue);
end;

function TOption.GetAsFlags(index: integer): boolean;
begin
  result := GetFlags(FValue, index);
end;

function TOption.GetAsFloat: double;
begin
  result := GetFloat(FValue);
end;

function TOption.GetAsFloatArray(index: integer): double;
begin
  result := GetFloatArray(FValue, index);
end;

function TOption.GetAsFont: TFont;
begin
  result := GetFont(FValue);
end;

function TOption.GetAsIntArray(index: integer): integer;
begin
  result := GetIntArray(FValue, index);
end;

function TOption.GetAsInteger: integer;
begin
  result := GetInteger(FValue);
end;

function TOption.GetAsStrArray(index: integer): string;
begin
  result := GetStrArray(FValue, index);
end;

function TOption.GetAsString: string;
begin
  result := FValue;
end;

function TOption.GetAsTime: TDateTime;
begin
  result := GetTime(FValue);
end;

procedure TOption.GetBooleanType;
begin
  if Associate is TCheckBox then
    AsBoolean := (Associate as TCheckBox).Checked
  else if Associate is TSpeedButton then
    AsBoolean := (Associate as TSpeedButton).Down
  else if Associate is TMenuItem then
    AsBoolean := (Associate as TMenuItem).Checked;
end;

procedure TOption.GetColorType;
begin
  if Associate is TColorDialog then
    AsColor := (Associate as TColorDialog).Color;
end;

function TOption.GetCount: integer;
begin
  if OptionType in [otString..otFont] then
    result := 1
  else if OptionType = otFlags then
    result := Length(FValue)
  else
    result := GCount(FValue);
end;

procedure TOption.GetDateTimeType;
begin
  if Associate is TEdit then
    AsDateTime := GetDateTime((Associate as TEdit).Text)
  else if Associate is TMaskEdit then
    AsDateTime := GetDateTime((Associate as TMaskEdit).Text)
  else if Associate is TDateTimePicker then
    AsDateTime := (Associate as TDateTimePicker).DateTime;
end;

procedure TOption.GetDateType;
begin
  if Associate is TEdit then
    AsDate := GetDate((Associate as TEdit).Text)
  else if Associate is TMaskEdit then
    AsDate := GetDate((Associate as TMaskEdit).Text)
  else if Associate is TDateTimePicker then
    AsDate := Int((Associate as TDateTimePicker).Date);
end;

function TOption.GetDefAsBoolean: boolean;
begin
  result := GetBoolean(FDefValue);
end;

function TOption.GetDefAsColor: TColor;
begin
  result := GetColor(FDefValue);
end;

function TOption.GetDefAsDate: TDateTime;
begin
  result := GetDate(FDefValue);
end;

function TOption.GetDefAsDateTime: TDateTime;
begin
  result := GetDateTime(FDefValue);
end;

function TOption.GetDefAsFlags(index: integer): boolean;
begin
  result := GetFlags(FDefValue, index);
end;

function TOption.GetDefAsFloat: double;
begin
  result := GetFloat(FDefValue);
end;

function TOption.GetDefAsFloatArray(index: integer): double;
begin
  result := GetFloatArray(FDefValue, index);
end;

function TOption.GetDefAsFont: TFont;
begin
  result := GetFont(FDefValue);
end;

function TOption.GetDefAsIntArray(index: integer): integer;
begin
  result := GetIntArray(FDefValue, index);
end;

function TOption.GetDefAsInteger: integer;
begin
  result := GetInteger(FDefValue);
end;

function TOption.GetDefAsStrArray(index: integer): string;
begin
  result := GetStrArray(FDefValue, index);
end;

function TOption.GetDefAsString: string;
begin
  result := FDefValue;
end;

function TOption.GetDefAsTime: TDateTime;
begin
  result := GetTime(FDefValue);
end;

function TOption.GetDefCount: integer;
begin
  if OptionType in [otString..otFont] then
    result := 1
  else if OptionType = otFlags then
    result := Length(FDefValue)
  else
    result := GCount(FDefValue);
end;

procedure TOption.GetFlagsType;
var
  i, j: integer;
begin
  if (Associate is TGroupBox) or (Associate is TPanel) then begin
    i := 0;
    j := 0;
    while (i < (Associate as TWinControl).ControlCount) and (j < Count) do begin
      if (Associate as TWinControl).Controls[i] is TCheckBox then begin
        AsFlags[j] := ((Associate as TWinControl).Controls[i] as TCheckBox).Checked;
        Inc(j);
      end else if (Associate as TWinControl).Controls[i] is TSpeedButton then begin
        AsFlags[j] := ((Associate as TWinControl).Controls[i] as TSpeedButton).Down;
        Inc(j);
      end;
      Inc(i);
    end;
  end else if Associate is TCheckListBox then begin
    i := 0;
    while (i < (Associate as TCheckListBox).Items.Count) and (i < Count) do begin
      AsFlags[i] := (Associate as TCheckListBox).Checked[i];
      Inc(i);
    end;
  end;
end;

procedure TOption.GetFloatArrayType;
var
  i, j: integer;
begin
  if Associate is TMemo then begin
    i := 0;
    while (i < (Associate as TMemo).Lines.Count) and (i < Count) do begin
      AsFloatArray[i] := GetFloat((Associate as TMemo).Lines[i]);
      Inc(i);
    end;
  end else if (Associate is TGroupBox) or (Associate is TPanel) then begin
    i := 0;
    j := 0;
    while (i < (Associate as TWinControl).ControlCount) and (j < Count) do begin
      if (Associate as TWinControl).Controls[i] is TEdit then begin
        AsFloatArray[j] := GetFloat(((Associate as TWinControl).Controls[i] as TEdit).Text);
        Inc(j);
      end else if (Associate as TWinControl).Controls[i] is TMaskEdit then begin
        AsFloatArray[j] := GetFloat(((Associate as TWinControl).Controls[i] as TMaskEdit).Text);
        Inc(j);
      end;
      Inc(i);
    end;
  end;
end;

procedure TOption.GetFloatType;
begin
  if Associate is TEdit then
    AsFloat := GetFloat((Associate as TEdit).Text)
  else if Associate is TMaskEdit then
    AsFloat := GetFloat((Associate as TMaskEdit).Text);
end;

procedure TOption.GetFontType;
begin
  if Associate is TFontDialog then
    AsFont := (Associate as TFontDialog).Font;
end;

function TOption.GetIndex: Integer;
begin
  if Options <> nil then
    result := Options.FOptionList.IndexOf(Self)
  else
    result := -1;
end;

procedure TOption.GetIntArrayType;
var
  i, j: integer;
begin
  if Associate is TMemo then begin
    i := 0;
    while (i < (Associate as TMemo).Lines.Count) and (i < Count) do begin
      AsIntArray[i] := GetInteger((Associate as TMemo).Lines[i]);
      Inc(i);
    end;
  end else if (Associate is TGroupBox) or (Associate is TPanel) then begin
    i := 0;
    j := 0;
    while (i < (Associate as TWinControl).ControlCount) and (j < Count) do begin
      if (Associate as TWinControl).Controls[i] is TEdit then begin
        AsIntArray[j] := GetInteger(((Associate as TWinControl).Controls[i] as TEdit).Text);
        Inc(j);
      end else if (Associate as TWinControl).Controls[i] is TMaskEdit then begin
        AsIntArray[j] := GetInteger(((Associate as TWinControl).Controls[i] as TMaskEdit).Text);
        Inc(j);
      end;
      Inc(i);
    end;
  end;
end;

procedure TOption.GetIntegerType;
begin
  if Associate is TEdit then
    AsInteger := GetInteger((Associate as TEdit).Text)
  else if Associate is TMaskEdit then
    AsInteger := GetInteger((Associate as TMaskEdit).Text)
  else if Associate is TListBox then
    AsInteger := (Associate as TListBox).ItemIndex
  else if Associate is TRadioGroup then
    AsInteger := (Associate as TRadioGroup).ItemIndex
  else if Associate is TTrackBar then
    AsInteger := (Associate as TTrackBar).Position
  else if Associate is TUpDown then
    AsInteger := (Associate as TUpDown).Position
  else if Associate is TSpinEdit then
    AsInteger := (Associate as TSpinEdit).Value;
end;

procedure TOption.GetStrArrayType;
var
  i, j: integer;
begin
  if Associate is TMemo then
    AsString := (Associate as TMemo).Text
  else if Associate is TRichEdit then
    AsString := (Associate as TRichEdit).Text
  else if (Associate is TGroupBox) or (Associate is TPanel) then begin
    i := 0;
    j := 0;
    while (i < (Associate as TWinControl).ControlCount) and (j < Count) do begin
      if (Associate as TWinControl).Controls[i] is TEdit then begin
        AsStrArray[j] := ((Associate as TWinControl).Controls[i] as TEdit).Text;
        Inc(j);
      end else if (Associate as TWinControl).Controls[i] is TMaskEdit then begin
        AsStrArray[j] := ((Associate as TWinControl).Controls[i] as TMaskEdit).Text;
        Inc(j);
      end else if (Associate as TWinControl).Controls[i] is TComboBox then begin
        AsStrArray[j] := ((Associate as TWinControl).Controls[i] as TComboBox).Text;
        Inc(j);
      end;
      Inc(j);
    end;
  end;
end;

procedure TOption.GetStringType;
begin
  if Associate is TEdit then
    AsString := (Associate as TEdit).Text
  else if Associate is TMaskEdit then
    AsString := (Associate as TMaskEdit).Text
  else if Associate is TComboBox then
    AsString := (Associate as TComboBox).Text
  else if Associate is TMemo then
    AsString := (Associate as TMemo).Text
  else if Associate is TRichEdit then
    AsString := (Associate as TRichEdit).Text;
end;

procedure TOption.GetTimeType;
begin
  if Associate is TEdit then
    AsTime := GetTime((Associate as TEdit).Text)
  else if Associate is TMaskEdit then
    AsTime := GetTime((Associate as TMaskEdit).Text)
  else if Associate is TDateTimePicker then
    AsTime := Frac((Associate as TDateTimePicker).Time);
end;

procedure TOption.SetArraySeparator(const AValue: string);
begin
  if FArraySeparator <> AValue then begin
    FArraySeparator := AValue;
    if FArraySeparator <> Options.ArraySeparator then
      FParentArraySeparator := false;
  end;
end;

procedure TOption.SetAsBoolean(ABoolean: boolean);
begin
  Value := BoolTxt(BoolStr, ABoolean);
end;

procedure TOption.SetAsColor(AColor: TColor);
begin
  Value := ColorToString(AColor);
end;

procedure TOption.SetAsDate(ADate: TDateTime);
begin
  Value := DateToStr(ADate);
end;

procedure TOption.SetAsDateTime(ADateTime: TDateTime);
begin
  Value := DateTimeToStr(ADateTime);
end;

procedure TOption.SetAsFlags(index: integer; ABoolean: boolean);
begin
  Value := SetFlags(FValue, index, ABoolean);
end;

procedure TOption.SetAsFloat(AFloat: double);
begin
  Value := FormatFloat(FFloatMask, AFloat);
end;

procedure TOption.SetAsFloatArray(index: integer; AFloat: double);
begin
  Value := SetFloatArray(FValue, index, FloatMask, AFloat);
end;

procedure TOption.SetAsFont(AFont: TFont);
begin
  Value := SetFont(AFont);
end;

procedure TOption.SetAsIntArray(index: integer; AInteger: integer);
begin
  Value := SetIntArray(FValue, index, AInteger);
end;

procedure TOption.SetAsInteger(AInteger: integer);
begin
  Value := IntToStr(AInteger);
end;

procedure TOption.SetAssociate(const Value: TComponent);
begin
  if (Value <> Associate) and AcceptComponent(Value) then begin
    FAssociate := Value;
    if (Associate <> nil) and (csDesigning in ComponentState) then
      SetData;
  end;
end;

procedure TOption.SetAsStrArray(index: integer; AString: string);
begin
  Value := SetStrArray(FValue, index, AString);
end;

procedure TOption.SetAsString(const AString: string);
begin
  Value := AString;
end;

procedure TOption.SetAsTime(ATime: TDateTime);
begin
  Value := TimeToStr(ATime);
end;

procedure TOption.SetBooleanType(Def: boolean);
var
  B: boolean;
begin
  if Def then
    B := DefAsBoolean
  else
    B := AsBoolean;
  if Associate is TCheckBox then
    (Associate as TCheckBox).Checked := B
  else if Associate is TSpeedButton then
    (Associate as TSpeedButton).Down := B
  else if Associate is TMenuItem then
    (Associate as TMenuItem).Checked := B;
end;

procedure TOption.SetBoolStr(const AValue: TBoolStr);
begin
  if FBoolStr <> AValue then begin
    FBoolStr := AValue;
    if FBoolStr <> Options.BoolStr then
      FParentBoolStr := false;
    if OptionType = otBoolean then begin
      FValue := BoolTxt(BoolStr, AsBoolean);
      DefValue := BoolTxt(BoolStr, DefAsBoolean);
    end;
  end;
end;

procedure TOption.SetColorType(Def: boolean);
var
  C: TColor;
begin
  if Def then
    C := DefAsColor
  else
    C := AsColor;
  if Associate is TColorDialog then
    (Associate as TColorDialog).Color := C;
end;

procedure TOption.SetDateTimeType(Def: boolean);
var
  D: TDateTime;
begin
  if Def then
    D := DefAsDateTime
  else
    D := AsDateTime;
  if Associate is TEdit then
    (Associate as TEdit).Text := DateTimeToStr(D)
  else if Associate is TMaskEdit then
    (Associate as TMaskEdit).Text := DateTimeToStr(D)
  else if Associate is TDateTimePicker then
    (Associate as TDateTimePicker).DateTime := D;
end;

procedure TOption.SetDateType(Def: boolean);
var
  D: TDateTime;
begin
  if Def then
    D := DefAsDate
  else
    D := AsDate;
  if Associate is TEdit then
    (Associate as TEdit).Text := DateToStr(D)
  else if Associate is TMaskEdit then
    (Associate as TMaskEdit).Text := DateToStr(D)
  else if Associate is TDateTimePicker then
    (Associate as TDateTimePicker).Date := D;
end;

procedure TOption.SetDefAsBoolean(ABoolean: boolean);
begin
  DefValue := BoolTxt(BoolStr, ABoolean);
end;

procedure TOption.SetDefAsColor(AColor: TColor);
begin
  DefValue := ColorToString(AColor);
end;

procedure TOption.SetDefAsDate(ADate: TDateTime);
begin
  DefValue := DateToStr(ADate);
end;

procedure TOption.SetDefAsDateTime(ADateTime: TDateTime);
begin
  DefValue := DateTimeToStr(ADateTime);
end;

procedure TOption.SetDefAsFlags(index: integer; ABoolean: boolean);
begin
  DefValue := SetFlags(FDefValue, index, ABoolean);
end;

procedure TOption.SetDefAsFloat(AFloat: double);
begin
  DefValue := FormatFloat(FFloatMask, AFloat);
end;

procedure TOption.SetDefAsFloatArray(index: integer; AFloat: double);
begin
  DefValue := SetFloatArray(FDefValue, index, FloatMask, AFloat);
end;

procedure TOption.SetDefAsFont(AFont: TFont);
begin
  DefValue := SetFont(AFont);
end;

procedure TOption.SetDefAsIntArray(index: integer; AInteger: integer);
begin
  DefValue := SetIntArray(FDefValue, index, AInteger);
end;

procedure TOption.SetDefAsInteger(AInteger: integer);
begin
  DefValue := IntToStr(AInteger);
end;

procedure TOption.SetDefAsStrArray(index: integer; AString: string);
begin
  DefValue := SetStrArray(FDefValue, index, AString);
end;

procedure TOption.SetDefAsString(const AString: string);
begin
  DefValue := AString;
end;

procedure TOption.SetDefAsTime(ATime: TDateTime);
begin
  DefValue := TimeToStr(ATime);
end;

procedure TOption.SetDefValue(const Value: TDefValue);
begin
  if FDefValue <> Value then begin
    FDefValue := Value;
    if (Associate <> nil) and (csDesigning in ComponentState) then
      SetData;
  end;
end;

procedure TOption.SetFlagsType(Def: boolean);
var
  B: boolean;
  C, i, j: integer;
begin
  if Def then
    C := DefCount
  else
    C := Count;
  if (Associate is TGroupBox) or (Associate is TPanel) then begin
    i := 0;
    j := 0;
    while (i < (Associate as TWinControl).ControlCount) and (j < C) do begin
      while (i < (Associate as TWinControl).ControlCount) and
          (not (((Associate as TWinControl).Controls[i] is TCheckBox) or
          ((Associate as TWinControl).Controls[i] is TSpeedButton))) do
        Inc(i);
      if i < (Associate as TWinControl).ControlCount then begin
        if Def then
          B := DefAsFlags[j]
        else
          B := AsFlags[j];
        if (Associate as TWinControl).Controls[i] is TCheckBox then
          ((Associate as TWinControl).Controls[i] as TCheckBox).Checked := B
        else if (Associate as TWinControl).Controls[i] is TSpeedButton then
          ((Associate as TWinControl).Controls[i] as TSpeedButton).Down := B;
      end;
      Inc(i);
      Inc(j);
    end;
  end else if Associate is TCheckListBox then begin
    i := 0;
    while (i < (Associate as TCheckListBox).Items.Count) and (i < C) do begin
      if Def then
        B := DefAsFlags[i]
      else
        B := AsFlags[i];
      (Associate as TCheckListBox).Checked[i] := B;
      Inc(i);
    end;
  end;
end;

procedure TOption.SetFloatMask(const AValue: string);
begin
  if FFloatMask <> AValue then begin
    FFloatMask := Value;
    if FFloatMask <> Options.FloatMask then
      FParentFloatMask := false;
  end;
end;

procedure TOption.SetFloatArrayType(Def: boolean);
var
  F: double;
  C, i, j: integer;
begin
  if Def then
    C := DefCount
  else
    C := Count;
  if Associate is TMemo then begin
    (Associate as TMemo).Clear;
    for i := 0 to C - 1 do begin
      if Def then
        F := DefAsFloatArray[i]
      else
        F := AsFloatArray[i];
      (Associate as TMemo).Lines.Add(FormatFloat(FloatMask, F));
    end;
  end else if (Associate is TGroupBox) or (Associate is TPanel) then begin
    i := 0;
    j := 0;
    while (i < (Associate as TWinControl).ControlCount) and (j < C) do begin
      while (i < (Associate as TWinControl).ControlCount) and
          (not (((Associate as TWinControl).Controls[i] is TEdit) or
          ((Associate as TWinControl).Controls[i] is TMaskEdit))) do
        Inc(i);
      if i < (Associate as TWinControl).ControlCount then begin
        if Def then
          F := DefAsFloatArray[j]
        else
          F := AsFloatArray[j];
        if (Associate as TWinControl).Controls[i] is TEdit then
          ((Associate as TWinControl).Controls[i] as TEdit).Text := FormatFloat(FloatMask, F)
        else if (Associate as TWinControl).Controls[i] is TMaskEdit then
          ((Associate as TWinControl).Controls[i] as TMaskEdit).Text := FormatFloat(FloatMask, F);
      end;
      Inc(i);
      Inc(j);
    end;
  end;
end;

procedure TOption.SetFloatType(Def: boolean);
var
  F: double;
begin
  if Def then
    F := DefAsFloat
  else
    F := AsFloat;
  if Associate is TEdit then
    (Associate as TEdit).Text := FormatFloat(FloatMask, F)
  else if Associate is TMaskEdit then
    (Associate as TMaskEdit).Text := FormatFloat(FloatMask, F);
end;

procedure TOption.SetFontType(Def: boolean);
var
  F: TFont;
begin
  if Associate is TFontDialog then begin
    if Def then
      F := DefAsFont
    else
      F := AsFont;
    try
      (Associate as TFontDialog).Font.Assign(F);
    finally
      F.Free;
    end;
  end;
end;

procedure TOption.SetIndex(AValue: Integer);
var
  CurIndex, Count: Integer;
begin
  CurIndex := GetIndex;
  if CurIndex >= 0 then
  begin
    Count := Options.FOptionList.Count;
    if AValue < 0 then
      AValue := 0;
    if AValue >= Count then
      AValue := Count - 1;
    if AValue <> CurIndex then
    begin
      Options.FOptionList.Delete(CurIndex);
      Options.FOptionList.Insert(AValue, Self);
    end;
  end;
end;

procedure TOption.SetIntArrayType(Def: boolean);
var
  V, C, i, j: integer;
begin
  if Def then
    C := DefCount
  else
    C := Count;
  if Associate is TMemo then begin
    (Associate as TMemo).Clear;
    for i := 0 to C - 1 do begin
      if Def then
        V := DefAsIntArray[i]
      else
        V := AsIntArray[i];
      (Associate as TMemo).Lines.Add(IntToStr(V));
    end;
  end else if (Associate is TGroupBox) or (Associate is TPanel) then begin
    i := 0;
    j := 0;
    while (i < (Associate as TWinControl).ControlCount) and (j < C) do begin
      while (i < (Associate as TWinControl).ControlCount) and
          (not (((Associate as TWinControl).Controls[i] is TEdit) or
          ((Associate as TWinControl).Controls[i] is TMaskEdit))) do
        Inc(i);
      if i < (Associate as TWinControl).ControlCount then begin
        if Def then
          V := DefAsIntArray[j]
        else
          V := AsIntArray[j];
        if (Associate as TWinControl).Controls[i] is TEdit then
          ((Associate as TWinControl).Controls[i] as TEdit).Text := IntToStr(V)
        else if (Associate as TWinControl).Controls[i] is TMaskEdit then
          ((Associate as TWinControl).Controls[i] as TMaskEdit).Text := IntToStr(V);
      end;
      Inc(i);
      Inc(j);
    end;
  end;
end;

procedure TOption.SetIntegerType(Def: boolean);
var
  I: integer;
begin
  if Def then
    I := DefAsInteger
  else
    I := AsInteger;
  if Associate is TEdit then
    (Associate as TEdit).Text := IntToStr(I)
  else if Associate is TMaskEdit then
    (Associate as TMaskEdit).Text := IntToStr(I)
  else if Associate is TListBox then begin
    if I < -1 then
      I := -1;
    if I >= (Associate as TListBox).Items.Count then
      I := (Associate as TListBox).Items.Count - 1;
    (Associate as TListBox).ItemIndex := I;
  end else if Associate is TRadioGroup then begin
    if I < -1 then
      I := -1;
    if I >= (Associate as TRadioGroup).Items.Count then
      I := (Associate as TRadioGroup).Items.Count - 1;
    (Associate as TRadioGroup).ItemIndex := I;
  end else if Associate is TTrackBar then begin
    if I < (Associate as TTrackBar).Min then
      I := (Associate as TTrackBar).Min;
    if I > (Associate as TTrackBar).Max then
      I := (Associate as TTrackBar).Max;
    (Associate as TTrackBar).Position := I;
  end else if Associate is TUpDown then begin
    if I < (Associate as TUpDown).Min then
      I := (Associate as TUpDown).Min;
    if I > (Associate as TUpDown).Max then
      I := (Associate as TUpDown).Max;
    (Associate as TUpDown).Position := I;
  end else if Associate is TSpinEdit then begin
    if I < (Associate as TSpinEdit).MinValue then
      I := (Associate as TSpinEdit).MinValue;
    if I > (Associate as TSpinEdit).MaxValue then
      I := (Associate as TSpinEdit).MaxValue;
    (Associate as TSpinEdit).Value := I;
  end;
end;

procedure TOption.SetNameSeparator(const AValue: string);
begin
  if FNameSeparator <> AValue then begin
    FNameSeparator := AValue;
    if FNameSeparator <> Options.NameSeparator then
      FParentNameSeparator := false;
  end;
end;

procedure TOption.SetOptions(const AValue: TOptions);
begin
  if FOptions <> AValue then begin
    if FOptions <> nil then
      FOptions.RemoveOption(Self);
    if AValue <> nil then
      AValue.AddOption(Self);
  end;
end;

procedure TOption.SetOptionType(T: TOptionType);
begin
  if T <> FOptionType then begin
    FOptionType := T;
    if not (csReading in ComponentState) then begin
      case T of
        otString, otFlags, otStrArray, otIntArray, otFloatArray: FValue := '';
        otInteger, otFloat: FValue := '0';
        otDate: FValue := DateToStr(now);
        otTime: FValue := TimeToStr(now);
        otDateTime: FValue := DateTimeToStr(now);
        otBoolean: FValue := strFalse;
        otColor: FValue := ColorToString(clBlack);
        otFont: FValue := strDefFont;
      end;
      if not AcceptComponent(Associate) then
        Associate := nil;
      DefValue := FValue;
    end;
  Update;
  end;
end;

procedure TOption.SetParentArraySeparator(const AValue: boolean);
begin
  if FParentArraySeparator <> AValue then begin
    FParentArraySeparator := AValue;
    if AValue then
      FArraySeparator := Options.ArraySeparator;
  end;
end;

procedure TOption.SetParentBoolStr(const AValue: boolean);
begin
  if FParentBoolStr <> AValue then begin
    FParentBoolStr := AValue;
    if AValue then
      BoolStr := Options.BoolStr;
  end;
end;

procedure TOption.SetParentFloatMask(const AValue: boolean);
begin
  if FParentFloatMask <> AValue then begin
    FParentFloatMask := AValue;
    if AValue then
      FloatMask := Options.FloatMask;
  end;
end;

procedure TOption.SetParentNameSeparator(const AValue: boolean);
begin
  if FParentNameSeparator <> AValue then begin
    FParentNameSeparator := AValue;
    if AValue then
      NameSeparator := Options.NameSeparator;
  end;
end;

procedure TOption.SetParentStoreMode(b: boolean);
begin
  if b <> FParentStoreMode then begin
    FParentStoreMode := b;
    if b then
      FStoreMode := FOptions.StoreMode;
  end;
end;

procedure TOption.SetStoreMode(S: TStoreMode);
begin
  if S <> FStoreMode then begin
    FStoreMode := S;
    if Options.StoreMode <> FStoreMode then
      FParentStoreMode := false;
  end;
end;

procedure TOption.SetStrArrayType(Def: boolean);
var
  S: string;
  C, i, j: integer;
begin
  if Def then
    S := DefAsString
  else
    S := AsString;
  if Associate is TMemo then
    (Associate as TMemo).Text := S
  else if Associate is TRichEdit then
    (Associate as TRichEdit).Text := S
  else if (Associate is TGroupBox) or (Associate is TPanel) then begin
    if Def then
      C := DefCount
    else
      C := Count;
    i := 0;
    j := 0;
    while (i < (Associate as TWinControl).ControlCount) and (j < C) do begin
      while (i < (Associate as TWinControl).ControlCount) and
          (not (((Associate as TWinControl).Controls[i] is TEdit) or
          ((Associate as TWinControl).Controls[i] is TMaskEdit) or
          ((Associate as TWinControl).Controls[i] is TComboBox))) do
        Inc(i);
      if i < (Associate as TWinControl).ControlCount then begin
        if Def then
          S := DefAsStrArray[j]
        else
          S := AsStrArray[j];
        if (Associate as TWinControl).Controls[i] is TEdit then
          ((Associate as TWinControl).Controls[i] as TEdit).Text := S
        else if (Associate as TWinControl).Controls[i] is TMaskEdit then
          ((Associate as TWinControl).Controls[i] as TMaskEdit).Text := S
        else if (Associate as TWinControl).Controls[i] is TComboBox then
          ((Associate as TWinControl).Controls[i] as TComboBox).Text := S;
      end;
      Inc(i);
      Inc(j);
    end;
  end;
end;

procedure TOption.SetStringType(Def: boolean);
var
  S: string;
begin
  if Def then
    S := DefAsString
  else
    S := AsString;
  if Associate is TEdit then
    (Associate as TEdit).Text := S
  else if Associate is TMaskEdit then
    (Associate as TMaskEdit).Text := S
  else if Associate is TComboBox then
    (Associate as TComboBox).Text := S
  else if Associate is TMemo then
    (Associate as TMemo).Text := S
  else if Associate is TRichEdit then
    (Associate as TRichEdit).Text := S;
end;

procedure TOption.SetTimeType(Def: boolean);
var
  T: TDateTime;
begin
  if Def then
    T := DefAsTime
  else
    T := AsTime;
  if Associate is TEdit then
    (Associate as TEdit).Text := TimeToStr(T)
  else if Associate is TMaskEdit then
    (Associate as TMaskEdit).Text := TimeToStr(T)
  else if Associate is TDateTimePicker then
    (Associate as TDateTimePicker).Time := T;
end;

procedure TOption.SetValue(const AValue: string);
begin
  if FValue <> AValue then begin
    FValue := AValue;
    Update;
  end;
end;

procedure TOption.Load(ARoot: HKEY; AKey: string; Target: TTarget; MemIni: TStrings);
begin
  if [csReading, csDesigning, csLoading] * ComponentState = [] then
    with TOpSaver.Create(Self) do
      try
        case Target of
          stIni: LoadIni(AKey);
          stReg: LoadReg(ARoot, AKey);
          stMem: LoadMem(MemIni);
        end;
      finally
        Free;
      end;
end;

procedure TOption.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = FAssociate) and (Operation = opRemove) then
    FAssociate := nil;
end;

procedure TOption.Save(ARoot: HKEY; AKey: string; Target: TTarget; MemIni: TStrings);
begin
  if [csReading, csDesigning, csLoading] * ComponentState = [] then
    with TOpSaver.Create(Self) do
      try
        case Target of
          stIni: SaveIni(AKey);
          stReg: SaveReg(ARoot, AKey);
          stMem: SaveMem(MemIni);
        end;
      finally
        Free;
      end;
end;

procedure TOption.SetDefault;
begin
  Value := FDefValue;
end;

procedure TOption.SetParentComponent(AParent: TComponent);
begin
  if AParent is TOptions then
    Options := TOptions(AParent);
end;

procedure TOption.Update;
begin
  if not (csDesigning in ComponentState) then begin
    if Assigned(OnUpdated) then
      OnUpdated(Self);
    if Options <> nil then begin
      if Options.SaveOnUpdate then
        Options.Save(Index);
      if Assigned(Options.OnUpdated) then
        Options.OnUpdated(Options, Self);
    end;
  end;
end;

function TOption.AcceptComponent(Component: TComponent): boolean;
begin
  if Component = nil then
    result := true
  else begin
    result := false;
    case OptionType of
      otString: result := (Component is TEdit) or (Component is TMemo) or
          (Component is TComboBox) or (Component is TMaskEdit) or
          (Component is TRichEdit);
      otInteger: result := (Component is TEdit) or (Component is TMaskEdit) or
          (Component is TListBox) or (Component is TRadioGroup) or
          (Component is TTrackBar) or (Component is TUpDown) or
          (Component is TSpinEdit);
      otFloat: result := (Component is TEdit) or (Component is TMaskEdit);
      otBoolean: result := (Component is TCheckBox) or
          (Component is TSpeedButton) or (Component is TMenuItem);
      otColor: result := Component is TColorDialog;
      otDate: result := (Component is TEdit) or (Component is TMaskEdit) or
          (Component is TDateTimePicker) or (Component is TMonthCalendar);
      otTime: result := (Component is TEdit) or (Component is TMaskEdit) or
          (Component is TDateTimePicker);
      otDateTime: result := (Component is TEdit) or (Component is TMaskEdit) or
          (Component is TDateTimePicker);
      otFont: result := Component is TFontDialog;
      otFlags: result := (Component is TGroupBox) or (Component is TPanel) or
          (Component is TCheckListBox);
      otStrArray: result := (Component is TMemo) or (Component is TRichEdit) or
          (Component is TGroupBox) or (Component is TCustomPanel);
      otIntArray: result := (Component is TMemo) or (Component is TGroupBox) or
          (Component is TPanel);
      otFloatArray: result := (Component is TMemo) or (Component is TGroupBox) or
          (Component is TPanel);
    end;
  end;
end;

constructor TOption.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FArraySeparator := '||';
  FBoolStr := bsTrue;
  FCategory := '';
  FDefValue := '';
  FFloatMask := strDefMask;
  FNameSeparator := '_';
  FOptions := nil;
  FOptionType := otString;
  FParentArraySeparator := true;
  FParentBoolStr := true;
  FParentFloatMask := true;
  FParentNameSeparator := true;
  FParentStoreMode := true;
  FStoreMode := stmStrArray;
  FValue := '';
end;

destructor TOption.Destroy;
begin
  if FOptions <> nil then
    FOptions.RemoveOption(Self);
  inherited Destroy;
end;

procedure TOption.GetData;
begin
  if Associate <> nil then begin
    case OptionType of
      otString: GetStringType;
      otInteger: GetIntegerType;
      otFloat: GetFloatType;
      otBoolean: GetBooleanType;
      otColor: GetColorType;
      otDate: GetDateType;
      otTime: GetTimeType;
      otDateTime: GetDateTimeType;
      otFont: GetFontType;
      otFlags: GetFlagsType;
      otStrArray: GetStrArrayType;
      otIntArray: GetIntArrayType;
      otFloatArray: GetFloatArrayType;
    end;
  end;
end;

function TOption.GetParentComponent: TComponent;
begin
  if FOptions <> nil then
    result := FOptions
  else
    result := inherited GetParentComponent;
end;

function TOption.HasParent: boolean;
begin
  if Options <> nil then
    result := true
  else
    result := inherited HasParent;
end;

procedure TOption.SetData;
var
  Def: boolean;
begin
  if (Associate <> nil) and not (csReading in ComponentState) then begin
    Def := csDesigning in ComponentState;
    case OptionType of
      otString: SetStringType(Def);
      otInteger: SetIntegerType(Def);
      otFloat: SetFloatType(Def);
      otBoolean: SetBooleanType(Def);
      otColor: SetColorType(Def);
      otDate: SetDateType(Def);
      otTime: SetTimeType(Def);
      otDateTime: SetDateTimeType(Def);
      otFont: SetFontType(Def);
      otFlags: SetFlagsType(Def);
      otStrArray: SetStrArrayType(Def);
      otIntArray: SetIntArrayType(Def);
      otFloatArray: SetFloatArrayType(Def);
    end;
  end;
end;

{ EOptError }

constructor EOptError.Create(Code: integer);
var
  Msg: string;
begin
  case Code of
    oerInvalidName              : Msg := strInvalidName;
    oerNameAlreadyExists        : Msg := strNameAlreadyExists;
    oerNotInteger               : Msg := strNotInteger;
    oerNotColor                 : Msg := strNotColor;
    oerNotFloat                 : Msg := strNotFloat;
    oerNotDateTime              : Msg := strNotDateTime;
    oerNotFont                  : Msg := strNotFont;
    oerIndexOutOfBounds         : Msg := strIndexOutOfBounds;
    oerRegLoad                  : Msg := strRegLoad;
  else
    Msg := Format(strUnknownError, [Code]);
  end;
  inherited Create(Msg);
  FErrorCode := Code;
end;

{ TOpSaver }

function TOpSaver.CheckFloat(const s: string): boolean;
var
  i: integer;
  L: TStringList;
begin
  result := true;
  L := TStringList.Create;
  try
    L.BeginUpdate;
    L.Text := s;
    L.EndUpdate;
    i := 0;
    while result and (i < L.Count) do begin
      try
        if L[i] <> '' then
          StrToFloat(L[i]);
      except
        result := false;
      end;
      Inc(i);
    end;
  finally
    L.Free;
  end;
end;

function TOpSaver.CheckInt(const s: string): boolean;
var
  i: integer;
  L: TStringList;
begin
  result := true;
  L := TStringList.Create;
  try
    L.BeginUpdate;
    L.Text := s;
    L.EndUpdate;
    i := 0;
    while result and (i < L.Count) do begin
      try
        if L[i] <> '' then
          StrToInt(L[i]);
      except
        result := false;
      end;
      Inc(i);
    end;
  finally
    L.Free;
  end;
end;

constructor TOpSaver.Create(AOption: TOption);
begin
  inherited Create;
  Option := AOption;
end;

function TOpSaver.IName(i: integer): string;
begin
  result := Option.Name + Option.NameSeparator + IntToStr(i);
end;

procedure TOpSaver.LIBA;
begin
  case Option.OptionType of
    otString     : LIBASt;
    otInteger    : LIBAIn;
    otFloat      : LIBAFl;
    otBoolean    : LIBABo;
    otColor      : LIBACo;
    otDate       : LIBADa;
    otTime       : LIBATi;
    otDateTime   : LIBADT;
    otFont       : LIBAFo;
    otFlags      : LIBAFs;
    otStrArray   : LIBASA;
    otIntArray   : LIBAIA;
    otFloatArray : LIBAFA;
  end;
end;

procedure TOpSaver.LIBABo;
begin
  LIBiBo;
end;

procedure TOpSaver.LIBACo;
begin
  LIBiCo;
end;

procedure TOpSaver.LIBADa;
begin
  LIBiDa;
end;

procedure TOpSaver.LIBADT;
begin
  LIBiDT;
end;

procedure TOpSaver.LIBAFA;
begin
  LISAFA;
end;

procedure TOpSaver.LIBAFl;
begin
  LIBiFl;
end;

procedure TOpSaver.LIBAFo;
begin
  LISAFo;
end;

procedure TOpSaver.LIBAFs;
begin
  LISAFs;
end;

procedure TOpSaver.LIBAIA;
begin
  LISAIA;
end;

procedure TOpSaver.LIBAIn;
begin
  LIBiIn;
end;

procedure TOpSaver.LIBASA;
begin
  LISASA;
end;

procedure TOpSaver.LIBASt;
begin
  LIBiSt;
end;

procedure TOpSaver.LIBATi;
begin
  LIBiTi;
end;

procedure TOpSaver.LIBi;
begin
  case Option.OptionType of
    otString     : LIBiSt;
    otInteger    : LIBiIn;
    otFloat      : LIBiFl;
    otBoolean    : LIBiBo;
    otColor      : LIBiCo;
    otDate       : LIBiDa;
    otTime       : LIBiTi;
    otDateTime   : LIBiDT;
    otFont       : LIBiFo;
    otFlags      : LIBiFs;
    otStrArray   : LIBiSA;
    otIntArray   : LIBiIA;
    otFloatArray : LIBiFA;
  end;
end;

procedure TOpSaver.LIBiBo;
begin
  LIStBo;
end;

procedure TOpSaver.LIBiCo;
begin
  LIStCo;
end;

procedure TOpSaver.LIBiDa;
begin
  try
    Option.AsDate := FIni.ReadFloat(Option.Category, Option.Name, Option.DefAsDate);
  except
    raise EOptError.Create(oerNotDateTime);
  end;
end;

procedure TOpSaver.LIBiDT;
begin
  try
    Option.AsDateTime := FIni.ReadFloat(Option.Category, Option.Name,
        Option.DefAsDateTime);
  except
    raise EOptError.Create(oerNotDateTime);
  end;
end;

procedure TOpSaver.LIBiFA;
begin
  LIStFA;
end;

procedure TOpSaver.LIBiFl;
begin
  LIStFl;
end;

procedure TOpSaver.LIBiFo;
begin
  LiStFo;
end;

procedure TOpSaver.LIBiFs;
begin
  LiStFs;
end;

procedure TOpSaver.LIBiIA;
begin
  LIStIA;
end;

procedure TOpSaver.LIBiIn;
begin
  LIStIn
end;

procedure TOpSaver.LIBiSA;
begin
  LIStSA;
end;

procedure TOpSaver.LIBiSt;
begin
  LIStSt;
end;

procedure TOpSaver.LIBiTi;
begin
  try
    Option.AsTime := FIni.ReadFloat(Option.Category, Option.Name, Option.DefAsTime);
  except
    raise EOptError.Create(oerNotDateTime);
  end;
end;

procedure TOpSaver.LISA;
begin
  case Option.OptionType of
    otString     : LISASt;
    otInteger    : LISAIn;
    otFloat      : LISAFl;
    otBoolean    : LISABo;
    otColor      : LISACo;
    otDate       : LISADa;
    otTime       : LISATi;
    otDateTime   : LISADT;
    otFont       : LISAFo;
    otFlags      : LISAFs;
    otStrArray   : LISASA;
    otIntArray   : LISAIA;
    otFloatArray : LISAFA;
  end;
end;

procedure TOpSaver.LISABo;
begin
  LIStBo;
end;

procedure TOpSaver.LISACo;
begin
  LIStCo;
end;

procedure TOpSaver.LISADa;
begin
  LIStDa;
end;

procedure TOpSaver.LISADT;
begin
  LIStDT;
end;

procedure TOpSaver.LISAFA;
var
  L: TStringList;
  i, k: integer;
begin
  Option.SetDefault;
  L := TStringList.Create;
  try
    FIni.ReadSection(Option.Category, L);
    try
      for i := 0 to L.Count - 1 do begin
        k := NameI(L[i]);
        if k >= 0 then
          Option.AsFloatArray[k] := FIni.ReadFloat(Option.Category, L[i], 0);
      end;
    except
      raise EOptError.Create(oerNotFloat);
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LISAFl;
begin
  LIStFl;
end;

procedure TOpSaver.LISAFo;
var
  F: TFont;
begin
  F := Option.DefAsFont;
  try
    try
      F.Charset := FIni.ReadInteger(Option.Category, SName(strCharset), F.Charset);
      F.Color := String2Color(FIni.ReadString(Option.Category, SName(strColor),
          IntToStr(F.Color)));
      F.Name := FIni.ReadString(Option.Category, SName(strName), F.Name);
      F.Pitch := String2Pitch(FIni.ReadString(Option.Category, SName(strPitch),
          IntToStr(integer(F.Pitch))));
      F.Size := FIni.ReadInteger(Option.Category, SName(strSize), F.Size);
      F.Style := String2Style(FIni.ReadString(Option.Category, SName(strStyle),
          Style2String(F.Style, false)));
      Option.AsFont := F;
    except
      raise EOptError.Create(oerNotFont);
    end;
  finally
    F.Free;
  end;
end;

procedure TOpSaver.LISAFs;
var
  L: TStringList;
  i, k: integer;
begin
  Option.SetDefault;
  L := TStringList.Create;
  try
    FIni.ReadSection(Option.Category, L);
    for i := 0 to L.Count - 1 do begin
      k := NameI(L[i]);
      if k >= 0 then
        Option.AsFlags[k] := GetBoolean(FIni.ReadString(Option.Category, L[i], ''));
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LISAIA;
var
  L: TStringList;
  i, k: integer;
begin
  Option.SetDefault;
  L := TStringList.Create;
  try
    FIni.ReadSection(Option.Category, L);
    try
      for i := 0 to L.Count - 1 do begin
        k := NameI(L[i]);
        if k >= 0 then
          Option.AsIntArray[k] := FIni.ReadInteger(Option.Category, L[i], 0);
      end;
    except
      raise EOptError.Create(oerNotInteger);
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LISAIn;
begin
  LIStIn;
end;

procedure TOpSaver.LISASA;
var
  L: TStringList;
  i, k: integer;
begin
  Option.SetDefault;
  L := TStringList.Create;
  try
    FIni.ReadSection(Option.Category, L);
    for i := 0 to L.Count - 1 do begin
      k := NameI(L[i]);
      if k >= 0 then
        Option.AsStrArray[k] := FIni.ReadString(Option.Category, L[i], '');
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LISASt;
begin
  LIStSt;
end;

procedure TOpSaver.LISATi;
begin
  LIStTi;
end;

procedure TOpSaver.LISt;
begin
  case Option.OptionType of
    otString     : LIStSt;
    otInteger    : LIStIn;
    otFloat      : LIStFl;
    otBoolean    : LIStBo;
    otColor      : LIStCo;
    otDate       : LIStDa;
    otTime       : LIStTi;
    otDateTime   : LIStDT;
    otFont       : LIStFo;
    otFlags      : LIStFs;
    otStrArray   : LIStSA;
    otIntArray   : LIStIA;
    otFloatArray : LIStFA;
  end;
end;

procedure TOpSaver.LIStBo;
begin
  Option.AsBoolean := GetBoolean(FIni.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LIStCo;
begin
  Option.AsColor := GetColor(FIni.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LIStDa;
begin
  Option.AsDate := GetDate(FIni.ReadString(Option.Category, Option.Name,
      Option.DefAsString)); 
end;

procedure TOpSaver.LIStDT;
begin
  Option.AsDateTime := GetDateTime(FIni.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LIStFA;
var
  Temp: string;
begin
  Temp := StringReplace(FIni.ReadString(Option.Category, Option.Name,
      StringReplace(Option.DefAsString, strCRLF, Option.ArraySeparator,
      [rfReplaceAll])), Option.ArraySeparator, strCRLF, [rfReplaceAll]);
  if CheckFloat(Temp) then
    Option.AsString := Temp
  else
    raise EOptError.Create(oerNotFloat);
end;

procedure TOpSaver.LIStFl;
begin
  Option.AsFloat := GetFloat(FIni.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LIStFo;
var
  F: TFont;
begin
  F := TFont.Create;
  try
    ReadFont(StringReplace(FIni.ReadString(Option.Category, Option.Name,
        StringReplace(Option.DefAsString, strCRLF, Option.ArraySeparator,
        [rfReplaceAll])), Option.ArraySeparator, strCRLF, [rfReplaceAll]), F);
    Option.AsFont := F;
  finally
    F.Free;
  end;
end;

procedure TOpSaver.LIStFs;
begin
  Option.AsString := ReadFlags(FIni.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LIStIA;
var
  Temp: string;
begin
  Temp := StringReplace(FIni.ReadString(Option.Category, Option.Name,
      StringReplace(Option.DefAsString, strCRLF, Option.ArraySeparator,
      [rfReplaceAll])), Option.ArraySeparator, strCRLF, [rfReplaceAll]);
  if CheckInt(Temp) then
    Option.AsString := Temp
  else
    raise EOptError.Create(oerNotInteger);
end;

procedure TOpSaver.LIStIn;
begin
  Option.AsInteger := GetInteger(FIni.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LIStSA;
begin
  Option.AsString := StringReplace(FIni.ReadString(Option.Category, Option.Name,
      StringReplace(Option.DefAsString, strCRLF, Option.ArraySeparator,
      [rfReplaceAll])), Option.ArraySeparator, strCRLF, [rfReplaceAll]);
end;

procedure TOpSaver.LIStSt;
begin
  Option.AsString := FIni.ReadString(Option.Category, Option.Name,
      Option.DefAsString);
end;

procedure TOpSaver.LIStTi;
begin
  Option.AsTime := GetTime(FIni.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LoadIni(const AIniName: string);
begin
  if Option <> nil then begin
    FIni := TIniFile.Create(AIniName);
    try
      case Option.StoreMode of
        stmString: LISt;
        stmBinary: LIBi;
        stmStrArray: LISA;
        stmBinArray: LIBA;
      end;
    finally
      FIni.Free;
    end;
  end;
end;

procedure TOpSaver.LoadMem(MemIni: TStrings);
begin
  if Option <> nil then begin
    FMem := TMemSaver.Create(MemIni, Option.BoolStr, Option.FloatMask);
    try
      case Option.StoreMode of
        stmString: LTSt;
        stmBinary: LTBi;
        stmStrArray: LTSA;
        stmBinArray: LTBA;
      end;
    finally
      FMem.Free;
    end;
  end;
end;

procedure TOpSaver.LoadReg(ARoot: HKEY; const ARegKey: string);
var
  Temp: string;
begin
  if Option <> nil then begin
    Option.SetDefault;
    try
      FReg := TRegistry.Create;
      try
        FReg.RootKey := ARoot;
        Temp := ARegKey;
        if (Temp <> '') and (Temp[Length(Temp)] <> '\') then
          Temp := Temp + '\';
        Temp := Temp + Option.Category;
        if FReg.KeyExists(Temp) then begin
          FReg.OpenKey(Temp, false);
          case Option.StoreMode of
            stmString: LRSt;
            stmBinary: LRBi;
            stmStrArray: LRSA;
            stmBinArray: LRBA;
          end;
        end;
      except
        raise EOptError.Create(oerRegLoad);
      end;
    finally
      FReg.Free;
    end;
  end;
end;

procedure TOpSaver.LRBA;
begin
  case Option.OptionType of
    otString     : LRBASt;
    otInteger    : LRBAIn;
    otFloat      : LRBAFl;
    otBoolean    : LRBABo;
    otColor      : LRBACo;
    otDate       : LRBADa;
    otTime       : LRBATi;
    otDateTime   : LRBADT;
    otFont       : LRBAFo;
    otFlags      : LRBAFs;
    otStrArray   : LRBASA;
    otIntArray   : LRBAIA;
    otFloatArray : LRBAFA;
  end;
end;

procedure TOpSaver.LRBABo;
begin
  LRBiBo;
end;

procedure TOpSaver.LRBACo;
begin
  LRBiCo;
end;

procedure TOpSaver.LRBADa;
begin
  LRBiDa;
end;

procedure TOpSaver.LRBADT;
begin
  LRBiDT;
end;

procedure TOpSaver.LRBAFA;
var
  L: TStringList;
  i, k: integer;
begin
  L := TStringList.Create;
  try
    FReg.GetValueNames(L);
    for i := 0 to L.Count - 1 do begin
      k := NameI(L[i]);
      if k >= 0 then
        Option.AsFloatArray[k] := FReg.ReadFloat(L[i]);
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LRBAFl;
begin
  LRBiFl;
end;

procedure TOpSaver.LRBAFo;
var
  F: TFont;
  Temp, Temp2: string;
  ln: integer;
begin
  F := Option.DefAsFont;
  try
    Temp := SName(strCharset);
    if FReg.ValueExists(Temp) then
      F.Charset := FReg.ReadInteger(Temp);
    Temp := SName(strColor);
    if FReg.ValueExists(Temp) then
      F.Color := FReg.ReadInteger(Temp);
    Temp := SName(strName);
    if FReg.ValueExists(Temp) then begin
      ln := FReg.GetDataSize(Temp);
      SetLength(Temp2, ln);
      FReg.ReadBinaryData(Temp, Temp2[1], ln);
      F.Name := Temp2;
    end;
    Temp := SName(strPitch);
    if FReg.ValueExists(Temp) then
      F.Pitch := TFontPitch(FReg.ReadInteger(Temp));
    Temp := SName(strSize);
    if FReg.ValueExists(Temp) then
      F.Size := FReg.ReadInteger(Temp);
    Temp := SName(strStyle);
    if FReg.ValueExists(Temp) then
      F.Style := Integer2Style(FReg.ReadInteger(Temp));
    Option.AsFont := F;
  finally
    F.Free;
  end;
end;

procedure TOpSaver.LRBAFs;
var
  L: TStringList;
  i, k: integer;
begin
  L := TStringList.Create;
  try
    FReg.GetValueNames(L);
    for i := 0 to L.Count - 1 do begin
      k := NameI(L[i]);
      if k >= 0 then
        Option.AsFlags[k] := FReg.ReadBool(L[i]);
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LRBAIA;
var
  L: TStringList;
  i, k: integer;
begin
  L := TStringList.Create;
  try
    FReg.GetValueNames(L);
    for i := 0 to L.Count - 1 do begin
      k := NameI(L[i]);
      if k >= 0 then
        Option.AsIntArray[k] := FReg.ReadInteger(L[i]);
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LRBAIn;
begin
  LRBiIn;
end;

procedure TOpSaver.LRBASA;
var
  L: TStringList;
  i, k, ln: integer;
  Temp: string;
begin
  L := TStringList.Create;
  try
    FReg.GetValueNames(L);
    for i := 0 to L.Count - 1 do begin
      k := NameI(L[i]);
      if k >= 0 then begin
        ln := FReg.GetDataSize(L[i]);
        SetLength(Temp, ln);
        FReg.ReadBinaryData(L[i], Temp[1], ln);
        Option.AsStrArray[k] := Temp;
      end;
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LRBASt;
begin
  LRBiSt;
end;

procedure TOpSaver.LRBATi;
begin
  LRBiTi;
end;

procedure TOpSaver.LRBi;
begin
  case Option.OptionType of
    otString     : LRBiSt;
    otInteger    : LRBiIn;
    otFloat      : LRBiFl;
    otBoolean    : LRBiBo;
    otColor      : LRBiCo;
    otDate       : LRBiDa;
    otTime       : LRBiTi;
    otDateTime   : LRBiDT;
    otFont       : LRBiFo;
    otFlags      : LRBiFs;
    otStrArray   : LRBiSA;
    otIntArray   : LRBiIA;
    otFloatArray : LRBiFA;
  end;
end;

procedure TOpSaver.LRBiBo;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsBoolean := FReg.ReadBool(Option.Name);
end;

procedure TOpSaver.LRBiCo;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsColor := FReg.ReadInteger(Option.Name);
end;

procedure TOpSaver.LRBiDa;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsDate := FReg.ReadDate(Option.Name);
end;

procedure TOpSaver.LRBiDT;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsDateTime := FReg.ReadDateTime(Option.Name);
end;

procedure TOpSaver.LRBiFA;
var
  ln, i: integer;
  Temp: string;
  TempF: double;
begin
  if FReg.ValueExists(Option.Name) then begin
    ln := FReg.GetDataSize(Option.Name);
    if ln mod SizeOf(double) = 0 then begin
      SetLength(Temp, ln);
      FReg.ReadBinaryData(Option.Name, Temp[1], ln);
      for i := 0 to ln div SizeOf(double) - 1 do begin
        Move(Temp[i * SizeOf(double) + 1], TempF, SizeOf(double));
        Option.AsFloatArray[i] := TempF;
      end;
    end else
      raise EOptError.Create(oerRegLoad);
  end;
end;

procedure TOpSaver.LRBiFl;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsFloat := FReg.ReadFloat(Option.Name);
end;

procedure TOpSaver.LRBiFo;
var
  Temp, TempS: string;
  ln, i, TempI: integer;
  TempB: byte;
  F: TFont;

  function CheckLength: boolean;
  begin
    result := ln > SizeOf(byte) + 2 * SizeOf(integer);
    if result then begin
      Move(Temp[SizeOf(byte) + SizeOf(integer) + 1], TempI, SizeOf(integer));
      result := ln = 3 * SizeOf(integer) + 3 * SizeOf(byte) + TempI;
    end;
  end;

begin
  if FReg.ValueExists(Option.Name) then begin
    ln := FReg.GetDataSize(Option.Name);
    SetLength(Temp, ln);
    FReg.ReadBinaryData(Option.Name, Temp[1], ln);
    if CheckLength then begin
      F := TFont.Create;
      try
        i := 1;
        Move(Temp[i], TempB, SizeOf(byte));
        F.Charset := TempB;
        Inc(i, SizeOf(byte));
        Move(Temp[i], TempI, SizeOf(integer));
        F.Color := TempI;
        Inc(i, SizeOf(integer));
        Move(Temp[i], TempI, SizeOf(integer));
        Inc(i, SizeOf(integer));
        SetLength(TempS, TempI);
        Move(Temp[i], TempS[1], TempI);
        F.Name := TempS;
        Inc(i, TempI);
        Move(Temp[i], TempB, SizeOf(byte));
        F.Pitch := TFontPitch(TempB);
        Inc(i, SizeOf(byte));
        Move(Temp[i], TempI, SizeOf(integer));
        F.Size := TempI;
        Inc(i, SizeOf(integer));
        Move(Temp[i], TempB, SizeOf(byte));
        F.Style := Integer2Style(TempB);
      finally
        F.Free;
      end;
    end else
      raise EOptError.Create(oerRegLoad);
  end;
end;

procedure TOpSaver.LRBiFs;
var
  Temp: string;
  i, ln, kln: integer;
begin
  if FReg.ValueExists(Option.Name) then begin
    ln := FReg.GetDataSize(Option.Name);
    if ln > SizeOf(integer) then begin
      SetLength(Temp, ln);
      FReg.ReadBinaryData(Option.Name, Temp[1], ln);
      Move(Temp[1], ln, SizeOf(integer));
      kln := ln div 8 + 1 + SizeOf(integer);
      if Length(Temp) < kln then
        Temp := Temp + StringOfChar(#0, kln - Length(Temp));
      Delete(Temp, 1, SizeOf(integer));
      for i := 0 to ln - 1 do
        Option.AsFlags[i] := byte(Temp[i div 8 + 1]) and (1 shl (i mod 8)) <> 0;
    end else
      raise EOptError.Create(oerRegLoad);
  end;
end;

procedure TOpSaver.LRBiIA;
var
  ln, i, TempI: integer;
  Temp: string;
begin
  if FReg.ValueExists(Option.Name) then begin
    ln := FReg.GetDataSize(Option.Name);
    if ln mod SizeOf(integer) = 0 then begin
      SetLength(Temp, ln);
      FReg.ReadBinaryData(Option.Name, Temp[1], ln);
      for i := 0 to ln div SizeOf(integer) - 1 do begin
        Move(Temp[i * SizeOf(integer) + 1], TempI, SizeOf(integer));
        Option.AsIntArray[i] := TempI;
      end;
    end else
      raise EOptError.Create(oerRegLoad);
  end;
end;

procedure TOpSaver.LRBiIn;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsInteger := FReg.ReadInteger(Option.Name);
end;

procedure TOpSaver.LRBiSA;
begin
  LRBiSt;
end;

procedure TOpSaver.LRBiSt;
var
  ln: integer;
  Temp: string;
begin
  if FReg.ValueExists(Option.Name) then begin
    ln := FReg.GetDataSize(Option.Name);
    SetLength(Temp, ln);
    FReg.ReadBinaryData(Option.Name, Temp[1], ln);
    Option.AsString := Temp;
  end;
end;

procedure TOpSaver.LRBiTi;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsTime := FReg.ReadTime(Option.Name);
end;

procedure TOpSaver.LRSa;
begin
  case Option.OptionType of
    otString     : LRSASt;
    otInteger    : LRSAIn;
    otFloat      : LRSAFl;
    otBoolean    : LRSABo;
    otColor      : LRSACo;
    otDate       : LRSADa;
    otTime       : LRSATi;
    otDateTime   : LRSADT;
    otFont       : LRSAFo;
    otFlags      : LRSAFs;
    otStrArray   : LRSASA;
    otIntArray   : LRSAIA;
    otFloatArray : LRSAFA;
  end;
end;

procedure TOpSaver.LRSABo;
begin
  LRStBo;
end;

procedure TOpSaver.LRSACo;
begin
  LRStCo;
end;

procedure TOpSaver.LRSADa;
begin
  LRStDa;
end;

procedure TOpSaver.LRSADT;
begin
  LRStDT;
end;

procedure TOpSaver.LRSAFA;
var
  L: TStringList;
  i, k: integer;
begin
  L := TStringList.Create;
  try
    FReg.GetValueNames(L);
    for i := 0 to L.Count - 1 do begin
      k := NameI(L[i]);
      if k >= 0 then
        Option.AsFloatArray[k] := StrToFloat(FReg.ReadString(L[i]));
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LRSAFl;
begin
  LRStFl;
end;

procedure TOpSaver.LRSAFo;
var
  F: TFont;
  Temp: string;
begin
  F := Option.DefAsFont;
  try
    Temp := SName(strCharset);
    if FReg.ValueExists(Temp) then
      F.Charset := StrToInt(FReg.ReadString(Temp));
    Temp := SName(strColor);
    if FReg.ValueExists(Temp) then
      F.Color := StringToColor(FReg.ReadString(Temp));
    Temp := SName(strName);
    if FReg.ValueExists(Temp) then
      F.Name := FReg.ReadString(Temp);
    Temp := SName(strPitch);
    if FReg.ValueExists(Temp) then
      F.Pitch := String2Pitch(FReg.ReadString(Temp));
    Temp := SName(strSize);
    if FReg.ValueExists(Temp) then
      F.Size := StrToInt(FReg.ReadString(Temp));
    Temp := SName(strStyle);
    if FReg.ValueExists(Temp) then
      F.Style := String2Style(FReg.ReadString(Temp));
  finally
    F.Free;
  end;
end;

procedure TOpSaver.LRSAFs;
var
  L: TStringList;
  i, k: integer;
begin
  L := TStringList.Create;
  try
    FReg.GetValueNames(L);
    for i := 0 to L.Count - 1 do begin
      k := NameI(L[i]);
      if k >= 0 then
        Option.AsFlags[k] := GetBoolean(FReg.ReadString(L[i]));
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LRSAIA;
var
  L: TStringList;
  i, k: integer;
begin
  L := TStringList.Create;
  try
    FReg.GetValueNames(L);
    for i := 0 to L.Count - 1 do begin
      k := NameI(L[i]);
      if k >= 0 then
        Option.AsIntArray[k] := StrToInt(FReg.ReadString(L[i]));
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LRSAIn;
begin
  LRStIn;
end;

procedure TOpSaver.LRSASA;
var
  L: TStringList;
  i, k: integer;
begin
  L := TStringList.Create;
  try
    FReg.GetValueNames(L);
    for i := 0 to L.Count - 1 do begin
      k := NameI(L[i]);
      if k >= 0 then
        Option.AsStrArray[k] := FReg.ReadString(L[i]);
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LRSASt;
begin
  LRStSt;
end;

procedure TOpSaver.LRSATi;
begin
  LRStTi;
end;

procedure TOpSaver.LRSt;
begin
  case Option.OptionType of
    otString     : LRStSt;
    otInteger    : LRStIn;
    otFloat      : LRStFl;
    otBoolean    : LRStBo;
    otColor      : LRStCo;
    otDate       : LRStDa;
    otTime       : LRStTi;
    otDateTime   : LRStDT;
    otFont       : LRStFo;
    otFlags      : LRStFs;
    otStrArray   : LRStSA;
    otIntArray   : LRStIA;
    otFloatArray : LRStFA;
  end;
end;

procedure TOpSaver.LRStBo;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsBoolean := GetBoolean(FReg.ReadString(Option.Name));
end;

procedure TOpSaver.LRStCo;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsColor := GetColor(FReg.ReadString(Option.Name));
end;

procedure TOpSaver.LRStDa;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsDate := GetDate(FReg.ReadString(Option.Name));
end;

procedure TOpSaver.LRStDT;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsDateTime := GetDateTime(FReg.ReadString(Option.Name));
end;

procedure TOpSaver.LRStFA;
var
  Temp: string;
begin
  if FReg.ValueExists(Option.Name) then
    Temp := StringReplace(FReg.ReadString(Option.Name), Option.ArraySeparator,
        strCRLF, [rfReplaceAll]);
    if CheckFloat(Temp) then
      Option.AsString := Temp
    else
      raise EOptError.Create(oerNotFloat);
end;

procedure TOpSaver.LRStFl;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsFloat := GetFloat(FReg.ReadString(Option.Name));
end;

procedure TOpSaver.LRStFo;
var
  F: TFont;
begin
  if FReg.ValueExists(Option.Name) then begin
    F := TFont.Create;
    try
      ReadFont(StringReplace(FReg.ReadString(Option.Name), Option.ArraySeparator,
          strCRLF, [rfReplaceAll]), F);
      Option.AsFont := F;
    finally
      F.Free;
    end;
  end;
end;

procedure TOpSaver.LRStFs;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsString := ReadFlags(FReg.ReadString(Option.Name));
end;

procedure TOpSaver.LRStIA;
var
  Temp: string;
begin
  if FReg.ValueExists(Option.Name) then begin
    Temp := StringReplace(FReg.ReadString(Option.Name), Option.ArraySeparator,
        strCRLF, [rfReplaceAll]);
    if CheckInt(Temp) then
      Option.AsString := Temp
    else
      raise EOptError.Create(oerNotInteger);
  end;
end;

procedure TOpSaver.LRStIn;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsInteger := GetInteger(FReg.ReadString(Option.Name));
end;

procedure TOpSaver.LRStSA;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsString := StringReplace(FReg.ReadString(Option.Name),
        Option.ArraySeparator, strCRLF, [rfReplaceAll]);
end;

procedure TOpSaver.LRStSt;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsString := FReg.ReadString(Option.Name);
end;

procedure TOpSaver.LRStTi;
begin
  if FReg.ValueExists(Option.Name) then
    Option.AsTime := GetTime(FReg.ReadString(Option.Name));
end;

procedure TOpSaver.LTBA;
begin
  case Option.OptionType of
    otString     : LTBASt;
    otInteger    : LTBAIn;
    otFloat      : LTBAFl;
    otBoolean    : LTBABo;
    otColor      : LTBACo;
    otDate       : LTBADa;
    otTime       : LTBATi;
    otDateTime   : LTBADT;
    otFont       : LTBAFo;
    otFlags      : LTBAFs;
    otStrArray   : LTBASA;
    otIntArray   : LTBAIA;
    otFloatArray : LTBAFA;
  end;
end;

procedure TOpSaver.LTBABo;
begin
  LTBiBo;
end;

procedure TOpSaver.LTBACo;
begin
  LTBiCo;
end;

procedure TOpSaver.LTBADa;
begin
  LTBiDa
end;

procedure TOpSaver.LTBADT;
begin
  LTBiDT;
end;

procedure TOpSaver.LTBAFA;
begin
  LTSAFA;
end;

procedure TOpSaver.LTBAFl;
begin
  LTBiFl;
end;

procedure TOpSaver.LTBAFo;
begin
  LTSAFo;
end;

procedure TOpSaver.LTBAFs;
begin
  LTSAFs;
end;

procedure TOpSaver.LTBAIA;
begin
  LTSAIA;
end;

procedure TOpSaver.LTBAIn;
begin
  LTBiIn;
end;

procedure TOpSaver.LTBASA;
begin
  LTSASA;
end;

procedure TOpSaver.LTBASt;
begin
  LTBiSt;
end;

procedure TOpSaver.LTBATi;
begin
  LTBiTi;
end;

procedure TOpSaver.LTBi;
begin
  case Option.OptionType of
    otString     : LTBiSt;
    otInteger    : LTBiIn;
    otFloat      : LTBiFl;
    otBoolean    : LTBiBo;
    otColor      : LTBiCo;
    otDate       : LTBiDa;
    otTime       : LTBiTi;
    otDateTime   : LTBiDT;
    otFont       : LTBiFo;
    otFlags      : LTBiFs;
    otStrArray   : LTBiSA;
    otIntArray   : LTBiIA;
    otFloatArray : LTBiFA;
  end;
end;

procedure TOpSaver.LTBiBo;
begin
  LTStBo;
end;

procedure TOpSaver.LTBiCo;
begin
  LTStCo;
end;

procedure TOpSaver.LTBiDa;
begin
  try
    Option.AsDate := FMem.ReadFloat(Option.Category, Option.Name, Option.DefAsDate);
  except
    raise EOptError.Create(oerNotDateTime);
  end;
end;

procedure TOpSaver.LTBiDT;
begin
  try
    Option.AsDateTime := FMem.ReadFloat(Option.Category, Option.Name,
        Option.DefAsDateTime);
  except
    raise EOptError.Create(oerNotDateTime);
  end;
end;

procedure TOpSaver.LTBiFA;
begin
  LTStFA;
end;

procedure TOpSaver.LTBiFl;
begin
  LTStFl;
end;

procedure TOpSaver.LTBiFo;
begin
  LTStFo;
end;

procedure TOpSaver.LTBiFs;
begin
  LTStFs;
end;

procedure TOpSaver.LTBiIA;
begin
  LTStIA;
end;

procedure TOpSaver.LTBiIn;
begin
  LTStIn;
end;

procedure TOpSaver.LTBiSA;
begin
  LTStSA;
end;

procedure TOpSaver.LTBiSt;
begin
  LTStSt;
end;

procedure TOpSaver.LTBiTi;
begin
  try
    Option.AsTime := FMem.ReadFloat(Option.Category, Option.Name, Option.DefAsTime);
  except
    raise EOptError.Create(oerNotDateTime);
  end;
end;

procedure TOpSaver.LTSA;
begin
  case Option.OptionType of
    otString     : LTSASt;
    otInteger    : LTSAIn;
    otFloat      : LTSAFl;
    otBoolean    : LTSABo;
    otColor      : LTSACo;
    otDate       : LTSADa;
    otTime       : LTSATi;
    otDateTime   : LTSADT;
    otFont       : LTSAFo;
    otFlags      : LTSAFs;
    otStrArray   : LTSASA;
    otIntArray   : LTSAIA;
    otFloatArray : LTSAFA;
  end;
end;

procedure TOpSaver.LTSABo;
begin
  LTStBo;
end;

procedure TOpSaver.LTSACo;
begin
  LTStCo;
end;

procedure TOpSaver.LTSADa;
begin
  LTStDa;
end;

procedure TOpSaver.LTSADT;
begin
  LTStDT;
end;

procedure TOpSaver.LTSAFA;
var
  L: TStringList;
  i, k: integer;
begin
  Option.SetDefault;
  L := TStringList.Create;
  try
    FMem.ReadSection(Option.Category, L);
    try
      for i := 0 to L.Count - 1 do begin
        k := NameI(L[i]);
        if k >= 0 then
          Option.AsFloatArray[k] := FMem.ReadFloat(Option.Category, L[i], 0);
      end;
    except
      raise EOptError.Create(oerNotFloat);
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LTSAFl;
begin
  LTStFl;
end;

procedure TOpSaver.LTSAFo;
var
  F: TFont;
begin
  F := Option.DefAsFont;
  try
    try
      F.Charset := FMem.ReadInteger(Option.Category, SName(strCharset), F.Charset);
      F.Color := String2Color(FMem.ReadString(Option.Category, SName(strColor),
          IntToStr(F.Color)));
      F.Name := FMem.ReadString(Option.Category, SName(strName), F.Name);
      F.Pitch := String2Pitch(FMem.ReadString(Option.Category, SName(strPitch),
          IntToStr(integer(F.Pitch))));
      F.Size := FMem.ReadInteger(Option.Category, SName(strSize), F.Size);
      F.Style := String2Style(FMem.ReadString(Option.Category, SName(strStyle),
          Style2String(F.Style, false)));
      Option.AsFont := F;
    except
      raise EOptError.Create(oerNotFont);
    end;
  finally
    F.Free;
  end;
end;

procedure TOpSaver.LTSAFs;
var
  L: TStringList;
  i, k: integer;
begin
  Option.SetDefault;
  L := TStringList.Create;
  try
    FMem.ReadSection(Option.Category, L);
    for i := 0 to L.Count - 1 do begin
      k := NameI(L[i]);
      if k >= 0 then
        Option.AsFlags[k] := GetBoolean(FMem.ReadString(Option.Category, L[i], ''));
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LTSAIA;
var
  L: TStringList;
  i, k: integer;
begin
  Option.SetDefault;
  L := TStringList.Create;
  try
    FMem.ReadSection(Option.Category, L);
    try
      for i := 0 to L.Count - 1 do begin
        k := NameI(L[i]);
        if k >= 0 then
          Option.AsIntArray[k] := FMem.ReadInteger(Option.Category, L[i], 0);
      end;
    except
      raise EOptError.Create(oerNotInteger);
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LTSAIn;
begin
  LTStIn;
end;

procedure TOpSaver.LTSASA;
var
  L: TStringList;
  i, k: integer;
begin
  Option.SetDefault;
  L := TStringList.Create;
  try
    FMem.ReadSection(Option.Category, L);
    for i := 0 to L.Count - 1 do begin
      k := NameI(L[i]);
      if k >= 0 then
        Option.AsStrArray[k] := FMem.ReadString(Option.Category, L[i], '');
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.LTSASt;
begin
  LTStSt;
end;

procedure TOpSaver.LTSATi;
begin
  LTStTi;
end;

procedure TOpSaver.LTSt;
begin
  case Option.OptionType of
    otString     : LTStSt;
    otInteger    : LTStIn;
    otFloat      : LTStFl;
    otBoolean    : LTStBo;
    otColor      : LTStCo;
    otDate       : LTStDa;
    otTime       : LTStTi;
    otDateTime   : LTStDT;
    otFont       : LTStFo;
    otFlags      : LTStFs;
    otStrArray   : LTStSA;
    otIntArray   : LTStIA;
    otFloatArray : LTStFA;
  end;
end;

procedure TOpSaver.LTStBo;
begin
  Option.AsBoolean := GetBoolean(FMem.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LTStCo;
begin
  Option.AsColor := GetColor(FMem.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LTStDa;
begin
  Option.AsDate := GetDate(FMem.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LTStDT;
begin
  Option.AsDateTime := GetDateTime(FMem.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LTStFA;
var
  Temp: string;
begin
  Temp := StringReplace(FMem.ReadString(Option.Category, Option.Name,
      StringReplace(Option.DefAsString, strCRLF, Option.ArraySeparator,
      [rfReplaceAll])), Option.ArraySeparator, strCRLF, [rfReplaceAll]);
  if CheckFloat(Temp) then
    Option.AsString := Temp
  else
    raise EOptError.Create(oerNotFloat);
end;

procedure TOpSaver.LTStFl;
begin
  Option.AsFloat := GetFloat(FMem.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LTStFo;
var
  F: TFont;
begin
  F := TFont.Create;
  try
    ReadFont(StringReplace(FMem.ReadString(Option.Category, Option.Name,
        StringReplace(Option.DefAsString, strCRLF, Option.ArraySeparator,
        [rfReplaceAll])), Option.ArraySeparator, strCRLF, [rfReplaceAll]), F);
    Option.AsFont := F;
  finally
    F.Free;
  end;
end;

procedure TOpSaver.LTStFs;
begin
  Option.AsString := ReadFlags(FMem.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LTStIA;
var
  Temp: string;
begin
  Temp := StringReplace(FMem.ReadString(Option.Category, Option.Name,
      StringReplace(Option.DefAsString, strCRLF, Option.ArraySeparator,
      [rfReplaceAll])), Option.ArraySeparator, strCRLF, [rfReplaceAll]);
  if CheckInt(Temp) then
    Option.AsString := Temp
  else
    raise EOptError.Create(oerNotInteger);
end;

procedure TOpSaver.LTStIn;
begin
  Option.AsInteger := GetInteger(FMem.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

procedure TOpSaver.LTStSA;
begin
  Option.AsString := StringReplace(FMem.ReadString(Option.Category, Option.Name,
      StringReplace(Option.DefAsString, strCRLF, Option.ArraySeparator,
      [rfReplaceAll])), Option.ArraySeparator, strCRLF, [rfReplaceAll]);
end;

procedure TOpSaver.LTStSt;
begin
  Option.AsString := FMem.ReadString(Option.Category, Option.Name,
      Option.DefAsString);
end;

procedure TOpSaver.LTStTi;
begin
  Option.AsTime := GetTime(FMem.ReadString(Option.Category, Option.Name,
      Option.DefAsString));
end;

function TOpSaver.NameI(const s: string): integer;
begin
  if Pos(Option.Name + Option.NameSeparator, s) = 1 then
    try
      result := StrToInt(Copy(s, Length(Option.Name + Option.NameSeparator) + 1,
          Length(s) - Length(Option.Name + Option.NameSeparator)));
    except
      result := -1;
    end
  else
    result := -1;
end;

function TOpSaver.ReadFlags(const s: string): string;
var
  L: TStringList;
  i: integer;
begin
  L := TStringList.Create;
  try
    L.BeginUpdate;
    L.Text := StringReplace(s, Option.ArraySeparator, strCRLF, [rfReplaceAll]);
    L.EndUpdate;
    if (L.Count = 1) and (Length(L[0]) > 1) then begin
      result := StringOfChar('0', Length(L[0]));
      for i := 0 to Length(L[0]) do
        if GetBoolean(L[0][i]) then
          result[i] := '1';
    end else begin
      result := StringOfChar('0', L.Count);
      for i := 0 to L.Count - 1 do
        if GetBoolean(L[i]) then
          result[i + 1] := '1';
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.ReadFont(const s: string; F: TFont);
var
  L: TStringList;
begin
  L := TStringList.Create;
  try
    L.BeginUpdate;
    L.Text := s;
    L.EndUpdate;
    try
      F.Charset := StrToInt(L[0]);
      F.Color := String2Color(L[1]);
      F.Name := L[2];
      F.Pitch := String2Pitch(L[3]);
      F.Size := StrToInt(L[4]);
      F.Style := String2Style(L[5]);
    except
      raise EOptError.Create(oerNotFont);
    end;
  finally
    L.Free;
  end;
end;

procedure TOpSaver.SaveIni(const AIniName: string);
begin
  if Option <> nil then begin
    FIni := TIniFile.Create(AIniName);
    try
      case Option.StoreMode of
        stmString: SISt;
        stmBinary: SIBi;
        stmStrArray: SISA;
        stmBinArray: SIBA;
      end;
    finally
      FIni.Free;
    end;
  end;
end;

procedure TOpSaver.SaveMem(MemIni: TStrings);
begin
  if Option <> nil then begin
    FMem := TMemSaver.Create(MemIni, Option.BoolStr, Option.FloatMask);
    try
      case Option.StoreMode of
        stmString: STSt;
        stmBinary: STBi;
        stmStrArray: STSA;
        stmBinArray: STBA;
      end;
    finally
      FMem.Free;
    end;
  end;
end;

procedure TOpSaver.SaveReg(ARoot: HKEY; const ARegKey: string);
var
  Temp: string;
begin
  if Option <> nil then begin
    FReg := TRegistry.Create;
    try
      FReg.RootKey := ARoot;
      Temp := ARegKey;
      if (Temp = '') or (Temp[Length(Temp)] <> '') then
        Temp := Temp + '\';
      Temp := Temp + Option.Category;
      FReg.OpenKey(Temp, true);
      case Option.StoreMode of
        stmString: SRSt;
        stmBinary: SRBi;
        stmStrArray: SRSA;
        stmBinArray: SRBA;
      end;
    finally
      FReg.Free;
    end;
  end;
end;

procedure TOpSaver.SIBA;
begin
  case Option.OptionType of
    otString     : SIBASt;
    otInteger    : SIBAIn;
    otFloat      : SIBAFl;
    otBoolean    : SIBABo;
    otColor      : SIBACo;
    otDate       : SIBADa;
    otTime       : SIBATi;
    otDateTime   : SIBADT;
    otFont       : SIBAFo;
    otFlags      : SIBAFs;
    otStrArray   : SIBASA;
    otIntArray   : SIBAIA;
    otFloatArray : SIBAFA;
  end;
end;

procedure TOpSaver.SIBABo;
begin
  SIBiBo;
end;

procedure TOpSaver.SIBACo;
begin
  SIBiCo;
end;

procedure TOpSaver.SIBADa;
begin
  SIBiDa;
end;

procedure TOpSaver.SIBADT;
begin
  SIBiDT;
end;

procedure TOpSaver.SIBAFA;
begin
  SISAFA;
end;

procedure TOpSaver.SIBAFl;
begin
  SIBiFl;
end;

procedure TOpSaver.SIBAFo;
var
  F: TFont;
begin
  F := Option.AsFont;
  try
    FIni.WriteInteger(Option.Category, SName(strCharset), F.Charset);
    FIni.WriteInteger(Option.Category, SName(strColor), F.Color);
    FIni.WriteString(Option.Category, SName(strName), F.Name);
    FIni.WriteInteger(Option.Category, SName(strPitch), integer(F.Pitch));
    FIni.WriteInteger(Option.Category, SName(strSize), F.Size);
    FIni.WriteString(Option.Category, SName(strStyle), Style2String(F.Style, false));
  finally
    F.Free;
  end;
end;

procedure TOpSaver.SIBAFs;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FIni.WriteBool(Option.Category, IName(i), Option.AsFlags[i]);
end;

procedure TOpSaver.SIBAIA;
begin
  SISAIA;
end;

procedure TOpSaver.SIBAIn;
begin
  SIBiIn;
end;

procedure TOpSaver.SIBASA;
begin
  SISASA;
end;

procedure TOpSaver.SIBASt;
begin
  SIBiSt;
end;

procedure TOpSaver.SIBATi;
begin
  SIBiTi;
end;

procedure TOpSaver.SIBi;
begin
  case Option.OptionType of
    otString     : SIBiSt;
    otInteger    : SIBiIn;
    otFloat      : SIBiFl;
    otBoolean    : SIBiBo;
    otColor      : SIBiCo;
    otDate       : SIBiDa;
    otTime       : SIBiTi;
    otDateTime   : SIBiDT;
    otFont       : SIBiFo;
    otFlags      : SIBiFs;
    otStrArray   : SIBiSA;
    otIntArray   : SIBiIA;
    otFloatArray : SIBiFA;
  end;
end;

procedure TOpSaver.SIBiBo;
begin
  FIni.WriteBool(Option.Category, Option.Name, Option.AsBoolean);
end;

procedure TOpSaver.SIBiCo;
begin
  FIni.WriteInteger(Option.Category, Option.Name, Option.AsColor);
end;

procedure TOpSaver.SIBiDa;
begin
  FIni.WriteFloat(Option.Category, Option.Name, Option.AsDate);
end;

procedure TOpSaver.SIBiDT;
begin
  FIni.WriteFloat(Option.Category, Option.Name, Option.AsDateTime);
end;

procedure TOpSaver.SIBiFA;
begin
  SIStFA;
end;

procedure TOpSaver.SIBiFl;
begin
  SIStFl;
end;

procedure TOpSaver.SIBiFo;
var
  F: TFont;
begin
  F := TFont.Create;
  try
    ReadFont(Option.AsString, F);
    FIni.WriteString(Option.Category, Option.Name, WriteFont(F, false, Option.ArraySeparator));
  finally
    F.Free;
  end;
end;

procedure TOpSaver.SIBiFs;
begin
  FIni.WriteString(Option.Category, Option.Name, Option.AsString);
end;

procedure TOpSaver.SIBiIA;
begin
  SIStIA;
end;

procedure TOpSaver.SIBiIn;
begin
  SIStIn;
end;

procedure TOpSaver.SIBiSA;
begin
  SIStSA;
end;

procedure TOpSaver.SIBiSt;
begin
  SIStSt;
end;

procedure TOpSaver.SIBiTi;
begin
  FIni.WriteFloat(Option.Category, Option.Name, Option.AsTime);
end;

procedure TOpSaver.SISA;
begin
  case Option.OptionType of
    otString     : SISASt;
    otInteger    : SISAIn;
    otFloat      : SISAFl;
    otBoolean    : SISABo;
    otColor      : SISACo;
    otDate       : SISADa;
    otTime       : SISATi;
    otDateTime   : SISADT;
    otFont       : SISAFo;
    otFlags      : SISAFs;
    otStrArray   : SISASA;
    otIntArray   : SISAIA;
    otFloatArray : SISAFA;
  end;
end;

procedure TOpSaver.SISABo;
begin
  SIStBo;
end;

procedure TOpSaver.SISACo;
begin
  SIStCo;
end;

procedure TOpSaver.SISADa;
begin
  SIStDa;
end;

procedure TOpSaver.SISADT;
begin
  SIStDT;
end;

procedure TOpSaver.SISAFA;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FIni.WriteString(Option.Category, IName(i), Option.DefAsStrArray[i]);
end;

procedure TOpSaver.SISAFl;
begin
  SIStFl;
end;

procedure TOpSaver.SISAFo;
var
  F: TFont;
begin
  F := Option.AsFont;
  try
    FIni.WriteInteger(Option.Category, SName(strCharset), F.Charset);
    FIni.WriteString(Option.Category, SName(strColor), ColorToString(F.Color));
    FIni.WriteString(Option.Category, SName(strName), F.Name);
    FIni.WriteString(Option.Category, SName(strPitch), Pitch2String(F.Pitch));
    FIni.WriteInteger(Option.Category, SName(strSize), F.Size);
    FIni.WriteString(Option.Category, SName(strStyle), Style2String(F.Style, true));
  finally
    F.Free;
  end;
end;

procedure TOpSaver.SISAFs;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FIni.WriteString(Option.Category, IName(i), BoolTxt(Option.BoolStr, Option.AsFlags[i]));
end;

procedure TOpSaver.SISAIA;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FIni.WriteInteger(Option.Category, IName(i), Option.AsIntArray[i]);
end;

procedure TOpSaver.SISAIn;
begin
  SIStIn;
end;

procedure TOpSaver.SISASA;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FIni.WriteString(Option.Category, IName(i), Option.AsStrArray[i]);
end;

procedure TOpSaver.SISASt;
begin
  SIStSt;
end;

procedure TOpSaver.SISATi;
begin
  SIStTi;
end;

procedure TOpSaver.SISt;
begin
  case Option.OptionType of
    otString     : SIStSt;
    otInteger    : SIStIn;
    otFloat      : SIStFl;
    otBoolean    : SIStBo;
    otColor      : SIStCo;
    otDate       : SIStDa;
    otTime       : SIStTi;
    otDateTime   : SIStDT;
    otFont       : SIStFo;
    otFlags      : SIStFs;
    otStrArray   : SIStSA;
    otIntArray   : SIStIA;
    otFloatArray : SIStFA;
  end;
end;

procedure TOpSaver.SIStBo;
begin
  FIni.WriteString(Option.Category, Option.Name, BoolTxt(Option.BoolStr, Option.AsBoolean));
end;

procedure TOpSaver.SIStCo;
begin
  FIni.WriteString(Option.Category, Option.Name, ColorToString(Option.AsColor));
end;

procedure TOpSaver.SIStDa;
begin
  FIni.WriteString(Option.Category, Option.Name, DateToStr(Option.AsDate));
end;

procedure TOpSaver.SIStDT;
begin
  FIni.WriteString(Option.Category, Option.Name, DateTimeToStr(Option.AsDateTime));
end;

procedure TOpSaver.SIStFA;
begin
  if CheckFloat(Option.AsString) then
    FIni.WriteString(Option.Category, Option.Name, StringReplace(Option.AsString,
        strCRLF, Option.ArraySeparator, [rfReplaceAll]))
    else
      raise EOptError.Create(oerNotFloat);
end;

procedure TOpSaver.SIStFl;
begin
  FIni.WriteString(Option.Category, Option.Name, FormatFloat(Option.FloatMask,
      Option.AsFloat));
end;

procedure TOpSaver.SIStFo;
var
  F: TFont;
begin
  F := TFont.Create;
  try
    ReadFont(Option.AsString, F);
    FIni.WriteString(Option.Category, Option.Name, WriteFont(F, true, Option.ArraySeparator));
  finally
    F.Free;
  end;
end;

procedure TOpSaver.SIStFs;
var
  i: integer;
  Temp: string;
begin
  Temp := '';
  for i := 0 to Option.Count - 1 do
    Temp := Temp + BoolTxt(Option.BoolStr, Option.AsFlags[i]) + Option.ArraySeparator;
  FIni.WriteString(Option.Category, Option.Name, Temp);
end;

procedure TOpSaver.SIStIA;
begin
  if CheckInt(Option.AsString) then
    FIni.WriteString(Option.Category, Option.Name, StringReplace(Option.AsString,
        strCRLF, Option.ArraySeparator, [rfReplaceAll]))
  else
    raise EOptError.Create(oerNotInteger);
end;

procedure TOpSaver.SIStIn;
begin
  FIni.WriteString(Option.Category, Option.Name, IntToStr(Option.AsInteger));
end;

procedure TOpSaver.SIStSA;
begin
  FIni.WriteString(Option.Category, Option.Name, StringReplace(Option.AsString,
      strCRLF, Option.ArraySeparator, [rfReplaceAll]));
end;

procedure TOpSaver.SIStSt;
begin
  FIni.WriteString(Option.Category, Option.Name, Option.AsString);
end;

procedure TOpSaver.SIStTi;
begin
  FIni.WriteString(Option.Category, Option.Name, TimeToStr(Option.AsTime));
end;

function TOpSaver.SName(const s: string): string;
begin
  result := Option.Name + Option.NameSeparator + s;
end;

procedure TOpSaver.SRBA;
begin
  case Option.OptionType of
    otString     : SRBASt;
    otInteger    : SRBAIn;
    otFloat      : SRBAFl;
    otBoolean    : SRBABo;
    otColor      : SRBACo;
    otDate       : SRBADa;
    otTime       : SRBATi;
    otDateTime   : SRBADT;
    otFont       : SRBAFo;
    otFlags      : SRBAFs;
    otStrArray   : SRBASA;
    otIntArray   : SRBAIA;
    otFloatArray : SRBAFA;
  end;
end;

procedure TOpSaver.SRBABo;
begin
  SRBiBo;
end;

procedure TOpSaver.SRBACo;
begin
  SRBiCo;
end;

procedure TOpSaver.SRBADa;
begin
  SRBiDa;
end;

procedure TOpSaver.SRBADT;
begin
  SRBiDT;
end;

procedure TOpSaver.SRBAFA;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FReg.WriteFloat(IName(i), Option.AsFloatArray[i]);
end;

procedure TOpSaver.SRBAFl;
begin
  SRBiFl;
end;

procedure TOpSaver.SRBAFo;
var
  F: TFont;
  Temp: string;
begin
  F := Option.AsFont;
  try
    FReg.WriteInteger(SName(strCharset), F.Charset);
    FReg.WriteInteger(SName(strColor), F.Color);
    Temp := F.Name;
    FReg.WriteBinaryData(SName(strName), Temp[1], Length(Temp));
    FReg.WriteInteger(SName(strPitch), integer(F.Pitch));
    FReg.WriteInteger(SName(strSize), F.Size);
    FReg.WriteInteger(SName(strStyle), Style2Integer(F.Style));
  finally
    F.Free;
  end;
end;

procedure TOpSaver.SRBAFs;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FReg.WriteBool(IName(i), Option.AsFlags[i]);
end;

procedure TOpSaver.SRBAIA;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FReg.WriteInteger(IName(i), Option.AsIntArray[i]);
end;

procedure TOpSaver.SRBAIn;
begin
  SRBiIn;
end;

procedure TOpSaver.SRBASA;
var
  i: integer;
  Temp: string;
begin
  for i := 0 to Option.Count - 1 do begin
    Temp := Option.AsStrArray[i];
    FReg.WriteBinaryData(IName(i), Temp[1], Length(Temp));
  end;
end;

procedure TOpSaver.SRBASt;
begin
  SRBiSt;
end;

procedure TOpSaver.SRBATi;
begin
  SRBiTi;
end;

procedure TOpSaver.SRBi;
begin
  case Option.OptionType of
    otString     : SRBiSt;
    otInteger    : SRBiIn;
    otFloat      : SRBiFl;
    otBoolean    : SRBiBo;
    otColor      : SRBiCo;
    otDate       : SRBiDa;
    otTime       : SRBiTi;
    otDateTime   : SRBiDT;
    otFont       : SRBiFo;
    otFlags      : SRBiFs;
    otStrArray   : SRBiSA;
    otIntArray   : SRBiIA;
    otFloatArray : SRBiFA;
  end;
end;

procedure TOpSaver.SRBiBo;
begin
  FReg.WriteBool(Option.Name, Option.AsBoolean);
end;

procedure TOpSaver.SRBiCo;
begin
  FReg.WriteInteger(Option.Name, Option.AsColor);
end;

procedure TOpSaver.SRBiDa;
begin
  FReg.WriteDate(Option.Name, Option.AsDate);
end;

procedure TOpSaver.SRBiDT;
begin
  FReg.WriteDateTime(Option.Name, Option.AsDateTime);
end;

procedure TOpSaver.SRBiFA;
var
  Temp: string;
  TempD: double;
  i: integer;
begin
  Temp := '';
  for i := 0 to Option.Count - 1 do begin
    SetLength(Temp, Length(Temp) + SizeOf(double));
    TempD := Option.AsFloatArray[i];
    Move(TempD, Temp[i * SizeOf(double) + 1], SizeOf(double));
  end;
  FReg.WriteBinaryData(Option.Name, Temp[1], Length(Temp));
end;

procedure TOpSaver.SRBiFl;
begin
  FReg.WriteFloat(Option.Name, Option.AsFloat);
end;

procedure TOpSaver.SRBiFo;
var
  Temp: string;
  i, TempI: integer;
  F: TFont;
  TempB: byte;
begin
  F := Option.AsFont;
  try
    SetLength(Temp, 3 * SizeOf(integer) + Length(F.Name) + 3 * SizeOf(byte));
    i := 1;
    TempB := F.Charset;
    Move(TempB, Temp[i], SizeOf(byte));
    Inc(i, SizeOf(byte));
    TempI := F.Color;
    Move(TempI, Temp[i], SizeOf(integer));
    Inc(i, SizeOf(integer));
    TempI := Length(F.Name);
    Move(TempI, Temp[i], SizeOf(integer));
    Inc(i, SizeOf(integer));
    Move(F.Name[1], Temp[i], Length(F.Name));
    Inc(i, Length(F.Name));
    TempB := byte(F.Pitch);
    Move(TempB, Temp[i], SizeOf(byte));
    Inc(i, SizeOf(byte));
    TempI := F.Size;
    Move(TempI, Temp[i], SizeOf(integer));
    Inc(i, SizeOf(integer));
    TempB := Style2Integer(F.Style);
    Move(TempB, Temp[i], SizeOf(byte));
  finally
    F.Free;
  end;
  FReg.WriteBinaryData(Option.Name, Temp[1], Length(Temp));
end;

procedure TOpSaver.SRBiFs;
var
  Temp: string;
  TempI, i: integer;
begin
  Temp := StringOfChar(#0, Option.Count div 8 + 1 + SizeOf(integer));
  TempI := Option.Count;
  Move(TempI, Temp[1], SizeOf(integer));
  for i := 0 to Option.Count - 1 do
    if Option.AsFlags[i] then begin
      TempI := SizeOf(integer) + 1 + i div 8;
      Temp[TempI] := char(byte(Temp[TempI]) or (1 shl (i mod 8)));
    end;
  FReg.WriteBinaryData(Option.Name, Temp[1], Length(Temp));
end;

procedure TOpSaver.SRBiIA;
var
  Temp: string;
  i, TempI: integer;
begin
  Temp := '';
  for i := 0 to Option.Count - 1 do begin
    SetLength(Temp, Length(Temp) + SizeOf(integer));
    TempI := Option.AsIntArray[i];
    Move(TempI, Temp[i * SizeOf(integer) + 1], SizeOf(integer));
  end;
  FReg.WriteBinaryData(Option.Name, Temp[1], Length(Temp));
end;

procedure TOpSaver.SRBiIn;
begin
  FReg.WriteInteger(Option.Name, Option.AsInteger);
end;

procedure TOpSaver.SRBiSA;
begin
  SRBiSt;
end;

procedure TOpSaver.SRBiSt;
var
  Temp: string;
begin
  Temp := Option.AsString;
  FReg.WriteBinaryData(Option.Name, Temp[1], Length(Temp));
end;

procedure TOpSaver.SRBiTi;
begin
  FReg.WriteTime(Option.Name, Option.AsTime);
end;

procedure TOpSaver.SRSA;
begin
  case Option.OptionType of
    otString     : SRSASt;
    otInteger    : SRSAIn;
    otFloat      : SRSAFl;
    otBoolean    : SRSABo;
    otColor      : SRSACo;
    otDate       : SRSADa;
    otTime       : SRSATi;
    otDateTime   : SRSADT;
    otFont       : SRSAFo;
    otFlags      : SRSAFs;
    otStrArray   : SRSASA;
    otIntArray   : SRSAIA;
    otFloatArray : SRSAFA;
  end;
end;

procedure TOpSaver.SRSABo;
begin
  SRStBo;
end;

procedure TOpSaver.SRSACo;
begin
  SRStCo;
end;

procedure TOpSaver.SRSADa;
begin
  SRStDa;
end;

procedure TOpSaver.SRSADT;
begin
  SRStDT;
end;

procedure TOpSaver.SRSAFA;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FReg.WriteString(IName(i), FormatFloat(Option.FloatMask, Option.AsFloatArray[i]));
end;

procedure TOpSaver.SRSAFl;
begin
  SRStFl;
end;

procedure TOpSaver.SRSAFo;
var
  F: TFont;
begin
  F := Option.AsFont;
  try
    FReg.WriteString(SName(strCharset), IntToStr(F.Charset));
    FReg.WriteString(SName(strColor), ColorToString(F.Color));
    FReg.WriteString(SName(strName), F.Name);
    FReg.WriteString(SName(strPitch), Pitch2String(F.Pitch));
    FReg.WriteString(SName(strSize), IntToStr(F.Size));
    FReg.WriteString(SName(strStyle), Style2String(F.Style, true));
  finally
    F.Free;
  end;
end;

procedure TOpSaver.SRSAFs;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FReg.WriteString(IName(i), BoolTxt(Option.BoolStr, Option.AsFlags[i]));
end;

procedure TOpSaver.SRSAIA;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FReg.WriteString(IName(i), IntToStr(Option.AsIntArray[i]));
end;

procedure TOpSaver.SRSAIn;
begin
  SRStIn;
end;

procedure TOpSaver.SRSASA;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FReg.WriteString(IName(i), Option.AsStrArray[i]);
end;

procedure TOpSaver.SRSASt;
begin
  SRStSt;
end;

procedure TOpSaver.SRSATi;
begin
  SRStTi;
end;

procedure TOpSaver.SRSt;
begin
  case Option.OptionType of
    otString     : SRStSt;
    otInteger    : SRStIn;
    otFloat      : SRStFl;
    otBoolean    : SRStBo;
    otColor      : SRStCo;
    otDate       : SRStDa;
    otTime       : SRStTi;
    otDateTime   : SRStDT;
    otFont       : SRStFo;
    otFlags      : SRStFs;
    otStrArray   : SRStSA;
    otIntArray   : SRStIA;
    otFloatArray : SRStFA;
  end;
end;

procedure TOpSaver.SRStBo;
begin
  FReg.WriteString(Option.Name, BoolTxt(Option.BoolStr, Option.AsBoolean));
end;

procedure TOpSaver.SRStCo;
begin
  FReg.WriteString(Option.Name, ColorToString(Option.AsColor));
end;

procedure TOpSaver.SRStDa;
begin
  FReg.WriteString(Option.Name, DateToStr(Option.AsDate));
end;

procedure TOpSaver.SRStDT;
begin
  FReg.WriteString(Option.Name, DateTimeToStr(Option.AsDateTime));
end;

procedure TOpSaver.SRStFA;
begin
  if CheckFloat(Option.AsString) then
    FReg.WriteString(Option.Name, StringReplace(Option.AsString, strCRLF,
        Option.ArraySeparator, [rfReplaceAll]));
end;

procedure TOpSaver.SRStFl;
begin
  FReg.WriteString(Option.Name, FormatFloat(Option.FloatMask, Option.AsFloat));
end;

procedure TOpSaver.SRStFo;
var
  F: TFont;
begin
  F := Option.AsFont;
  try
    FReg.WriteString(Option.Name, WriteFont(F, true, Option.ArraySeparator));
  finally
    F.Free;
  end;
end;

procedure TOpSaver.SRStFs;
var
  i: integer;
  Temp: string;
begin
  Temp := '';
  for i := 0 to Option.Count - 1 do
    Temp := Temp + BoolTxt(Option.BoolStr, Option.AsFlags[i]) + Option.ArraySeparator;
  FReg.WriteString(Option.Name, Temp);
end;

procedure TOpSaver.SRStIA;
begin
  if CheckInt(Option.AsString) then
    FReg.WriteString(Option.Name, StringReplace(Option.AsString, strCRLF,
        Option.ArraySeparator, [rfReplaceAll]))
  else
    raise EOptError.Create(oerNotInteger);
end;

procedure TOpSaver.SRStIn;
begin
  FReg.WriteString(Option.Name, IntToStr(Option.AsInteger));
end;

procedure TOpSaver.SRStSA;
begin
  FReg.WriteString(Option.Name, StringReplace(Option.AsString, strCRLF,
      Option.ArraySeparator, [rfReplaceAll]));
end;

procedure TOpSaver.SRStSt;
begin
  FReg.WriteString(Option.Name, Option.AsString);
end;

procedure TOpSaver.SRStTi;
begin
  FReg.WriteString(Option.Name, TimeToStr(Option.AsTime));
end;

procedure TOpSaver.STBA;
begin
  case Option.OptionType of
    otString     : STBASt;
    otInteger    : STBAIn;
    otFloat      : STBAFl;
    otBoolean    : STBABo;
    otColor      : STBACo;
    otDate       : STBADa;
    otTime       : STBATi;
    otDateTime   : STBADT;
    otFont       : STBAFo;
    otFlags      : STBAFs;
    otStrArray   : STBASA;
    otIntArray   : STBAIA;
    otFloatArray : STBAFA;
  end;
end;

procedure TOpSaver.STBABo;
begin
  STBiBo;
end;

procedure TOpSaver.STBACo;
begin
  STBiCo;
end;

procedure TOpSaver.STBADa;
begin
  STBiDa;
end;

procedure TOpSaver.STBADT;
begin
  STBiDT;
end;

procedure TOpSaver.STBAFA;
begin
  STSAFA;
end;

procedure TOpSaver.STBAFl;
begin
  STBiFl;
end;

procedure TOpSaver.STBAFo;
var
  F: TFont;
begin
  F := Option.AsFont;
  try
    FMem.WriteInteger(Option.Category, SName(strCharset), F.Charset);
    FMem.WriteInteger(Option.Category, SName(strColor), F.Color);
    FMem.WriteString(Option.Category, SName(strName), F.Name);
    FMem.WriteInteger(Option.Category, SName(strPitch), integer(F.Pitch));
    FMem.WriteInteger(Option.Category, SName(strSize), F.Size);
    FMem.WriteString(Option.Category, SName(strStyle), Style2String(F.Style, false));
  finally
    F.Free;
  end;
end;

procedure TOpSaver.STBAFs;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FMem.WriteBool(Option.Category, IName(i), Option.AsFlags[i]);
end;

procedure TOpSaver.STBAIA;
begin
  STSAIA;
end;

procedure TOpSaver.STBAIn;
begin
  STBiIn;
end;

procedure TOpSaver.STBASA;
begin
  STSASA;
end;

procedure TOpSaver.STBASt;
begin
  STBiSt;
end;

procedure TOpSaver.STBATi;
begin
  STBiTi;
end;

procedure TOpSaver.STBi;
begin
  case Option.OptionType of
    otString     : STBiSt;
    otInteger    : STBiIn;
    otFloat      : STBiFl;
    otBoolean    : STBiBo;
    otColor      : STBiCo;
    otDate       : STBiDa;
    otTime       : STBiTi;
    otDateTime   : STBiDT;
    otFont       : STBiFo;
    otFlags      : STBiFs;
    otStrArray   : STBiSA;
    otIntArray   : STBiIA;
    otFloatArray : STBiFA;
  end;
end;

procedure TOpSaver.STBiBo;
begin
  FMem.WriteBool(Option.Category, Option.Name, Option.AsBoolean);
end;

procedure TOpSaver.STBiCo;
begin
  FMem.WriteInteger(Option.Category, Option.Name, Option.AsColor);
end;

procedure TOpSaver.STBiDa;
begin
  FMem.WriteFloat(Option.Category, Option.Name, Option.AsDate);
end;

procedure TOpSaver.STBiDT;
begin
  FMem.WriteFloat(Option.Category, Option.Name, Option.AsDateTime);
end;

procedure TOpSaver.STBiFA;
begin
  STStFA;
end;

procedure TOpSaver.STBiFl;
begin
  STStFl;
end;

procedure TOpSaver.STBiFo;
var
  F: TFont;
begin
  F := TFont.Create;
  try
    ReadFont(Option.AsString, F);
    FMem.WriteString(Option.Category, Option.Name, WriteFont(F, false, Option.ArraySeparator));
  finally
    F.Free;
  end;
end;

procedure TOpSaver.STBiFs;
begin
  FMem.WriteString(Option.Category, Option.Name, Option.AsString);
end;

procedure TOpSaver.STBiIA;
begin
  STStIA;
end;

procedure TOpSaver.STBiIn;
begin
  STStIn;
end;

procedure TOpSaver.STBiSA;
begin
  STStSA;
end;

procedure TOpSaver.STBiSt;
begin
  STStSt;
end;

procedure TOpSaver.STBiTi;
begin
  FMem.WriteFloat(Option.Category, Option.Name, Option.AsTime);
end;

procedure TOpSaver.STSA;
begin
  case Option.OptionType of
    otString     : STSASt;
    otInteger    : STSAIn;
    otFloat      : STSAFl;
    otBoolean    : STSABo;
    otColor      : STSACo;
    otDate       : STSADa;
    otTime       : STSATi;
    otDateTime   : STSADT;
    otFont       : STSAFo;
    otFlags      : STSAFs;
    otStrArray   : STSASA;
    otIntArray   : STSAIA;
    otFloatArray : STSAFA;
  end;
end;

procedure TOpSaver.STSABo;
begin
  STStBo;
end;

procedure TOpSaver.STSACo;
begin
  STStCo;
end;

procedure TOpSaver.STSADa;
begin
  STStDa;
end;

procedure TOpSaver.STSADT;
begin
  STStDT;
end;

procedure TOpSaver.STSAFA;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FMem.WriteString(Option.Category, IName(i), Option.DefAsStrArray[i]);
end;

procedure TOpSaver.STSAFl;
begin
  STStFl;
end;

procedure TOpSaver.STSAFo;
var
  F: TFont;
begin
  F := Option.AsFont;
  try
    FMem.WriteInteger(Option.Category, SName(strCharset), F.Charset);
    FMem.WriteString(Option.Category, SName(strColor), ColorToString(F.Color));
    FMem.WriteString(Option.Category, SName(strName), F.Name);
    FMem.WriteString(Option.Category, SName(strPitch), Pitch2String(F.Pitch));
    FMem.WriteInteger(Option.Category, SName(strSize), F.Size);
    FMem.WriteString(Option.Category, SName(strStyle), Style2String(F.Style, true));
  finally
    F.Free;
  end;
end;

procedure TOpSaver.STSAFs;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FMem.WriteString(Option.Category, IName(i), BoolTxt(Option.BoolStr, Option.AsFlags[i]));
end;

procedure TOpSaver.STSAIA;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FMem.WriteInteger(Option.Category, IName(i), Option.AsIntArray[i]);
end;

procedure TOpSaver.STSAIn;
begin
  STStIn;
end;

procedure TOpSaver.STSASA;
var
  i: integer;
begin
  for i := 0 to Option.Count - 1 do
    FMem.WriteString(Option.Category, IName(i), Option.AsStrArray[i]);
end;

procedure TOpSaver.STSASt;
begin
  STStSt;
end;

procedure TOpSaver.STSATi;
begin
  STStTi;
end;

procedure TOpSaver.STSt;
begin
  case Option.OptionType of
    otString     : STStSt;
    otInteger    : STStIn;
    otFloat      : STStFl;
    otBoolean    : STStBo;
    otColor      : STStCo;
    otDate       : STStDa;
    otTime       : STStTi;
    otDateTime   : STStDT;
    otFont       : STStFo;
    otFlags      : STStFs;
    otStrArray   : STStSA;
    otIntArray   : STStIA;
    otFloatArray : STStFA;
  end;
end;

procedure TOpSaver.STStBo;
begin
  FMem.WriteString(Option.Category, Option.Name, BoolTxt(Option.BoolStr, Option.AsBoolean));
end;

procedure TOpSaver.STStCo;
begin
  FMem.WriteString(Option.Category, Option.Name, ColorToString(Option.AsColor));
end;

procedure TOpSaver.STStDa;
begin
  FMem.WriteString(Option.Category, Option.Name, DateToStr(Option.AsDate));
end;

procedure TOpSaver.STStDT;
begin
  FMem.WriteString(Option.Category, Option.Name, DateTimeToStr(Option.AsDateTime));
end;

procedure TOpSaver.STStFA;
begin
  if CheckFloat(Option.AsString) then
    FMem.WriteString(Option.Category, Option.Name, StringReplace(Option.AsString,
        strCRLF, Option.ArraySeparator, [rfReplaceAll]))
    else
      raise EOptError.Create(oerNotFloat);
end;

procedure TOpSaver.STStFl;
begin
  FMem.WriteString(Option.Category, Option.Name, FormatFloat(Option.FloatMask,
      Option.AsFloat));
end;

procedure TOpSaver.STStFo;
var
  F: TFont;
begin
  F := TFont.Create;
  try
    ReadFont(Option.AsString, F);
    FMem.WriteString(Option.Category, Option.Name, WriteFont(F, true, Option.ArraySeparator));
  finally
    F.Free;
  end;
end;

procedure TOpSaver.STStFs;
var
  i: integer;
  Temp: string;
begin
  Temp := '';
  for i := 0 to Option.Count - 1 do
    Temp := Temp + BoolTxt(Option.BoolStr, Option.AsFlags[i]) + Option.ArraySeparator;
  FMem.WriteString(Option.Category, Option.Name, Temp);
end;

procedure TOpSaver.STStIA;
begin
  if CheckInt(Option.AsString) then
    FMem.WriteString(Option.Category, Option.Name, StringReplace(Option.AsString,
        strCRLF, Option.ArraySeparator, [rfReplaceAll]))
  else
    raise EOptError.Create(oerNotInteger);
end;

procedure TOpSaver.STStIn;
begin
  FMem.WriteString(Option.Category, Option.Name, IntToStr(Option.AsInteger));
end;

procedure TOpSaver.STStSA;
begin
  FMem.WriteString(Option.Category, Option.Name, StringReplace(Option.AsString,
      strCRLF, Option.ArraySeparator, [rfReplaceAll]));
end;

procedure TOpSaver.STStSt;
begin
  FMem.WriteString(Option.Category, Option.Name, Option.AsString);
end;

procedure TOpSaver.STStTi;
begin
  FMem.WriteString(Option.Category, Option.Name, TimeToStr(Option.AsTime));
end;

function TOpSaver.WriteFont(F: TFont; SM: boolean; const Sep: string): string;
begin
  result := IntToStr(F.Charset) + Sep;
  if SM then
    result := result + ColorToString(F.Color) + Sep
  else
    result := result + IntToStr(F.Color) + Sep;
  result := result + F.Name + Sep;
  if SM then
    result := result + Pitch2String(F.Pitch) + Sep
  else
    result := result + IntToStr(integer(F.Pitch)) + Sep;
  result := result + IntToStr(F.Size) + Sep;
  result := result + Style2String(F.Style, SM) + Sep;
end;

{ TMemSaver }

function TMemSaver.AddSection(const Section: string): integer;
begin
  result := FindSection(Section);
  if result < 0 then begin
    if FLines.Count > 0 then
      FLines.Add('');
    FLines.Add('[' + Section + ']');
    result := FLines.Count - 1;
  end;
end;

constructor TMemSaver.Create(ALines: TStrings; ABoolStr: TBoolStr; const AFloatMask: string);
begin
  inherited Create;
  FLines := ALines;
end;

function TMemSaver.FindIdent(const Ident: string;
  SectionStart: integer): integer;
var
  temp: string;
begin
  temp := AnsiUpperCase(Ident);
  result := SectionStart + 1;
  while (result < FLines.Count) and (FLines[result] <> '') and (FLines[result][1] <> '[')
      and (Pos(temp, AnsiUpperCase(FLines[result])) <> 1) do
    Inc(result);
  if (result >= FLines.Count) or (FLines[result] = '') or (FLines[result][1] = '[') then
    result := -1;
end;

function TMemSaver.FindSection(const Section: string): integer;
var
  temp: string;
begin
  temp := AnsiUpperCase(Section);
  result := 0;
  while (result < FLines.Count) and
      (AnsiUpperCase(Copy(FLines[result], 2, Length(FLines[result]) - 2)) <> temp) do
    Inc(result);
  if result >= FLines.Count then
    result := -1;
end;

function TMemSaver.FindSectionEnd(const Section: string): integer;
begin
  result := FindSection(Section);
  if result >= 0 then begin
    Inc(result);
    while (result < FLines.Count) and (FLines[result] <> '') do
      Inc(result);
  end;
end;

function TMemSaver.ReadBool(const Section, Ident: string;
  Default: boolean): boolean;
begin
  result := GetBoolean(ReadString(Section, Ident, BoolTxt(bsNum, Default)));
end;

function TMemSaver.ReadFloat(const Section, Ident: string;
  Default: double): double;
begin
  result := GetFloat(ReadString(Section, Ident, FloatToStr(Default)));
end;

function TMemSaver.ReadInteger(const Section, Ident: string;
  Default: integer): integer;
begin
  result := GetInteger(ReadString(Section, Ident, IntToStr(Default)));
end;

procedure TMemSaver.ReadSection(const Section: string; Strings: TStrings);
var
  s, p: integer;
begin
  Strings.Clear;
  s := FindSection(Section);
  if s >= 0 then begin
    Inc(s);
    while (s < FLines.Count) and (FLines[s] <> '') and (FLines[s][1] <> '[') do begin
      p := Pos('=', FLines[s]);
      if p <> 0 then
        Strings.Add(Copy(FLines[s], 1, p - 1));
      Inc(s);
    end;
  end;
end;

function TMemSaver.ReadString(const Section, Ident,
  Default: string): string;
var
  s, p: integer;
begin
  result := Default;
  s := FindSection(Section);
  if s >= 0 then begin
    s := FindIdent(Ident, s);
    if s >= 0 then begin
      p := Pos('=', FLines[s]);
      result := Copy(FLines[s], p + 1, Length(FLines[s]) - p);
    end;
  end;
end;

procedure TMemSaver.WriteBool(const Section, Ident: string;
  Value: boolean);
begin
  WriteString(Section, Ident, BoolTxt(bsNum, Value));
end;

procedure TMemSaver.WriteFloat(const Section, Ident: string;
  Value: double);
begin
  WriteString(Section, Ident, FloatToStr(Value));
end;

procedure TMemSaver.WriteInteger(const Section, Ident: string;
  Value: integer);
begin
  WriteString(Section, Ident, IntToStr(Value));
end;

procedure TMemSaver.WriteString(const Section, Ident, Value: string);
var
  s, n: integer;
  temp: string;
begin
  temp := Ident + '=' + Value;
  s := AddSection(Section);
  n := FindIdent(Ident, s);
  if n < 0 then begin
    n := FindSectionEnd(Section);
    FLines.Insert(n, temp);
  end else
    FLines[n] := temp;
end;

end.
