{
 -----------------------------------------------------------------------------

 MODUL          mdCONTROLSREG.pas

 COPYRIGHT      micro dynamics GmbH
 AUTHOR(S)      Michael Frank
 UPDATE         30.06.2002

 COMPILER       Delphi 5/6
 PLATFORM       Windows 95/98/Me/NT4/2000/XP

 DESCRIPTION    Registration and design-editor for mdCONTROLS.pas

 -----------------------------------------------------------------------------
 Revision History:   Original Version  06.02

 -----------------------------------------------------------------------------
}
unit mdCONTROLSREG;

//============================================================================

interface

//============================================================================

uses
  Windows, Classes, SysUtils, Registry,
{$IFDEF VER140}
  DesignIntf, DesignEditors,
{$ELSE}
  DsgnIntf,
{$ENDIF}
  mdCONTROLS;

type
{ TXPPageControlEditor }

  TXPPageControlEditor = class(TDefaultEditor)
  public
    procedure ExecuteVerb (Index : integer); override;
    function GetVerb (Index : integer): string; override;
    function GetVerbCount : integer; override;
  end;

procedure Register;

//============================================================================

implementation

//============================================================================

//----------------------------------------------------------------------------
procedure Register;
//----------------------------------------------------------------------------

begin
  RegisterComponents('XPControls', [TXPThemeAPI, TXPPageControl, TXPGroupBox,
    TXPCheckListBox, TXPListView, TXPCheckBox, TXPRadioButton, TXPTrackBar,
    TXPAnimate, TXPStatusBar, TXPToolBar]);

  RegisterNoIcon([TXPTabSheet]);
  RegisterClasses([TXPTabSheet]);
  RegisterComponentEditor(TXPPageControl, TXPPageControlEditor);
  RegisterComponentEditor(TXPTabSheet, TXPPageControlEditor);
end;

//============================================================================
// class TXPPageControlEditor
//============================================================================

resourcestring
  sDEF_TABSHEET = 'XPTabSheet';

  sNEW_PAGE_GER = 'Neue Seite';
  sDEL_PAGE_GER = 'Seite lschen';
  sNEXT_PAGE_GER = 'Nchste Seite';
  sPREV_PAGE_GER = 'Vorige Seite';

  sNEW_PAGE_ENG = 'New Page';
  sDEL_PAGE_ENG = 'Delete Page';
  sNEXT_PAGE_ENG = 'Next Page';
  sPREV_PAGE_ENG = 'Previous Page';

//----------------------------------------------------------------------------
procedure TXPPageControlEditor.ExecuteVerb (Index : integer);
//----------------------------------------------------------------------------

var
  NewPage : TXPTabSheet;
  PControl : TXPPageControl;

begin
  if Component is TXPPageControl then
    PControl:= TXPPageControl(Component)
  else
    PControl:= TXPPageControl(TXPTabSheet(Component).PageControl);

  case Index of
    0 : begin  // New Page
	  NewPage:= TXPTabSheet.Create(Designer.GetRoot);
	  with NewPage do
	  begin
	    Parent:= PControl;
	    PageControl:= PControl;
	    Caption:= sDEF_TABSHEET + IntToStr(PControl.PageCount);
	    Name:= Caption;
	  end;
	end;
    1 : begin  // Delete Page
	  with PControl do
	  begin
	    NewPage:= TXPTabSheet(ActivePage);
	    NewPage.PageControl:= nil;
	    NewPage.Free;
	  end;
	end;
    2 : begin  // Next Page
	  PControl.FindNextPage(PControl.ActivePage, true, false);
	end;
    3 : begin  // Previous Page
	  PControl.FindNextPage(PControl.ActivePage, false, false);
	end;
  end;
  if (Designer <> nil) then
    Designer.Modified;
end;

//----------------------------------------------------------------------------
function TXPPageControlEditor.GetVerb (Index : integer) : string;
//----------------------------------------------------------------------------

const
  sLocales = 'Software\Borland\Locales';
  sDelphi5 = 'Borland\Delphi5\Bin\Delphi32.exe';
  sDelphi6 = 'Borland\Delphi6\Bin\Delphi32.exe';

var
  i, j : integer;
  Lines : TStringList;
  s : string;
  found : boolean;

begin
  with TRegistry.Create do
  begin
    Lines:= TStringList.Create;

    RootKey:= HKEY_CURRENT_USER;
    OpenKey(sLocales,false);
    GetValueNames(Lines);

    i:= 0;
    found:= false;
    while (i < Lines.Count) and not found do
    begin
{$IFDEF VER140}
      j:= Length(Lines[i])-Length(sDelphi6);
{$ELSE}
      j:= Length(Lines[i])-Length(sDelphi5);
{$ENDIF}
      if (j > 0) then
      begin
        s:= Lines[i];
        Delete(s,1,j);
{$IFDEF VER140}
        if (s = sDelphi6) then
{$ELSE}
        if (s = sDelphi5) then
{$ENDIF}
        begin
          if (ReadString(Lines[i]) = 'DE') then
            found:= true;
        end;
      end;
      inc(i,1);
    end;
    Lines.Free;
    Free;
  end;

  if found then
  begin
    case Index of
      0 : Result:= sNEW_PAGE_GER;
      1 : Result:= sDEL_PAGE_GER;
      2 : Result:= sNEXT_PAGE_GER;
      3 : Result:= sPREV_PAGE_GER;
    end;
  end else
  begin
    case Index of
      0 : Result:= sNEW_PAGE_ENG;
      1 : Result:= sDEL_PAGE_ENG;
      2 : Result:= sNEXT_PAGE_ENG;
      3 : Result:= sPREV_PAGE_ENG;
    end;
  end;
end;

//----------------------------------------------------------------------------
function TXPPageControlEditor.GetVerbCount : integer;
//----------------------------------------------------------------------------

begin
  Result:= 4;
end;

//============================================================================

end.
